----------------------------------------------------------------------------------------
--FAR CRY script file
-- ASSAULT Game Rules
-- Created by Alberto Demichelis
-- modified by Martin Mittring
-- modified by Kevin Jenkins (changing teams and scoring)
--
-- team red is the attacker team
-- team blue is the defender team
--
----------------------------------------------------------------------------------------
Script:LoadScript("SCRIPTS/MULTIPLAYER/VotingState.lua");


GameRules={
	MapCheckPoints ={},								-- table of stored checkpoint entities

	respawndelay = 3,
	countnext = 0,
	

	mapstart = 0,
	intermissionstart = 0,
	scoreupdate = 3000,
		
	CurrentCheckPoint_Number = nil,		-- from the touched checkpoint (Properties.CheckPoint_Number) - is not the index in MapCheckPoints
	CheckPointIndices={},								-- refresh with self:RecalcCheckPointIndices()
	
	attackerTeam="red",
	defenderTeam="blue",

	has_vip=0,
	heropoint=0,
	newhero=0,
	goodhero=0,

	numgolda=0,
	numgoldb=0,

	red_played_attack=0,
	red_played_defense=0,
	blue_played_attack=0,
	blue_played_defense=0,
	
	first_attackers_time=0,
	timelimit=getglobal("gr_TimeLimit"),
	

	-- These two timers are handled by StartGameRulesLibTimer and DoGameRulesLibTimer
	respawnCycle=1, 									-- Set this 1 to enable the respawn cycle feature for this mod
	switchTeamsTimer=5, 							-- Seconds before switching teams after a win
	states={},												-- table of tall the ASSAULTCheckpoints
	CurrentProgressStep=0,						-- 0=not capturable(warmup), 1=touchable, [1.. GetCaptureStepCount()]
	
	ScoreMultipliers=
	{
		xCaptureStarted = 50,
		xCaptureAverted = 20,
		xCaptureFinished = 50,
		xBuildingBuilt = 20,
		xBuildingRepaired = 20,
		xBuildingDestroyed = 20,
		xHeal = 20,
		xKill = 20,
		xDeath = -1,
	},
}

Script:LoadScript("SCRIPTS/MULTIPLAYER/GameRulesClassLib.lua");	-- derive from class bases game rules
Script:LoadScript("scripts/ASSAULT/shared.lua");

GameRules.InitialPlayerStatistics["nCaptureFinished"]=0;			-- this entry can be used by MPStatistics:AddStatisticsDataEntity();
GameRules.InitialPlayerStatistics["nCaptureStarted"]=0;
GameRules.InitialPlayerStatistics["nCaptureAverted"]=0;

-- server_slot.
--	InitialPlayerProperties =
--	{
--		health		= 255, 			-- range: 0=dead .. 255=full health		health		= 100,
--		armor		= 100,
--		model		= "",			-- if this is set it is used as your player model e.g. "Objects/characters/workers/evil_worker/evil_worker.cgf"
--		equipEquipment	="NetPlayerPack"..playerid,
--	},

-------------------------------------------------------------------------------
--UTILITIES FUNC
-------------------------------------------------------------------------------

----------------------------------------------------------------------------------------
function GameRules:ScoreboardUpdate()
	local SlotMap=Server:GetServerSlotMap();
	
	for i, Slot in SlotMap do
		local Player = GetSlotPlayer(Slot);
		
		if Player and Player.cnt then
			local ClientId = Slot:GetId();

			local iDeaths = Player.cnt.deaths;			
			if not iDeaths then
				iDeaths=0;
			end

			local iKing = 0;			
			if (Player.cnt.vip) then
				iKing = Player.cnt.vip;
			end
			
			local iTotalScore = 0;
			local iPlayerScore = 0;
			local iCapScore = 0;
			local iSupportScore = 0;
			local iKillScore = 0;

			iTotalScore, iPlayerScore, iCapScore, iSupportScore, iKillScore = GameRules:CalcPlayerScore(Slot);

			local szPlayerClass = Player.sCurrentPlayerClass;		
			local iPlayerClass = -1;
		
			if (szPlayerClass) then
				if (Player.sCurrentPlayerClass == "Grunt") then
					iPlayerClass = 0;
				elseif (Player.sCurrentPlayerClass == "Sniper") then
					iPlayerClass = 1;
				elseif (Player.sCurrentPlayerClass == "Heavy") then
					iPlayerClass = 2;
				elseif (Player.sCurrentPlayerClass == "Long") then
					iPlayerClass = 3;
				elseif (Player.sCurrentPlayerClass == "Support") then
					iPlayerClass = 4;
				elseif (Player.sCurrentPlayerClass == "MutMortar") then
					iPlayerClass = 5;
				elseif (Player.sCurrentPlayerClass == "MutantMan") then
					iPlayerClass = 6;
				elseif (Player.sCurrentPlayerClass == "Light") then
					iPlayerClass = 7;
				end
			end
			
--			System:Log("szPlayerClass "..tostring(szPlayerClass).." "..tostring(iPlayerClass));

			local iPlayerTeam = -1;

			local szTeam=Game:GetEntityTeam(Player.id);

			if szTeam=="spectators" then
				iPlayerTeam = 0;
			elseif szTeam=="red" then
				iPlayerTeam = 1;
			elseif szTeam=="blue" then
				iPlayerTeam = 2 
			end
			
--			System:Log("PlayerTeam "..tostring(szTeam).." "..tostring(iPlayerTeam)); -- debug

			local iSuicides = 0;
			if Slot.Statistics and Slot.Statistics["nSelfKills"] then
				iSuicides = Slot.Statistics["nSelfKills"];
			end
			
			self:SetScoreboardEntryXY(ScoreboardTableColumns.sName,ClientId,Slot:GetName());
			self:SetScoreboardEntryXY(ScoreboardTableColumns.iTotalScore,ClientId,iTotalScore);
			self:SetScoreboardEntryXY(ScoreboardTableColumns.iPlayerScore,ClientId,iPlayerScore);
--			self:SetScoreboardEntryXY(ScoreboardTableColumns.iDeaths,ClientId,iDeaths);
			self:SetScoreboardEntryXY(ScoreboardTableColumns.iCapScore,ClientId,iCapScore);
--			self:SetScoreboardEntryXY(ScoreboardTableColumns.iSupportScore,ClientId,iSupportScore);
--			self:SetScoreboardEntryXY(ScoreboardTableColumns.iSuicides,ClientId,iSuicides);
--			self:SetScoreboardEntryXY(ScoreboardTableColumns.iKillScore,ClientId,iKillScore);
			self:SetScoreboardEntryXY(ScoreboardTableColumns.iPlayerClass,ClientId,iPlayerClass);
			self:SetScoreboardEntryXY(ScoreboardTableColumns.iPlayerTeam,ClientId,iPlayerTeam);
			self:SetScoreboardEntryXY(ScoreboardTableColumns.iKing,ClientId,iKing);
		end
	end
end


-------------------------------------------------------------------------------
function GameRules:OnInit()
	System:Log("$5GameRules Init: "..self:ModeDesc());
	Server:RemoveTeam("red");
	Server:RemoveTeam("blue");
	e_deformable_terrain=0;
	Server:AddTeam("red");
	Server:AddTeam("blue");
	CreateStateMachine(self);
	self.voting_state=VotingState:new();
end

-----------------------------------------------------------------------
-- For Server Browsers
-- Refer to GetPlayerStats in QueryHandler.lua for more info
-----------------------------------------------------------------------
function GameRules:GetPlayerStats()
	local SlotMap = Server:GetServerSlotMap();
	local Stats = {};
	local j = 1; -- to make it 1..n because SlotMap is 0..n-1
	
	for i, Slot in SlotMap do
		Stats[j] = {};	
		
		local Player = Stats[j];
		local iScore = GameRules:CalcPlayerScore(Slot);
		
		Player.Name = Slot:GetName();
		Player.Team = Game:GetEntityTeam(Slot:GetPlayerId());
		Player.Skin = "";
		Player.Score = iScore;
--		Player.Vip = iVip;
		Player.Ping = Slot:GetPing();
		Player.Time = floor(Slot:GetPlayTime() / 60).."m";

		j = j + 1;
	end

	return Stats;
end


function GameRules:CalcPlayerScore(ServerSlot)
	if (not ServerSlot.Statistics) then
		ServerSlot.Statistics = GameRules:GetInitialPlayerStatistics();
	end
	
	local Stat = ServerSlot.Statistics;
	local Mult = GameRules.ScoreMultipliers;
	local Player = System:GetEntity(ServerSlot:GetPlayerId());

	-- calculate total score
	local iScore = 0;
	local iCapScore = 0;
	local iSupportScore = 0;
	local iKillScore = 0;
	local iPlayerScore = 0;
	local iPlayerDeaths = 0;
	local iKing = 0;
	
	if Player then
		if Player.cnt then
			if (Player.cnt.score) then
				iPlayerScore = Player.cnt.score;
				end
			if (Player.cnt.deaths) then
				iPlayerDeaths = Player.cnt.deaths;
			end
--			if (Player.cnt.vip) then
--				iKing = Player.cnt.vip;
--			end
		end
	end
	
	if(Stat["nCaptureStarted"])then
		iCapScore = iCapScore + Stat["nCaptureStarted"] * Mult["xCaptureStarted"];
		iCapScore = iCapScore + Stat["nCaptureAverted"] * Mult["xCaptureAverted"];
		iCapScore = iCapScore + Stat["nCaptureFinished"] * Mult["xCaptureFinished"];
--		iCapScore = iCapScore + iKing * 500;
		iSupportScore = iSupportScore + Stat["nBuildingBuilt"] * Mult["xBuildingBuilt"];
		iSupportScore = iSupportScore + Stat["nBuildingRepaired"] * Mult["xBuildingRepaired"];
		iSupportScore = iSupportScore + Stat["nBuildingDestroyed"] * Mult["xBuildingDestroyed"];
		iSupportScore = iSupportScore + Stat["nHealed"] * Mult["xHeal"];
		iKillScore = iKillScore + iPlayerScore * Mult["xKill"];
		iKillScore = iKillScore - iPlayerDeaths * Mult["xDeath"];
		iScore = iKillScore + iSupportScore + iCapScore;
	end
	
	return iScore, iPlayerScore, iCapScore, iSupportScore, iKillScore;
end




-------------------------------------------------------------------------------
function GetPlayerFromSSId(ssid)
	local Slot = Server:GetServerSlotBySSId(ssid);
	return System:GetEntity(Slot:GetPlayerId());
end

-------------------------------------------------------------------------------
function GameRules:InvalidateCapture(ssid)
	
	local currentcp = GameRules:GetNextBiggerCheckPointNumber();
	
	for i,cp in self.MapCheckPoints do
		if ((cp.Properties.bVisible == 1) and (cp.Properties.CheckPoint_Number == currentcp)) then
			if (cp.captureCollider and (cp.captureCollider == ssid)) then
				cp:Event_Averted();
			end
		end
	end
end

-------------------------------------------------------------------------------
function GameRules:OnAfterLoad()
end

-------------------------------------------------------------------------------
-- private method
function GameRules:_GetCaptureString()
	local next = self:CalcCheckPointIndex(self.CurrentCheckPoint_Number);
	if (tonumber(getglobal("gr_tMode")) == 2) then
		return "P2 "..next.." "..self.numgolda.." "..self.numgoldb;
	end
	return "P2 ".. next .." ".. self.CurrentProgressStep .." ".. count(self.CheckPointIndices)-1;		-- P2 0 [0..GetCaptureStepCount()] GoalCount
end

-------------------------------------------------------------------------------
function GameRules:UpdateCaptureProgressAllPlayers()
  Server:BroadcastCommand(self:_GetCaptureString());						-- invoke OnServerCmd() for all the clients
end
-------------------------------------------------------------------------------
function GameRules:UpdateCaptureProgressOnePlayer( server_slot )
  server_slot:SendCommand(self:_GetCaptureString());						-- invoke OnServerCmd() on the given client
end


-------------------------------------------------------------------------------
-- called on capturing a checkpoint
function GameRules:OnCaptureStep_Warmup()
	self.CurrentProgressStep=0;
	self:UpdateCaptureProgressAllPlayers();
end

-------------------------------------------------------------------------------
-- called on avert capture and on warmup end time
-- \param checkpointno number of the touched checkpoint
function GameRules:OnCaptureStep_Touchable( checkpointno )
	local next=GameRules:GetNextBiggerCheckPointNumber();

	if next~=nil and next==checkpointno then
		self.CurrentProgressStep=1;
		self:UpdateCaptureProgressAllPlayers();
	end
end

-------------------------------------------------------------------------------
-- \return 1,nil
function GameRules:OnCaptureStep_Next()
	self.CurrentProgressStep=self.CurrentProgressStep+1;

	if self.CurrentProgressStep>self:GetCaptureStepCount()+1 then
		self.CurrentProgressStep=0;
		return 1;
	end

	self:UpdateCaptureProgressAllPlayers();
	return nil;
end

-------------------------------------------------------------------------------
function GameRules:GetCaptureStepCount()
	return 4;
end


-------------------------------------------------------------------------------
-- This function is called whenever the server needs to return a packet containing
-- a description of the current Game: since a "mod" can contain multiple modes, this
-- should be taken into account here. the string should be relatively short.
function GameRules:ModeDesc()
    return "Assault";
end


-------------------------------------------------------------------------------
-- This function resets the game...
function GameRules:DoRestart()

	self.restartbegin = nil;
	self.restartend = nil;
	self.switchTeamsCountdown = nil;
	
	self.attackerTeam="red";
	self.defenderTeam="blue";
	
	self.red_played_attack=0;
	self.red_played_defense=0;
	

	self.numgolda=0;
	self.numgoldb=0;

	self.blue_played_attack=0;
	self.blue_played_defense=0;

	--3up fort score flag start
	if (tonumber(getglobal("gr_tMode")) == 2) then
		Server:SetTeamScore("blue", tonumber(gr_FortScoreStart));
 		Server:SetTeamScore("red", tonumber(gr_FortScoreStart));
	else
		Server:SetTeamScore("blue", 0);
	 	Server:SetTeamScore("red", 0);
	end
	--3up fort score flag end


	if (tonumber(getglobal("gr_tMode")) == 3) then
 		self:ResetPlayerScores(0, 0, 0, 2);
	else
 		self:ResetPlayerScores(0, 0, 0, 0);	
	end


 	self:ResetMap();
 	self:UpdateTimeLimit(getglobal("gr_TimeLimit"));
end

-------------------------------------------------------------------------------
-- called from the map reloading process
-- gamerules.lua is reloaded and thus reset before this is called
function GameRules:OnMapChange()

	self:DoRestart();

	if tostring(getglobal("gr_PrewarOn"))~="0" then
		self:NewGameState(CGS_PREWAR);
	else
		self:NewGameState(CGS_INPROGRESS);
	end	

	if (MapCycle) then
		MapCycle:OnMapChanged();
	end
	
	GameRules:CreateScorebordEntity();
end

-------------------------------------------------------------------------------
--
function GameRules:SetToFirstCheckPoint()
--	System:Log("GameRules:SetToFirstCheckPoint");
	self.CurrentCheckPoint_Number=nil;
	
	-- find the lowest number in entity.Properties.CheckPoint_Number and store it in CurrentCheckPoint_Number
	for i,entity in self.MapCheckPoints do
		if self.CurrentCheckPoint_Number==nil or entity.Properties.CheckPoint_Number<self.CurrentCheckPoint_Number then
			self.CurrentCheckPoint_Number = entity.Properties.CheckPoint_Number;
		end
	end
	

--3up fort start
	if tonumber(getglobal("gr_tMode")) == 2 then
		self.CurrentCheckPoint_Number = self:GetNextBiggerCheckPointNumber();
		if tonumber(getglobal("gr_currFlag")) == 3 then
			self.CurrentCheckPoint_Number = self:GetNextBiggerCheckPointNumber();
		end
		Server:BroadcastText("Enjoy the game of $4Tacto$0!");

	end

	if tonumber(getglobal("gr_tMode")) == 3 then
		if tonumber(getglobal("gr_currFlag")) == 3 then
			self.CurrentCheckPoint_Number = self:GetNextBiggerCheckPointNumber();
			self.CurrentCheckPoint_Number = self:GetNextBiggerCheckPointNumber();
		end

		if tonumber(getglobal("gr_currFlag")) == 2 then
			self.CurrentCheckPoint_Number = self:GetNextBiggerCheckPointNumber();
		end
		Server:BroadcastText("Enjoy the game of $4Tacto$0!");
	end
--3up fort end


	self:RefreshCheckpoints();
	self:OnCaptureStep_Warmup();
end





-------------------------------------------------------------------------------
-- send the player 
function GameRules:ASSAULTHandleJoinTeamRequest( server_slot,team )
	self:InvalidateCapture(server_slot:GetId());
	self:HandleJoinTeamRequest(server_slot,team);
	self:UpdateCaptureProgressOnePlayer(server_slot);
end


-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
function GameRules:INPROGRESS_OnDamage(hit)
	local damage_ret=self:UsualDamageCalculation(hit);
	
--	System:Log("GameRules:INPROGRESS_OnDamage "..tostring(damage_ret));

	-- we have to unregister dead players from the assault checkpoints
	if (damage_ret ~= nil) then		
		local deadguy_pos = hit.target:GetPos();
		deadguy_pos.z = deadguy_pos.z + 1.5;
		
--3up fort add gold
		local ss_target=Server:GetServerSlotByEntityId(hit.target.id);

		if ((hit.target.cnt.vip == 2) and (tonumber(getglobal("gr_tMode")) == 2) ) then
			local gold = Server:SpawnEntity("FortGold", deadguy_pos);

			if (gold) then
				gold:Launch(hit.target, deadguy_pos);
				gold.Properties.IsDropPack = 1;
--				gold:Physicalize(hit.target);
			end
			local ss_hero=Server:GetServerSlotByEntityId(hit.target.id);
			if ss_hero then
				local hero = System:GetEntity(ss_hero:GetPlayerId());
				Server:BroadcastText(hero:GetName().." dropped the gold!");
--				System:LogAlways(hero:GetName().." dropped the gold!");
				
			end
			Server:BroadcastCommand("SNQ C");

		
			

			if GameRules:IsAttacker(hit.target) then
				self.numgolda = self.numgolda - 1;
				local capstr = "P2 0 "..self.numgolda.." "..self.numgoldb;
				--if self.numgolda > 0 then	
					Server:BroadcastCommand(capstr);		
				--end
			else
				self.numgoldb = self.numgoldb - 1;
				local capstr = "P2 0 "..self.numgolda.." "..self.numgoldb;
				--if self.numgoldb > 0 then
					Server:BroadcastCommand(capstr);		
				--end
			end
--			Server:BroadcastText(self.numgolda.." "..self.numgoldb);
--			System:LogAlways(self.numgolda.." "..self.numgoldb);
		else
			local pickup = Server:SpawnEntity("ClassAmmoPickup", deadguy_pos);
			if (pickup) then
				pickup:SetFadeTime(GameRules:GetPickupFadeTime());
				pickup.autodelete = 1;
				pickup.Properties.IsDropPack = 1;
				pickup:EnableSave(0);
				pickup:Physicalize(hit.target);
			end
		end
--3up fort add gold end
	end
	
	self:UsualScoreCalculation(hit,damage_ret);
end


function GameRules:HandleClientDisconnect(server_slot)
	self:InvalidateCapture(server_slot:GetId());
	
	if (self.respawnList) then
		self.respawnList[server_slot]=nil; -- Remove this guy from the pending respawn list
	end
--self:CheckPlayersCount();		-- goto prewar if player count is not ok
end

-------------------------------------------------
--PREWAR
-------------------------------------------------
GameRules.states.PREWAR={
--------------------------------
	OnBeginState=function (self)
		GameRules:SetToFirstCheckPoint();

--		if(tonumber(gr_AutoReady)~=0)then
--			self.autoready=_time;
--			self.autoreadynotif=tonumber(gr_AutoReadyTime)*60;
--			Server:BroadcastText("@TheGameStartsIn $3"..(tonumber(gr_AutoReadyTime)*60));	-- $1the game will start automatically in
--		else
--			Server:BroadcastText("@WaitingForReadyPlayer");		-- $1waiting for players ready
--		end
		self:ResetReadyState(nil);

	end,
	--------------------------------
	OnUpdate=GameRules.PREWAR_Team_OnUpdate,
	--------------------------------
	OnClientJoinTeamRequest=GameRules.ASSAULTHandleJoinTeamRequest,
	--------------------------------
	OnEndState=function(self)

--3up fort add
	if (tonumber(getglobal("gr_tMode")) == 2) then
		Server:SetTeamScore("blue", tonumber(gr_FortScoreStart));
 		Server:SetTeamScore("red", tonumber(gr_FortScoreStart));
	else
		Server:SetTeamScore("blue",0)
 		Server:SetTeamScore("red",0)
	end
--3up fort add emd
	end,
}
-------------------------------------------------
--COUNTDOWN
-------------------------------------------------
GameRules.states.COUNTDOWN={
--------------------------------
	OnBeginState=GameRules.COUNTDOWN_OnBeginState,
--------------------------------
	OnUpdate=GameRules.COUNTDOWN_OnUpdate,
--------------------------------
	OnClientJoinTeamRequest=GameRules.ASSAULTHandleJoinTeamRequest,
--------------------------------
	OnClientDisconnect = GameRules.HandleClientDisconnect,
}
-------------------------------------------------
--INPROGRESS
-------------------------------------------------
GameRules.states.INPROGRESS={
--------------------------------
	OnBeginState=function (self)
		GameRules:SetToFirstCheckPoint();
		GameRules:StartGameRulesLibTimer();
		GameRules:SetTeamObjectivesAllPlayers();		-- tell all players who is attacking and who defending
	end,
--------------------------------
	OnUpdate=function(self)
		
		if (MapCycle:IsShowingScores()) then
			return;
		end
				
		if(tonumber(gr_ScoreLimit)~=0)then
			local red_score=Game:GetTeamScore("red");
			local blue_score=Game:GetTeamScore("blue");		
			if(red_score>=tonumber(gr_ScoreLimit))then
				Server:BroadcastText("@RedTeamReachedScore");
	    	MapCycle:OnMapFinished();
			end
			if(blue_score>=tonumber(gr_ScoreLimit))then
		    Server:BroadcastText("@BlueTeamReachedScore");
	    	MapCycle:OnMapFinished();
			end
		end
	
--3up fort start
		if (self.newhero ~= 0) and (tonumber(getglobal("gr_tMode")) == 2) then
			local heroadj = self.newhero - 1;
			local ss_hero=Server:GetServerSlotByEntityId(heroadj);
			if ss_hero then
				local hero = System:GetEntity(ss_hero:GetPlayerId());
				MPStatistics:AddStatisticsDataEntity(hero,"nCaptureStarted",1);
				Server:BroadcastText(hero:GetName().." has the gold!");
				ss_hero:SendCommand("SNQ G");
				ss_hero:SendCommand("FLG Y");
				GameRules:AdjustSpeed(ss_hero);



				if GameRules:IsAttacker(hero) then
--			Server:BroadcastText(self.numgolda.." x a "..self.numgoldb);
--			System:LogAlways(self.numgolda.." x a "..self.numgoldb);
					self.numgolda = self.numgolda + 1;
					local capstr = "P2 0 "..self.numgolda.." "..self.numgoldb;	
					Server:BroadcastCommand(capstr);		
				else
--			Server:BroadcastText(self.numgolda.." x d "..self.numgoldb);
--			System:LogAlways(self.numgolda.." x d "..self.numgoldb);
					self.numgoldb = self.numgoldb + 1;
					local capstr = "P2 0 "..self.numgolda.." "..self.numgoldb;
					Server:BroadcastCommand(capstr);		
				end


				Server:BroadcastCommand("FX", g_Vectors.v000, g_Vectors.v000, hero.id, 5);
				Server:BroadcastCommand("DRAW Y", g_Vectors.v000, g_Vectors.v000, hero.id, 5);

				local entityid = hero.idKingHighlight;
				Server:BroadcastCommand("FXO 1", g_Vectors.v000, g_Vectors.v000, entityid, 5);

				
--				local highlightpos=new(hero:GetPos());
--				kinghighlight = Server:SpawnEntity("KingHighlight",highlightpos);


				Server:BroadcastCommand("SNQ B");

--				local thingpos = new(hero:GetPos());
--				local thing = Server:SpawnEntity("UnitHighlight",thingpos);
--		
--				hero.idGoldHighlight = thing.id;		-- only done on the server
--		
--				Server:BroadcastCommand("FX "..tonumber(hero.id), g_Vectors.v000, g_Vectors.v000,thing.id,0);

			end
			self.newhero = 0;
		end 


		if (self.heropoint ~= 0) and (tonumber(getglobal("gr_tMode")) == 2) then
			if (self.heropoint == 1) then
				local ss_hero=Server:GetServerSlotByEntityId(self.goodhero);
				if ss_hero then
					local hero = System:GetEntity(ss_hero:GetPlayerId());
					local team_score=Game:GetTeamScore(self.defenderTeam);
					team_score=team_score+1;
					MPStatistics:AddStatisticsDataEntity(hero,"nCaptureFinished",1);
					Server:BroadcastText(hero:GetName().." returns the gold for the "..self.defenderTeam.." team!");
					Server:BroadcastCommand("SNQ A");
					ss_hero:SendCommand("FLG ND");



					self.numgoldb = self.numgoldb - 1;
					local capstr = "P2 1 "..self.numgolda.." "..self.numgoldb;
					--if self.numgoldb > 0 then
						Server:BroadcastCommand(capstr);		
					--end

--			Server:BroadcastText(self.numgolda.." q "..self.numgoldb);
--			System:LogAlways(self.numgolda.." q "..self.numgoldb);

					Server:SetTeamScore(self.defenderTeam,team_score);
					GameRules:FortCheck();
					Server:BroadcastCommand("FX", g_Vectors.v000, g_Vectors.v000, hero.id, 1);
					Server:BroadcastCommand("DRAW N", g_Vectors.v000, g_Vectors.v000, hero.id, 5);

					local entityid = hero.idKingHighlight;
					Server:BroadcastCommand("FXO 0", g_Vectors.v000, g_Vectors.v000, entityid, 5);
					


--					if hero.idGoldHighlight then
--						Server:RemoveEntity(hero.idGoldHighlight);
--					end
				end
				self.heropoint = 0;
			end
			if (self.heropoint == 2) then
				local ss_hero=Server:GetServerSlotByEntityId(self.goodhero);
				if ss_hero then
					local hero = System:GetEntity(ss_hero:GetPlayerId());
					local team_score=Game:GetTeamScore(self.attackerTeam);
					team_score=team_score+1;
					MPStatistics:AddStatisticsDataEntity(hero,"nCaptureFinished",1);
					Server:BroadcastText(hero:GetName().." returns the gold for the "..self.attackerTeam.." team!");
					Server:BroadcastCommand("SNQ A");
					ss_hero:SendCommand("FLG NA");

					self.numgolda = self.numgolda - 1;
					local capstr = "P2 0 "..self.numgolda.." "..self.numgoldb;
					--if self.numgolda > 0 then	
						Server:BroadcastCommand(capstr);		
					--end

--			Server:BroadcastText(self.numgolda.." w "..self.numgoldb);
--			System:LogAlways(self.numgolda.." w "..self.numgoldb);

					Server:SetTeamScore(self.attackerTeam,team_score);
					GameRules:FortCheck();
					Server:BroadcastCommand("FX", g_Vectors.v000, g_Vectors.v000, hero.id, 1);
					Server:BroadcastCommand("DRAW N", g_Vectors.v000, g_Vectors.v000, hero.id, 5);

					local entityid = hero.idKingHighlight;
					Server:BroadcastCommand("FXO 0", g_Vectors.v000, g_Vectors.v000, entityid, 5);


--					if hero.idGoldHighlight then
--						Server:RemoveEntity(hero.idGoldHighlight);
--					end
				end
				self.heropoint = 0;
			end
		end 
--3up fort end




		if (self.timelimit and (tonumber(self.timelimit) > 0)) then
			if _time>self.mapstart+tonumber(self.timelimit)*60 then
				-- Defenders win
--3up fort start
				if (tonumber(getglobal("gr_tMode")) ~= 2) then
					local team_score=Game:GetTeamScore(self.defenderTeam);
					team_score=team_score+1;
					Server:SetTeamScore(self.defenderTeam,team_score);
					Server:BroadcastText("@TheAttackersFail");
				end
--3up fort end
				self:SwitchTeams();
				self.mapstart = _time;
		  	end
		end
	end,
--------------------------------
	OnDamage=GameRules.INPROGRESS_OnDamage,
--------------------------------
	OnKill=function(self,server_slot)
		local id = server_slot:GetPlayerId();
	    if id ~= 0 then
	        local ent = System:GetEntity(id);
	 		local team = Game:GetEntityTeam(id);
	        if team ~= "spectators" then
	            self:OnDamage({ target = ent, shooter = ent, damage = 1000, ipart = 0 });
	        end
	    end
	end,
--------------------------------
	OnClientJoinTeamRequest=GameRules.ASSAULTHandleJoinTeamRequest,
	OnTimer=function(self)
		if (self.switchTeamsCountdown~=nil and self.switchTeamsCountdown>0) then
			self.switchTeamsCountdown=self.switchTeamsCountdown-1;
			if (self.switchTeamsCountdown==0) then
				self:ForceScoreBoard(0);
				self:DoSwitchTeams();
			else
				Server:BroadcastText("@GameOverSwitchTeamsIn "..self.switchTeamsCountdown, 0.9);
			end
		end
		
		-- Propigate the timer
		self:DoGameRulesLibTimer();
	end,
--------------------------------
	OnClientDisconnect = GameRules.HandleClientDisconnect,
}
-------------------------------------------------
--INTERMISSION
-------------------------------------------------
GameRules.states.INTERMISSION={
--------------------------------
	OnBeginState=GameRules.INTERMISSION_BeginState,
--------------------------------
	OnUpdate=GameRules.INTERMISSION_OnUpdate,
}
---------------------------------------------------------------------------------



-------------------------------------------------------------------------------
-- is updating GameRules.RespawnPoints
-- \param team e.g. "red", "blue", "spectators" or "players"
-- \return table {[1]={x=,y=,z=,xA=,yA=,zA=},[2]={x=,y=,z=,xA=,yA=,zA=},..} 
function GameRules:RecreateTableOfRespawnPoints()
	local checkpointnumber=self.CurrentCheckPoint_Number;
	local ents=System:GetEntities();		-- very inefficient
	local i=1;
	
	GameRules.RespawnPoints={};

	-- got through all AssultCheckpoints	
	for i,entity in self.MapCheckPoints do
		if entity.Properties.CheckPoint_Number==checkpointnumber then
			if (entity.Properties.bAttackerSpawnPoint==1) then
				local pt = new(entity:GetPos());
				local rot=entity:GetAngles();
				GameRules.RespawnPoints[i] = {x=pt.x, y=pt.y, z=pt.z, xA=rot.x, yA=rot.y, zA=rot.z, name=self.attackerTeam };
				i=i+1;
			end
			if (entity.Properties.bDefenderSpawnPoint==1) then
				local pt = new(entity:GetPos());
				local rot=entity:GetAngles();
				GameRules.RespawnPoints[i] = {x=pt.x, y=pt.y, z=pt.z, xA=rot.x, yA=rot.y, zA=rot.z, name=self.defenderTeam };
				i=i+1;
			end
		end
	end

	return ret;
end






-------------------------------------------------------------------------------
-- private method
-- /return number or nil if there is no next number
function GameRules:GetNextBiggerCheckPointNumber()

	local ret;
	
	for i,entity in self.MapCheckPoints do
		if ret==nil then
			if entity.Properties.CheckPoint_Number>self.CurrentCheckPoint_Number then
				ret = entity.Properties.CheckPoint_Number;
			end
		else 
			if entity.Properties.CheckPoint_Number>self.CurrentCheckPoint_Number and entity.Properties.CheckPoint_Number<ret then
				ret = entity.Properties.CheckPoint_Number;
			end
		end
	end
	
	return ret;
end



-------------------------------------------------------------------------------
-- private method
function GameRules:RecalcCheckPointIndices()

	self.CheckPointIndices={};

	for i,entity in self.MapCheckPoints do
		self.CheckPointIndices[entity.Properties.CheckPoint_Number]=1;
	end
end

-------------------------------------------------------------------------------
-- private method
-- \return 0..self.CheckPointIndices_Count
function GameRules:CalcCheckPointIndex( inCheckPointNumber )

	local ret=0;
	
	for key,value in self.CheckPointIndices do
		if key<inCheckPointNumber then
			ret=ret+1;
		end
	end
	
	return ret;
end

-------------------------------------------------------------------------------
-- is called by the ASSAULTCheckPoint entity, test if the entity is an attacker
function GameRules:IsAttacker(entity)
	if (entity) then
		local team=Game:GetEntityTeam(entity.id);
	
		if (team == self.attackerTeam) then
		
			local slot=Server:GetServerSlotByEntityId(entity.id);
			
			if (self.respawnList) then
				local respawn = self.respawnList[slot];
		
				-- if the player will respawn on a different team, then he is not an attacker anymore.	
				if (respawn and respawn.team) then
					if (respawn.team ~= team) then
						return nil;
					end
				end
			end
			
			return 1;
		end
	end
	
	return nil;
end

-------------------------------------------------------------------------------
-- test if the entity is a defender
function GameRules:IsDefender(entity)
	if (entity) then
		local team=Game:GetEntityTeam(entity.id);
	
		if (team == self.defenderTeam) then
			return 1;
		end
	end
	
	return nil;
end


-------------------------------------------------------------------------------
function GameRules:GetTeamRespawnPoint(team,entityToIgnore)

	local pt;

	pt = self:GetFreeTeamRespawnPoint(team,entityToIgnore);
	if(pt)then return pt; end
	
	System:Error("No possible respawn points ('red'/'blue' or 'players')");
end



-- /return 1=yes everything is ok / 2=was touched in advance / nil=no message
function GameRules:IsCheckpointTouchable(checkpoint,collider)
	
	local newNumber=checkpoint.Properties.CheckPoint_Number;
	
	if newNumber<=self.CurrentCheckPoint_Number then
		return; -- checkpoint was already touched
	end

	if GameRules:GetState()~="INPROGRESS" then
		return;
	end	
	
	if collider.type~="Player" then
		return;
	end
	
	if collider.cnt.health<=0 then
		return;
	end
		
	return 1;	-- checkpoint was triggered by event
end

function GameRules:IsCheckpointTouchableSSId(checkpoint,colliderslotid)

	local newNumber=checkpoint.Properties.CheckPoint_Number;
	
	if newNumber<=self.CurrentCheckPoint_Number then
		return; -- checkpoint was already touched
	end

	if GameRules:GetState()~="INPROGRESS" then
		return;
	end
	
	local collider = GetPlayerFromSSId(colliderslotid);
	
	if collider.type~="Player" then
		return;
	end
	
	if collider.cnt.health<=0 then
		return;
	end
		
	return 1;	-- checkpoint was triggered by event
end


function GameRules:FortCheck()
--3up fort add fort check
	if (tonumber(getglobal("gr_tMode")) == 2) then
		if self:HaveFortWon() then
			Server:BroadcastText("Total Dominance!");
			self:SwitchTeams();

			self.mapstart = _time;
			if (self.switchTeamsTimer and self.switchTeamsTimer > 0) then
				self.mapstart = self.mapstart + self.switchTeamsTimer;
			end
		end
	end
--3up fort add fort check end
end

-------------------------------------------------------------------------------
-- is called by the ASSAULTCheckPoint entity
-- \param checkpoint ASSAULTCheckPoint
-- \param collider playerentity or nil if there was no collider (order is not checked, team is not checked)
function GameRules:TouchedCheckpoint(checkpoint,colliderssid)

	local newNumber=checkpoint.Properties.CheckPoint_Number;
	local collider = GetPlayerFromSSId(colliderssid);

	if GameRules:IsAttacker(collider) == nil then										-- otherwise checkpoint was triggered by event
		return -- only attacker can touch waypoints
	end

	if newNumber>self.CurrentCheckPoint_Number then
		
		if collider then										-- otherwise checkpoint was triggered by event
			local expected=self:GetNextBiggerCheckPointNumber();
		
			if newNumber~=expected then 	-- Don't touch checkpoints in advance
				return;
			end
			
			Server:BroadcastText(collider:GetName().."$o @CapturedObj");

			MPStatistics:AddStatisticsDataEntity(collider,"nCaptureFinished",1);
		else
			Server:BroadcastText("@ObjWasCaptured");
		end

		self.CurrentCheckPoint_Number = newNumber;
		self:RefreshCheckpoints();
		self:OnCaptureStep_Warmup();
		
		--Server:BroadcastText("Next Objective is checkpoint "..self.CurrentCheckPoint_Number);
	end
	
	

	if self:HaveAttackersWon() then
		Server:BroadcastText("@AttackerWon");
		--GameCommands:restart();
		
		local team_score=Game:GetTeamScore(self.attackerTeam);
		team_score=team_score+1;
		Server:SetTeamScore(self.attackerTeam,team_score);
	  
		self:SwitchTeams();
		self.mapstart = _time;
		if (self.switchTeamsTimer and self.switchTeamsTimer > 0) then
			self.mapstart = self.mapstart + self.switchTeamsTimer;
		end
	end
	


end

-------------------------------------------------------------------------------
-- after attacker managed to capture a checkpoint or game restarts
function GameRules:ResetAtCurrentCheckpoint()
	local slots = Server:GetServerSlotMap();
	
	for i, slot in slots do
		local ent = System:GetEntity(slot:GetPlayerId());
	
	  	if ent~=nil and ent.type=="Player" and Game:GetEntityTeam(ent.id)~="spectators" and ent.cnt.health >0 then
			local pos=self:GetTeamRespawnPoint(Game:GetEntityTeam(ent.id), ent);
			if (pos) then
				-- beam to new position - amoo and others is not reseted			
				ent:SetPos(pos);
				ent:SetAngles({x=pos.xA, y=pos.yA, z=pos.zA});
			end
		end
	end
end

-------------------------------------------------------------------------------
-- Start the countdown for switching teams
function GameRules:SwitchTeams()

	self:InvalidateAllCaptures();

	self.numgolda=0;
	self.numgoldb=0;

	if (tostring(self.attackerTeam) == "red") then
		self.red_played_attack = self.red_played_attack + 1;
		self.blue_played_defense = self.blue_played_defense + 1;
	end

	if (tostring(self.attackerTeam) == "blue") then
		self.blue_played_attack = self.blue_played_attack + 1;
		self.red_played_defense = self.red_played_defense + 1;
	end


	local permap = tonumber(getglobal("gr_GamePerMap"));
	if (tonumber(getglobal("gr_tMode")) == 3)  then
		permap = permap * 6;
	end
	if ((self.red_played_attack == permap) and 
	(self.red_played_defense == permap)) then
		
		self:UpdateTimeLimit(getglobal("gr_TimeLimit"));
		
		MapCycle:OnMapFinished();
		
	else
		
		if (tonumber(getglobal("gr_tMode")) == 3)  then
			local currflagz = tonumber(getglobal("gr_currFlag")) + 1;
			System:Log("currflag "..currflagz);

			if (currflagz > 3) then
				currflagz = 1;
			end
			setglobal("gr_currFlag", currflagz);
		end
		if (tonumber(getglobal("gr_stopwatch")) == 0) or 
			(self.red_played_attack == self.red_played_defense) then
			self:UpdateTimeLimit(getglobal("gr_TimeLimit"));
		else
			if (self.timelimit and tonumber(self.timelimit) > 0) then
				self:UpdateTimeLimit((_time - self.mapstart) / 60);
			else
				self:UpdateTimeLimit(0);
			end
		end

		if (self.switchTeamsTimer~=nil and self.switchTeamsTimer>0) then

			self:ForceScoreBoard(1);
			self.switchTeamsCountdown=self.switchTeamsTimer;
		else
			self:ForceScoreBoard(0);
			self:DoSwitchTeams(); -- with no timer switch immediately
		end
	end
end
-------------------------------------------------------------------------------
-- Change all client teams
function GameRules:ResetMap()

	self.mapstart = _time;
	self.countnext = 0;
	
	self:ResetMapEntities();
	
	-- put all checkpoints of the map in a lua table
	self.MapCheckPoints={};

	local ents=System:GetEntities();
	for i,entity in ents do
		if entity.classname=="ASSAULTCheckPoint" then
			self.MapCheckPoints[getn(self.MapCheckPoints)+1]=entity;
		end
	end

	self:RecalcCheckPointIndices();
	self:SetToFirstCheckPoint(); -- reset the checkpoints
	self:ResetAtCurrentCheckpoint();

	self:SetTeamObjectivesAllPlayers();
	self:NewGameState(CGS_INPROGRESS);
end

-------------------------------------------------------------------------------
-- Change all client teams
function GameRules:DoSwitchTeams()
	
	if (tonumber(getglobal("gr_tMode")) == 2) then
		Server:SetTeamScore("blue", tonumber(gr_FortScoreStart));
 		Server:SetTeamScore("red", tonumber(gr_FortScoreStart));
	end

	self.attackerTeam, self.defenderTeam = self.defenderTeam, self.attackerTeam;

	self:ResetMap();
end

-------------------------------------------------------------------------------
-- refresh all checkpoint objects (set state depending on self.CurrentCheckPoint_Number)
function GameRules:RefreshCheckpoints()
	local current=self.CurrentCheckPoint_Number;
	local nextone=self:GetNextBiggerCheckPointNumber();
	
	for i,entity in self.MapCheckPoints do
		if nextone and entity.Properties.CheckPoint_Number>nextone then
			entity:Event_Blocked();
		elseif entity.Properties.CheckPoint_Number>current then
--			entity:Event_Untouched();	
			entity:Event_Warmup();	
		elseif entity.Properties.CheckPoint_Number<current then
			assert(self.attackerTeam)
			entity:Event_Touched();
		else
			entity:Event_Spawn();
		end
	end

	self:RecreateTableOfRespawnPoints();		-- update GameRules.RespawnPoints
end


-------------------------------------------------------------------------------
function GameRules:GetAttackerTeam()
	return self.attackerTeam;
end

-------------------------------------------------------------------------------
function GameRules:GetDefenderTeam()
	return self.defenderTeam;
end


-------------------------------------------------------------------------------
function GameRules:InvalidateAllCaptures()

	local currentcp = GameRules:GetNextBiggerCheckPointNumber();

--	System:Log("GameRules:InvalidateAllCaptures"..tostring(currentcp));	-- debug

	for i,cp in self.MapCheckPoints do
		if ((cp.Properties.bVisible == 1) and (cp.Properties.CheckPoint_Number == currentcp)) then
			if(cp.captureCollider)then
				
--				System:Log("GameRules:InvalidateAllCaptures Event_Averted "..tostring(cp));	-- debug

				cp:Event_Averted();
			end
		end
	end
end

-------------------------------------------------------------------------------
-- return 1 or nil if not  
function GameRules:HaveAttackersWon()

	if (self.switchTeamsCountdown~=nil and self.switchTeamsCountdown>0) then
		return nil;		-- ongoing counter to team switch
	end
	
	for i,entity in self.MapCheckPoints do
		if entity.Properties.CheckPoint_Number>self.CurrentCheckPoint_Number then
			return nil;
		end
	end

	return 1;	-- there was no checkpoint with a bigger number
end

-- 3up fort add end of game start
function GameRules:HaveFortWon()
	local red_score = Game:GetTeamScore("red");		
	local blue_score = Game:GetTeamScore("blue");
	local winscore = tonumber(gr_FortScoreEnd);
	if(red_score>=winscore) then
		return 1;		
	end
	if(blue_score>=winscore) then
		return 1;		
	end
	return nil;	
end
-- 3up fort add end of game end

-------------------------------------------------------------------------------
-- tell one player who is attacking and who defending
function GameRules:SetTeamObjectivesOnePlayer( slot )
  local ent = System:GetEntity(slot:GetPlayerId());
    
	if ent~=nil then
		if (tonumber(getglobal("gr_tMode")) == 3) then
			if (ent.cnt.vip >= 2) then
				slot:SendCommand("FLG V");
				ent.cnt.KingTimer = tonumber(getglobal("gr_KingTimer"));
				slot:SendCommand("UKL "..ent.cnt.KingTimer);

			else 
				ent.cnt.vip = 0;
				if (Game:GetEntityTeam(ent.id)==self.attackerTeam) then
					slot:SendCommand("CPO POAtt");
				elseif (Game:GetEntityTeam(ent.id)==self.defenderTeam) then	
					slot:SendCommand("CPO PODef");		
				end
			end

		else
			ent.cnt.vip = 0;
			if (Game:GetEntityTeam(ent.id)==self.attackerTeam) then
				slot:SendCommand("CPO POAtt");						-- invoke OnServerCmd() on the client
			elseif (Game:GetEntityTeam(ent.id)==self.defenderTeam) then	
				slot:SendCommand("CPO PODef");						-- invoke OnServerCmd() on the client
			else	
				slot:SendCommand("CPO");									-- remove player objective invoke OnServerCmd() on the client
			end
		end
	end
end

-------------------------------------------------------------------------------
-- tell all players who is attacking and who defending
function GameRules:SetTeamObjectivesAllPlayers()
	local slots = Server:GetServerSlotMap();

	for i, slot in slots do
	    self:SetTeamObjectivesOnePlayer(slot);    
	end
end
