--An animated boomgate to prevent vehicles from entering an area.
--by aarbro [aarbro@hotmail.com]

Boomgate = {

	Properties = {
		objMotor = "Objects/EXP/boomgate_motor.cgf",
		objBoomgate = "Objects/EXP/boomgate_gate.cgf",
		fSpeed = 40, --max speed in degrees per second
		fInertia = 20, --angle at which speedup/slowdown starts/ends
		fAngleClosed = 0, --down angle
		fAngleOpened = 75, --up angle
		vectorBBoxMin = { x=-6, y=-1, z=0 }, --must cover the whole animation cycle
		vectorBBoxMax = { x=1, y=1, z=7 },
		bOpened = 0, --boom state at start
		sndSound = "Sounds/EXP/boomgate.wav",
		bEnabled = 1,
	},
	
	currAngle = { x=0, y=0, z=0 },
	direction = 0,
};



function Boomgate:OnInit()
	self:OnReset();
end

function Boomgate:OnPropertyChange()
	self:OnReset();
end


function Boomgate:OnReset()

	self:LoadObject( self.Properties.objMotor, 0, 0 );
	self:CreateStaticEntity(100,-1);
	self:DrawObject( 0, 1 );
	
	local helperPos=self:GetHelperPos("gatepos", 1);
	self:LoadObject( self.Properties.objBoomgate, 1, 0);
	self:CreateStaticEntity(30,-1);
	self:SetObjectPos( 1 , helperPos );
	self:DrawObject( 1, 1 );
	
	if ( self.Properties.bOpened == 1 ) then
		self.currAngle.y = self.Properties.fAngleOpened;
	else
		self.currAngle.y = self.Properties.fAngleClosed;
	end
	
	self:SetObjectAngles( 1, self.currAngle );
	
	self.MotorSound=Sound:Load3DSound(self.Properties.sndSound);
	Sound:SetSoundVolume(self.MotorSound, 128);
	Sound:SetMinMaxDistance(self.MotorSound, 3, 20);
	Sound:SetSoundLoop(self.MotorSound, 1);

	self:SetBBox( self.Properties.vectorBBoxMin, self.Properties.vectorBBoxMax );

	self:EnableUpdate(0);
	
end


function Boomgate:OnUpdate(dt)

	local slowdown = 1;
	if ( ( self.Properties.fAngleOpened - self.currAngle.y ) < self.Properties.fInertia ) then
		slowdown = ( self.Properties.fAngleOpened - self.currAngle.y ) / self.Properties.fInertia;
	elseif ( ( self.currAngle.y - self.Properties.fAngleClosed ) < self.Properties.fInertia ) then
		slowdown = ( self.currAngle.y - self.Properties.fAngleClosed ) / self.Properties.fInertia;
	end
	if ( slowdown < 0.05 ) then
			slowdown = 0.05;
	end
	
	self.currAngle.y = self.currAngle.y + self.Properties.fSpeed * dt * self.direction * slowdown;

	if ( self.currAngle.y > self.Properties.fAngleOpened ) then
		self.currAngle.y = self.Properties.fAngleOpened;
		self.direction = 0;
		self:EnableUpdate(0);
		self.Properties.bOpened = 1;
		BroadcastEvent(self, "Opened");
		Sound:StopSound( self.MotorSound );
	end
	if ( self.currAngle.y < self.Properties.fAngleClosed ) then
		self.currAngle.y = self.Properties.fAngleClosed;
		self.direction = 0;
		self:EnableUpdate(0);
		self.Properties.bOpened = 0;
		BroadcastEvent(self, "Closed");
		Sound:StopSound( self.MotorSound );
	end

	self:SetObjectAngles( 1, self.currAngle );

end


function Boomgate:Event_Open()
	if ( self.Properties.bEnabled == 1 ) then
  	self:EnableUpdate(1);
  	self.direction = 1;
  	BroadcastEvent(self, "Opening");
  	self:PlaySound( self.MotorSound );
	end
end


function Boomgate:Event_Close()
	if ( self.Properties.bEnabled == 1 ) then
  	self:EnableUpdate(1);
  	self.direction = -1;
  	BroadcastEvent(self, "Closing");
  	self:PlaySound( self.MotorSound );
	end
end


function Boomgate:Event_Opened()
end

function Boomgate:Event_Closed()
end

function Boomgate:Event_Opening()
end

function Boomgate:Event_Closing()
end

function Boomgate:OnSave(stm)
	stm:WriteFloat( self.currAngle.y );
	stm:WriteInt( self.direction );
end

function Boomgate:OnLoad(stm)
	self.currAngle.y = stm:ReadFloat();
	self.direction = stm:ReadInt();
	if ( self.direction == 0 ) then
		self:EnableUpdate(0);
	elseif ( self.Properties.bEnabled == 1 ) then
		self:Event_Enable();
	end
end

function Boomgate:Event_Enable()
	self.Properties.bEnabled = 1;
	self:EnableUpdate(1);
	if ( self.direction ~= 0 ) then
  	self:PlaySound( self.MotorSound );
  end
 	BroadcastEvent(self, "Enable");
end

function Boomgate:Event_Disable()
	self.Properties.bEnabled = 0;
	self:EnableUpdate(0);
	Sound:StopSound( self.MotorSound );
 	BroadcastEvent(self, "Disable");
end
