--Connect to an area and it will give electric damage to the player with visual and audio effects.
--by aarbro [aarbro@hotmail.com]

ElectricDamage = {

	Editor = {
		Model="Objects/EXP/Editor/editor_flash.cgf",
	},

	Properties = {
		bEnabled = 1,
		fParticleScale = 5,			--scale of the spark particle
		fUpdateTimeMin = 0.1,		--shortest time between sparks in s
		fUpdateTimeMax = 0.3,		--longest time between sparks in s
		fDamage = 1,						--damage per spark
		fWhiteOutTime = 0.2,		--time to display whiteout effect in s, -1 disables
		ParticleEffect = "misc.sparks.a",
		Sound = {
			sndFile1 = "sounds/explosions/spark2.wav",	--sounds that are randomly played at a spark
			sndFile2 = "sounds/explosions/spark3.wav",
			sndFile3 = "sounds/explosions/spark4.wav",
			fVolume = 255,
			fMinDistance = 3,
			fMaxDistance = 20,
		},
	},
	
	inside = 0,
};
-------------------------------------------------------------------------------
function ElectricDamage:OnPropertyChange()
	self:OnReset();
end
-------------------------------------------------------------------------------

function ElectricDamage:OnInit()
	self:OnReset();
end
-------------------------------------------------------------------------------

function ElectricDamage:OnReset()

	if (self.Properties.sndSound1 ~= "") then
		self.Sound1=Sound:Load3DSound(self.Properties.Sound.sndFile1);
		Sound:SetSoundVolume(self.Sound1, self.Properties.Sound.fVolume);
		Sound:SetMinMaxDistance(self.Sound1, self.Properties.Sound.fMinDistance, self.Properties.Sound.fMaxDistance);
		Sound:SetSoundLoop(self.Sound1,0);
	else
		self.Sound1 = nil;
	end
	if (self.Properties.sndSound2 ~= "") then
		self.Sound2=Sound:Load3DSound(self.Properties.Sound.sndFile2);
		Sound:SetSoundVolume(self.Sound2, self.Properties.Sound.fVolume);
		Sound:SetMinMaxDistance(self.Sound2, self.Properties.Sound.fMinDistance, self.Properties.Sound.fMaxDistance);
		Sound:SetSoundLoop(self.Sound2,0);
	else
		self.Sound2 = nil;
	end
	if (self.Properties.sndSound3 ~= "") then
		self.Sound3=Sound:Load3DSound(self.Properties.Sound.sndFile3);
		Sound:SetSoundVolume(self.Sound3, self.Properties.Sound.fVolume);
		Sound:SetMinMaxDistance(self.Sound3, self.Properties.Sound.fMinDistance, self.Properties.Sound.fMaxDistance);
		Sound:SetSoundLoop(self.Sound3,0);
	else
		self.Sound3 = nil;
	end
	
	self:TrackColliders(1);
	self:EnableUpdate(0);
	
	if ( self.Properties.bEnabled == 1 ) then
		self:Event_Enable();
	else
		self:Event_Disable();
	end


end
-------------------------------------------------------------------------------

function ElectricDamage:OnShutDown()
end
-------------------------------------------------------------------------------

function ElectricDamage:OnUpdate()
end
-------------------------------------------------------------------------------

function ElectricDamage:OnSave(stm)
end
-------------------------------------------------------------------------------

function ElectricDamage:OnLoad(stm)
end
-------------------------------------------------------------------------------

function ElectricDamage:OnTimer( )
	--whiteout effect
	if ( self.Properties.fWhiteOutTime ~= -1 ) then
		local vec = _localplayer:GetPos();
		System:ProjectToScreen(vec);
		System:SetScreenFx("FlashBang", 1);
		System:SetScreenFxParamFloat("FlashBang", "FlashBangFlashPosX", vec.x);
		System:SetScreenFxParamFloat("FlashBang", "FlashBangFlashPosY", vec.y);
		System:SetScreenFxParamFloat("FlashBang", "FlashBangTimeScale", self.Properties.fWhiteOutTime );
	end

	--create spark
	local dir = self:GetDirectionVector();
	local playerPos = _localplayer:GetPos();
	playerPos.z = playerPos.z + _localplayer.PhysParams.eyeheight;
	Particle:SpawnEffect( playerPos, dir, self.Properties.ParticleEffect, self.Properties.fParticleScale );
	
	self:PlaySound( playerPos );
	
	--cause damage
if (self==_localplayer and Hud~=nil) then
  Hud:OnMeleeDamage( "MeleeDamageNormal" );		
end
 	local elecDam = {
 		dir = {x=0, y=0, z=1},
 		damage = self.Properties.fDamage,
 		target = _localplayer,
 		shooter = _localplayer,
 		landed = 1,
 		impact_force_mul_final=5,
 		impact_force_mul=5,
 		damage_type = "normal",
 	};
	_localplayer:Damage( elecDam );

	local next_time = random() * (self.Properties.fUpdateTimeMax - self.Properties.fUpdateTimeMin ) + self.Properties.fUpdateTimeMin;
	self:SetTimer( next_time * 1000 );

	BroadcastEvent(self, "Contact");

end 
-------------------------------------------------------------------------------
function ElectricDamage:OnEnterArea( player,areaId )
 	self.inside = 1;
 	if (  self.Properties.bEnabled == 1 ) then
 		self:OnTimer();
 	end
end
-------------------------------------------------------------------------------
function ElectricDamage:OnLeaveArea( player,areaId )
	self.inside = 0;
	self:KillTimer();
	System:SetScreenFx("ScreenBlur", 0);
end
-------------------------------------------------------------------------------

function ElectricDamage:Event_Enable()
	if ( self.inside == 1 ) then
		self:OnTimer();
	end
	BroadcastEvent(self, "Enable");
	self.Properties.bEnabled = 1;
end
-------------------------------------------------------------------------------

function ElectricDamage:Event_Disable()
	self:KillTimer();
	BroadcastEvent(self, "Disable");
	self.Properties.bEnabled = 0;
end
-------------------------------------------------------------------------------
function ElectricDamage:Event_Contact()
end
-------------------------------------------------------------------------------

function ElectricDamage:PlaySound( pos )

	local index = random( 1, 3 );	
		
	if ( index == 1 and self.Sound1 ~= nil ) then
		Sound:SetSoundPosition( self.Sound1, pos );
		Sound:PlaySound( self.Sound1 );
	end
	if ( index == 2 and self.Sound2 ~= nil ) then
		Sound:SetSoundPosition( self.Sound2, pos );
		Sound:PlaySound( self.Sound2 );
	end
	if ( index == 3 and self.Sound3 ~= nil ) then
		Sound:SetSoundPosition( self.Sound3, pos );
		Sound:PlaySound( self.Sound3 );
	end

end 

-------------------------------------------------------------------------------




