--Hit Target Entity v1.2 (redesigned for ShootingRange map)
--by aarbro [aarbro@hotmail.com]


HitTarget = {

	Properties = {
		Distance = 0, --minimum distance at which this target can be targeted
		MaxPoints = -1, --when points exceed this Event_MaxPoints is fired, -1 disables event
		Maxshots = -1, --when shots exceed this Event_MaxShots is fired, -1 disables event
		TargetData = {
			fOffsetX = 0, --x distance from mesh origin to target circle center, in m.
			fOffsetZ = 1.57, --z distance from mesh origin to target circle center, in m.
			fRadius = 0.24, --radius of target circle, in m.
			MinPoints = 5, --points at outer circle.
			MaxPoints = 10, --points at center.
			MissPoints = 0, --points when outside circle.
			objModel = "Objects/EXP/hittarget.cgf",
		},
	},
	
	points = 0, --last hit points
	totalPoints = 0, --sum of all hit points
	shots = 0, --number of shots that has hit
	series = "", --last ~30 shots as a string
};

-------------------------------------------------------------------------------
--SYSTEM EVENTS (called by builtin scripts or FC system)
-------------------------------------------------------------------------------

function HitTarget:OnPropertyChange()
	self:OnReset();
end

function HitTarget:OnInit()
	self:OnReset();
end

function HitTarget:OnReset()
	self:LoadObject( self.Properties.TargetData.objModel, 0, 1 );
	self:CreateStaticEntity( 100, -1 );
	self:DrawObject( 0, 1 );
	
	self:Event_ResetScore();
end

function HitTarget:OnShutDown()
end

function HitTarget:OnUpdate()
	Hud.hittarget_name = "Time";
	Hud.hittarget_hits = System:GetCurrTime();
end

function HitTarget:OnSave(stm)
end

function HitTarget:OnLoad(stm)
end


function HitTarget:OnDamage(Hit)


	local dmg=Hit.damage;
	local shooter;
	if not shooter then																								-- explosive damage (entity id might be reassigned)
		--local ss_shooter=Server:GetServerSlotBySSId(Hit.shooterSSID);
		--if ss_shooter then		-- might be nil e.g. vehicle destroyes itself
		--    	shooter = System:GetEntity(ss_shooter:GetPlayerId());
	      --end
	end

	if not shooter then 
		--Hud:AddMessage("No shooter",5);
		shooter = Hit.shooter;
	end



	--BroadcastEvent(self, "OnDamage");
	
	--calculate hit location as a distance in meters from the center
	local realPos = self:GetPos();
	realPos.x = self.Properties.TargetData.fOffsetX - ( Hit.pos.x - realPos.x );
	realPos.z = self.Properties.TargetData.fOffsetZ - ( Hit.pos.z - realPos.z );
	local dist = sqrt( realPos.x ^ 2 + realPos.z  ^ 2 );

	--convert distance to points
	if (dist < self.Properties.TargetData.fRadius ) then
		self.points = self.Properties.TargetData.MaxPoints - floor ( ( dist / self.Properties.TargetData.fRadius ) * ( self.Properties.TargetData.MinPoints + 1 )  );
	else
		self.points = self.Properties.TargetData.MissPoints;
	end
	
	--if shooter is too far from the set target distance ignore the hit
	if shooter then
		local dist = shooter:GetDistanceFromPoint( self:GetPos() );
	else
		local dist = 200;
	end
	if ( dist < self.Properties.Distance ) then
if (Hud~=nil) then
		Hud:AddMessage( "Target must be at " .. self.Properties.Distance .. "m or longer than "..dist, 5 );
end
		--return;
	end

	--signal event for bullseye, hit or miss
	if ( self.points == self.Properties.TargetData.MaxPoints ) then 
		BroadcastEvent(self, "Bullseye");
	elseif ( self.points == self.Properties.TargetData.MissPoints ) then
		BroadcastEvent(self, "Miss");
	else
		BroadcastEvent(self, "Hit");
	end

	--adjust statistics
	self.totalPoints = self.totalPoints + self.points;
	self.shots = self.shots + 1;

	--add last round to series string
	if ( self.series == "" ) then
		self.series = self.points;
	else
		self.series = self.series .. "," .. self.points;
	end

	--truncate series string at beginning if it's too long
	if ( strlen( self.series ) > 63 ) then
		self.series = "..." .. strsub( self.series, strlen(self.series) - 63 );
	end

	--show this shot's result and update scoreboard
	local avg = format("%.1f",self.totalPoints / max(self.shots,1) );
if (Hud~=nil) then
	Hud:AddMessage( self.points .." points at " .. self.Properties.Distance .. "m", 5);
end

	----------------------------
	----------------------------
	--Hud:ShowHitTarget( "HitTarget1", self.points);
	--Hud.hittarget_name = "Time";
	--Hud.hittarget_hits = System:GetCurrTime();
	----------------------------
	----------------------------

	self:OnUpdateScoreboard();
	
	--fire events if limits are exceeded
	if ( self.Properties.Maxshots ~= -1 ) then
		if ( self.shots > self.Properties.Maxshots ) then
				BroadcastEvent(self, "MaxShots");
		end
	end
	if ( self.Properties.MaxPoints ~= -1 ) then
		if ( self.totalPoints > self.Properties.MaxPoints ) then
				BroadcastEvent(self, "MaxPoints");
		end
	end
end



-------------------------------------------------------------------------------
--CUSTOM EVENTS (called by this or other custom entities)
-------------------------------------------------------------------------------

--reset score and update objective window
function HitTarget:Event_ResetScore()
	self.totalPoints = 0;
	self.shots = 0;
	self.series = "";
	self:OnUpdateScoreboard()
end

-------------------------------------------------------------------------------


--update scoreboard in objective window
function HitTarget:OnUpdateScoreboard()
if (Hud~=nil and _localplayer~=nil) then
	--Hud:FlashObjectives();
end
	local ents = System:GetEntities();
	for idx,e in ents do
		if (e.type == "HitTarget" ) then
			avg = format( "%.1f", e.totalPoints / max(e.shots, 1) );
if (Hud~=nil and _localplayer~=nil) then
			--Hud:PushObjective({}, e.Properties.Distance .."m - " .. e.totalPoints .. "p " .. e.shots .. "r (" .. avg .. ") series: ".. e.series);
end
		end
	end
end
-------------------------------------------------------------------------------
function HitTarget:Event_MaxPoints()
end
-------------------------------------------------------------------------------
function HitTarget:Event_MaxShots()
end
-------------------------------------------------------------------------------
function HitTarget:Event_Bullseye()
end
-------------------------------------------------------------------------------
function HitTarget:Event_Hit()
end
-------------------------------------------------------------------------------
function HitTarget:Event_Miss()
end
-------------------------------------------------------------------------------

