--A keypad that fires an event when the code is entered on the numpad keys by the player.
--by aarbro [aarbro@hotmail.com]

Keypad = {
	
	Properties = {
		bEnabled = 1, --whether the keypad is working
		sEnabledKeys = ":0123456789", --must begin with a : to force it to be a string
		bEnableCodes = 1,
		sCode1 = "1234",
		sCode2 = "-1",
		sCode3 = "-1",
		vectorBBox = {x=1,y=1,z=1},
		TextInstruction= "Enter Code with numpad.",
		sndSound = "Sounds/EXP/beep.wav",
		bUseNumpad = 1,
	},

	sequence = "",

	Editor={
		Model="Objects/EXP/Editor/editor_keypad.cgf",
	},

}

-------------------------------------------------------------------------------
function Keypad:OnPropertyChange()
	self:OnReset();
end

-------------------------------------------------------------------------------
function Keypad:OnInit()
	self:OnReset();
end

-------------------------------------------------------------------------------
function Keypad:OnReset()

	local Min = { x=-self.Properties.vectorBBox.x/2, y=-self.Properties.vectorBBox.y/2, z=-self.Properties.vectorBBox.z/2 };
	local Max = { x=self.Properties.vectorBBox.x/2, y=self.Properties.vectorBBox.y/2, z=self.Properties.vectorBBox.z/2 };
	self:SetBBox( Min, Max );
	
	if ( self.Properties.bEnabled == 1 ) then
		self:TrackColliders(1);
	else
		self:TrackColliders(0);
	end
	
	if ( self.Properties.sndSound ~= "" ) then
		self.Sound=Sound:Load3DSound(self.Properties.sndSound);
		Sound:SetSoundVolume(self.Sound, 128);
		Sound:SetMinMaxDistance(self.Sound, 3, 5);
	end
	
	self.sequence = "";
	
end
-------------------------------------------------------------------------------
function Keypad:OnEnterArea()
if (Hud~=nil) then
 		Hud:AddMessage( self.Properties.TextInstruction );
 end

	if ( self.Properties.bUseNumpad == 1 ) then
		Input:BindAction("MOVEMODE", "", "default", 1);
		Input:BindAction("MOVE_BACKWARD", "", "default", 1);
		Input:BindAction("MOVE_LEFT", "", "default", 1);
		Input:BindAction("MOVE_FORWARD", "", "default", 1);
		Input:BindAction("MOVE_RIGHT", "", "default", 1);
		Input:BindAction("LEANLEFT", "", "default", 1);
		Input:BindAction("LEANRIGHT", "", "default", 1);
	else
		Input:BindAction("WEAPON_1", "", "default", 1);
		Input:BindAction("WEAPON_2", "", "default", 1);
		Input:BindAction("WEAPON_3", "", "default", 1);
		Input:BindAction("WEAPON_4", "", "default", 1);
	end

 	self:EnableUpdate(1);

end

-------------------------------------------------------------------------------

function Keypad:OnLeaveArea()

	if ( self.Properties.bUseNumpad == 1 ) then
		Input:BindAction("MOVEMODE", "numpad0", "default", 1);
		Input:BindAction("MOVE_BACKWARD", "numpad2", "default", 1);
		Input:BindAction("MOVE_LEFT", "numpad4", "default", 1);
		Input:BindAction("MOVE_FORWARD", "numpad5", "default", 1);
		Input:BindAction("MOVE_RIGHT", "numpad6", "default", 1);
		Input:BindAction("LEANLEFT", "numpad7", "default", 1);
		Input:BindAction("LEANRIGHT", "numpad9", "default", 1);
	else
		Input:BindAction("WEAPON_1", "1", "default", 1);
		Input:BindAction("WEAPON_2", "2", "default", 1);
		Input:BindAction("WEAPON_3", "3", "default", 1);
		Input:BindAction("WEAPON_4", "4", "default", 1);
	end

	self:EnableUpdate(0);
	self.sequence = "";

end

-------------------------------------------------------------------------------

function Keypad:OnUpdate()
	local key = tostring( Input:GetXKeyPressedName() );
	local number = -1;
	
	if ( self.Properties.bUseNumpad == 1 ) then
		if ( strsub( key, 1, 6 ) ~= "numpad" ) then --numpad pressed
			return;
		end
		if ( strsub( key, 8, 12 ) == "enter" ) then --ignore numpad enter
			return;
		end
		number = strsub( key, 7, 7 );
		if ( strfind( self.Properties.sEnabledKeys, number, 1, plain ) == nil ) then --key disabled
			return;
		end
	else
		if ( strfind( "0123456789", key, 1, plain ) ~= nil ) then
			number = key;
		else
			return;
		end
		if ( strfind( self.Properties.sEnabledKeys, number, 1, plain ) == nil ) then --key disabled
			return;
		end
	end
	
	
	if ( self.Properties.bEnableCodes == 1 ) then --check codes
 		self.sequence = self.sequence..number; --enter number into sequence
 		if ( strfind( self.sequence, self.Properties.sCode1, 1, plain ) ~= nil ) then --Code1 correct
 			BroadcastEvent(self, "Code1Correct");
 			self.sequence = "";
 		end
 		if ( strfind( self.sequence, self.Properties.sCode2, 1, plain ) ~= nil ) then --Code2 correct
 			BroadcastEvent(self, "Code2Correct");
 			self.sequence = "";
 		end
 		if ( strfind( self.sequence, self.Properties.sCode3, 1, plain ) ~= nil ) then --Code3 correct
 			BroadcastEvent(self, "Code3Correct");
 			self.sequence = "";
 		end
	end
	
	BroadcastEvent( self, "Key"..number );
		--Hud:AddMessage( "Key"..number );
	
	if ( self.Properties.sndSound ~= "" ) then --play sound if available
		self:PlaySound( self.Sound );
	end

	
end

-------------------------------------------------------------------------------
function Keypad:OnSave(stm)	
end

-------------------------------------------------------------------------------
function Keypad:OnLoad(stm)	 
end

-------------------------------------------------------------------------------
function Keypad:Event_Code1Correct()
end
-------------------------------------------------------------------------------
function Keypad:Event_Code2Correct()
end
-------------------------------------------------------------------------------
function Keypad:Event_Code3Correct()
end
-------------------------------------------------------------------------------
function Keypad:Event_Key0()
end
-------------------------------------------------------------------------------
function Keypad:Event_Key1()
end
-------------------------------------------------------------------------------
function Keypad:Event_Key2()
end
-------------------------------------------------------------------------------
function Keypad:Event_Key3()
end
-------------------------------------------------------------------------------
function Keypad:Event_Key4()
end
-------------------------------------------------------------------------------
function Keypad:Event_Key5()
end
-------------------------------------------------------------------------------
function Keypad:Event_Key6()
end
-------------------------------------------------------------------------------
function Keypad:Event_Key7()
end
-------------------------------------------------------------------------------
function Keypad:Event_Key8()
end
-------------------------------------------------------------------------------
function Keypad:Event_Key9()
end
-------------------------------------------------------------------------------
function Keypad:Event_Enable()
	self.Properties.bEnabled = 1;
	self:TrackColliders(1);
	BroadcastEvent(self, "Enable");
end
-------------------------------------------------------------------------------
function Keypad:Event_Disable()
	self.Properties.bEnabled = 0;
	self:TrackColliders(0);
	BroadcastEvent(self, "Disable");

	self:EnableUpdate(0); --1.1
	self:OnLeaveArea(); --1.1
end
-------------------------------------------------------------------------------

function Keypad:OnShutdown()
	self:OnLeaveArea(); --1.1
end
