--RaceTrack.
--by aarbro [aarbro@hotmail.com]


RaceTrack = {

	Editor = {
		Model="Objects/EXP/Editor/editor_racetrack.cgf",
	},

	Properties = {
		StopWatch = "",
		Laps = 1,
		sndSoundStart = "Sounds/multiplayer/hit.wav",
		sndSoundCheckPoint = "Sounds/items/cam_with_flash.wav",
		sndSoundFinish = "Sounds/weapons/pancor/jackfire1.wav",
		sndSoundError = "Sounds/items/codepad.wav",
		fSoundVolume = 255,
		HUD = {
			bDisplayLaps = 1,
			bDisplayMessages = 1,
			fMessageTime = 2,
			MessageError = "Wrong Checkpoint!",
			MessageFinish = "Race Finished",
			bKPH = 1,
			LapFont = {
				Name = "hud",
				Effect = "ammo",
				Width = 40,
				Height = 40,
				LocationX = 700,
				LocationY = 10,
				Color = {r=1,g=1,b=1,t=0.75},
				Scale = 0.4,
			},
			MessageFont = {
				Name = "default",
				Effect = "default",
				Width = 60,
				Height = 80,
				LocationX = 0,
				LocationY = 200,
				Color = {r=1,g=1,b=0,t=0.5},
			},
		},
	},
	
	racing = 0,
	laps = 0,
	stopwatch = nil, --pointer
	highscore = {},
	message_active = 0,
	message = "",
	timebefore = 0,
	lastlap = 0,
	
};
-------------------------------------------------------------------------------
function RaceTrack:OnPropertyChange()
	self:OnReset();
end
-------------------------------------------------------------------------------

function RaceTrack:OnInit()
	--load HiScore
	local file = openfile( "Levels/"..Game:GetLevelName().."/HiScore", "r");
	if ( file ~= nil ) then
		local line = read( file, "*l" );
		while ( line ~= nil ) do
			tinsert( self.highscore, line );
			line = read( file, "*l" );
		end
		closefile( file );
	end
	while ( getn( self.highscore ) > 10 ) do --remove last if more than 10
		tremove( self.highscore );
	end
	
	self:OnReset();
end
-------------------------------------------------------------------------------

function RaceTrack:OnReset()

	self:ResetTrack();
	self.racing = 0;
	self.laps = self.Properties.Laps;
	self.timebefore = 0;
	self.lastlap = 0;

	self:EnableUpdate(0);
	self.message_active = 0;
if (Hud~=nil) then
	Hud:EnableCustomHud(1); --turn on global hud updates
end
	
	if ( self.Properties.sndSoundStart ~= "" ) then
		self.sound_start=Sound:LoadSound(self.Properties.sndSoundStart);
		Sound:SetSoundVolume(self.sound_start, self.Properties.fSoundVolume);
		Sound:SetSoundLoop(self.sound_start,0);
	else
		self.sound_start = nil;
	end
	
	if ( self.Properties.sndSoundFinish ~= "" ) then
		self.sound_finish=Sound:LoadSound(self.Properties.sndSoundFinish);
		Sound:SetSoundVolume(self.sound_finish, self.Properties.fSoundVolume);
		Sound:SetSoundLoop(self.sound_finish,0);
	else
		self.sound_finish = nil;
	end
	
	if ( self.Properties.sndSoundCheckPoint ~= "" ) then
		self.sound_cp=Sound:LoadSound(self.Properties.sndSoundCheckPoint);
		Sound:SetSoundVolume(self.sound_cp, self.Properties.fSoundVolume);
		Sound:SetSoundLoop(self.sound_cp,0);
	else
		self.sound_cp = nil;
	end
	
	if ( self.Properties.sndSoundError ~= "" ) then
		self.sound_mcp=Sound:LoadSound(self.Properties.sndSoundError);
		Sound:SetSoundVolume(self.sound_mcp, self.Properties.fSoundVolume);
		Sound:SetSoundLoop(self.sound_mcp,0);
	else
		self.sound_mcp = nil;
	end
	
	--create hiscores for screen
if (Hud~=nil and _localplayer~=nil) then
  Hud:FlashObjectives();
end
  for index, value in self.highscore do
  	if type(value) ~= "number" then
if (Hud~=nil and _localplayer~=nil) then
  		Hud:PushObjective({}, value );
end
  	end
  end
	
end

-------------------------------------------------------------------------------
function RaceTrack:OnUpdate()
end
-------------------------------------------------------------------------------
function RaceTrack:OnShutDown()
end
-------------------------------------------------------------------------------
function RaceTrack:OnSave(stm)
end
-------------------------------------------------------------------------------
function RaceTrack:OnLoad(stm)
end
-------------------------------------------------------------------------------

function RaceTrack:ResetTrack() --I
	local ents = System:GetEntities();
	for idx,entity in ents do
		if (entity.type == "RaceCheckPoint" and entity.Properties.TrackName == self:GetName() ) then
			entity.active = 0;
		end
	end
end
-------------------------------------------------------------------------------
function RaceTrack:Event_ResetHiScore() --I
	remove( "Levels/"..Game:GetLevelName().."/HiScore" );
	self.highscore = {};
if (Hud~=nil and _localplayer~=nil) then
	Hud:FlashObjectives({}, "");
end
end
-------------------------------------------------------------------------------

function RaceTrack:Event_QuitRace()
	self.stopwatch:Event_Stop();
	self.stopwatch:Event_Reset();
	self:ResetTrack();
	self.racing = 0;
	self.laps = 0;
	
	self.message_active = 1;
	self.message = self.Properties.HUD.MessageFinish;
	self:SetTimer( self.Properties.HUD.fMessageTime * 1000 );
	
end
-------------------------------------------------------------------------------
function RaceTrack:OnRaceStarted(rcp)
	self.stopwatch = System:GetEntityByName( self.Properties.StopWatch );
	self.stopwatch:Event_Reset();
	self.stopwatch:Event_Show();
	self.stopwatch:Event_Start();
	self.lastlap = 0;
	self.timebefore = 0;
	if ( self.sound_start ) then
		Sound:PlaySound( self.sound_start );
	end
end
-------------------------------------------------------------------------------
function RaceTrack:OnCheckPoint(rcp)
	if ( rcp.Properties.bSilent == 0 ) then
		self.stopwatch:Event_Intermediate();
		if ( self.sound_cp ) then
			Sound:PlaySound( self.sound_cp );
		end
	end
end
-------------------------------------------------------------------------------
function RaceTrack:OnLapPoint(rcp)
	if ( rcp.Properties.bSilent == 0 ) then
		self.stopwatch:Event_Intermediate();
		if ( self.sound_cp ) then
			Sound:PlaySound( self.sound_cp );
		end
	end
	
	local timenow = System:GetCurrTime() - self.stopwatch.start_time;
	local thislap = timenow - self.timebefore;
	local lapdiff = thislap - self.lastlap;
	
	if ( self.lastlap == 0 ) then
		self.message = TimeToStr( thislap,"%02u:%02u:%02u");
	else if ( lapdiff < 0 ) then
			self.message = TimeToStr( thislap,"%02u:%02u:%02u").." (-"..TimeToStr(lapdiff,"%02d:%02d:%02d")..")";
		else
			self.message = TimeToStr( thislap,"%02u:%02u:%02u").." (+"..TimeToStr(lapdiff,"%02d:%02d:%02d")..")";
		end
	end

	self.timebefore = timenow;
	self.lastlap = thislap;

	self.message_active = 1;

	self:SetTimer( self.Properties.HUD.fMessageTime * 1000 );

end
-------------------------------------------------------------------------------

function RaceTrack:OnRaceFinished(rcp)
	self.stopwatch:Event_Stop();
	if ( self.sound_finish ) then
		Sound:PlaySound( self.sound_finish );
	end
	
	local timefin = self.stopwatch.stop_time - self.stopwatch.start_time;
	
	self.message_active = 1;
	self.message = self.Properties.HUD.MessageFinish;
	self:SetTimer( self.Properties.HUD.fMessageTime * 1000 );

	--update hiscore list
	tinsert( self.highscore, TimeToStr(timefin,"%02u:%02u:%02u") .. "  -  " .. date() ); --add result
	sort( self.highscore ); --sort results
	while ( getn( self.highscore ) > 10 ) do --remove last if more than 10
		tremove( self.highscore );
	end

	--create hiscores for screen
if (Hud~=nil and _localplayer~=nil) then
	Hud:FlashObjectives({}, "");
end
	for index, value in self.highscore do
		if type(value) ~= "number" then
if (Hud~=nil and _localplayer~=nil) then
			Hud: ective({}, value );
end
		end
	end

	--save hiscore
	local file = openfile( "Levels/"..Game:GetLevelName().."/HiScore", "w");
	if ( file ~= nil ) then
		for index, value in self.highscore do
			if type(value) ~= "number" then
				write( file, value.."\n" );
			end
		end
		closefile( file );	
	end
end



-------------------------------------------------------------------------------
function RaceTrack:OnMissedCheckPoint(rcp)
	Sound:PlaySound( self.sound_mcp );
	self.message_active = 1;
	self.message = self.Properties.HUD.MessageError;
	self:SetTimer( self.Properties.HUD.fMessageTime * 1000 );
end
-------------------------------------------------------------------------------

function RaceTrack:OnUpdateCustomHud()

	if ( self.Properties.HUD.bDisplayLaps == 1 ) then --show lap numbers
  	local lapinfo = self.Properties.Laps - self.laps + 1 .. "/" .. self.Properties.Laps;
  	if ( self.laps == 0 ) then
  		lapinfo = "FIN";
  	end
  	Game:SetHUDFont( self.Properties.HUD.LapFont.Name, self.Properties.HUD.LapFont.Effect );
  	Game:WriteHudStringFixed( 
  		self.Properties.HUD.LapFont.LocationX,
  		self.Properties.HUD.LapFont.LocationY,
  		lapinfo,
  		self.Properties.HUD.LapFont.Color.r,
  		self.Properties.HUD.LapFont.Color.g,
  		self.Properties.HUD.LapFont.Color.b,
  		self.Properties.HUD.LapFont.Color.t,
  		self.Properties.HUD.LapFont.Width,
  		self.Properties.HUD.LapFont.Height,
  		self.Properties.HUD.LapFont.Scale
  	);
	end

	if ( self.Properties.HUD.bDisplayMessages == 1 and self.message_active == 1 ) then --show timed message
  	Game:SetHUDFont( self.Properties.HUD.MessageFont.Name, self.Properties.HUD.MessageFont.Effect );
  	Game:WriteHudString( 
  		self.Properties.HUD.MessageFont.LocationX,
  		self.Properties.HUD.MessageFont.LocationY,
  		self.message,
  		self.Properties.HUD.MessageFont.Color.r,
  		self.Properties.HUD.MessageFont.Color.g,
  		self.Properties.HUD.MessageFont.Color.b,
  		self.Properties.HUD.MessageFont.Color.t,
  		self.Properties.HUD.MessageFont.Width,
  		self.Properties.HUD.MessageFont.Height,
  		1
  	);
	end

end
-------------------------------------------------------------------------------

function RaceTrack:OnTimer()
	self.message_active = 0;
end
