--An animated slidegate to prevent all passage to an area.
--by aarbro [aarbro@hotmail.com]

Slidegate = {
	
	Properties = {
		objSupport = "Objects/EXP/slidegate_support.cgf",
		objSlidegate = "Objects/EXP/slidegate_gate.cgf",
		fSpeed = 0.5, --max speed in meters per second
		fInertia = 0.2, --distance at which slowdown starts
		fPosClosed = 0, --closed position from the center of the support
		fPosOpened = 5, --opened position from the center of the support
		vectorBBoxMin = { x=-8, y=-0.5, z=0 }, --must cover the whole animation cycle
		vectorBBoxMax = { x=6, y=0.3, z=3 },
		bOpened = 0, --state at start
		sndSound = "Sounds/building/rattlyairlp.wav",
		bEnabled = 1,
	},
	
	currPos = { x=0, y=0, z=0 },
	direction = 0,
};



function Slidegate:OnInit()
	self:OnReset();
end

function Slidegate:OnPropertyChange()
	self:OnReset();
end


function Slidegate:OnReset()

	self:LoadObject( self.Properties.objSupport, 0, 0 );
	self:CreateStaticEntity(100,-1);
	self:DrawObject( 0, 1 );
	
	local helperPos=self:GetHelperPos("gatepos", 1);
	self:LoadObject( self.Properties.objSlidegate, 1, 0);
	self:CreateStaticEntity(30,-1);
	self:SetObjectPos( 1 , helperPos );
	self:DrawObject( 1, 1 );
	
	if ( self.Properties.bOpened == 1 ) then
		self.currPos.x = self.Properties.fPosOpened;
	else
		self.currPos.x = self.Properties.fPosClosed;
	end
	
	self:SetObjectPos( 1, self.currPos );
	
	self.MotorSound=Sound:Load3DSound(self.Properties.sndSound);
	Sound:SetSoundVolume(self.MotorSound, 128);
	Sound:SetMinMaxDistance(self.MotorSound, 3, 20);
	Sound:SetSoundLoop(self.MotorSound, 1);

	self:SetBBox( self.Properties.vectorBBoxMin, self.Properties.vectorBBoxMax );

	self:EnableUpdate(0);
	
end


function Slidegate:OnUpdate(dt)

	local slowdown = 1;
	if ( ( self.Properties.fPosOpened - self.currPos.x ) < self.Properties.fInertia and self.direction == 1 ) then
		slowdown = ( self.Properties.fPosOpened - self.currPos.x ) / self.Properties.fInertia;
	elseif ( ( self.currPos.x - self.Properties.fPosClosed ) < self.Properties.fInertia and self.direction == -1) then
		slowdown = ( self.currPos.x - self.Properties.fPosClosed ) / self.Properties.fInertia;
	end
	if ( slowdown < 0.01 ) then
			slowdown = 0.01;
	end
	
	self.currPos.x = self.currPos.x + self.Properties.fSpeed * dt * self.direction * slowdown;

	if ( self.currPos.x > self.Properties.fPosOpened ) then
		self.currPos.x = self.Properties.fPosOpened;
		self.direction = 0;
		self:EnableUpdate(0);
		self.Properties.bOpened = 1;
		BroadcastEvent(self, "Opened");
		Sound:StopSound( self.MotorSound );
	end
	if ( self.currPos.x < self.Properties.fPosClosed ) then
		self.currPos.x = self.Properties.fPosClosed;
		self.direction = 0;
		self:EnableUpdate(0);
		self.Properties.bOpened = 0;
		BroadcastEvent(self, "Closed");
		Sound:StopSound( self.MotorSound );
	end

	self:SetObjectPos( 1, self.currPos );

end


function Slidegate:Event_Open()
	if ( self.Properties.bEnabled == 1 ) then
  	self:EnableUpdate(1);
  	self.direction = 1;
  	BroadcastEvent(self, "Opening");
  	self:PlaySound( self.MotorSound );
	end
end


function Slidegate:Event_Close()
	if ( self.Properties.bEnabled == 1 ) then
  	self:EnableUpdate(1);
  	self.direction = -1;
  	BroadcastEvent(self, "Closing");
  	self:PlaySound( self.MotorSound );
	end
end


function Slidegate:Event_Opened()
end

function Slidegate:Event_Closed()
end

function Slidegate:Event_Opening()
end

function Slidegate:Event_Closing()
end

function Slidegate:OnSave(stm)
	stm:WriteFloat( self.currPos.x );
	stm:WriteInt( self.direction );
end

function Slidegate:OnLoad(stm)
	self.currPos.x = stm:ReadFloat();
	self.direction = stm:ReadInt();
	if ( self.direction == 0 ) then
		self:EnableUpdate(0);
	elseif ( self.Properties.bEnabled == 1 ) then
		self:Event_Enable();
	end
end

function Slidegate:Event_Enable()
	self.Properties.bEnabled = 1;
	self:EnableUpdate(1);
	if ( self.direction ~= 0 ) then
  	self:PlaySound( self.MotorSound );
  end
 	BroadcastEvent(self, "Enable");
end

function Slidegate:Event_Disable()
	self.Properties.bEnabled = 0;
	self:EnableUpdate(0);
	Sound:StopSound( self.MotorSound );
 	BroadcastEvent(self, "Disable");
end
