--The Teleporter can transport an entity to another Teleporter at an instant.
--by aarbro [aarbro@hotmail.com]

Teleporter = {

	type = "TeleporterEntity",
	
	Editor = {
		Model="Objects/EXP/Editor/editor_teleport.cgf",
	},

	Properties = {
		bEnabled = 1,
		vectorBBox = {x=1,y=1,z=2},
		objModel = "Objects/EXP/teleporter.cgf", --model to use, empty for none

		sDestination = "", --Other teleporter name, empty for receiver.
		bInstantActivation = 1, --if 0, Energize event must be sent when player is inside BBox
		bKeepRotation = 0, --if 0, the arrow at the destination determine rotation

		bOnlyPlayer=1, --Affect sender only
		bOnlyMyPlayer=0,
		bOnlyAI = 0,
		bInVehicleOnly=0,
		bOnFootOnly=0,
		bAnyEntity=0,

		fParticleScale = 1,			--scale of the spark particle
		ParticleEffect = "bullet.hit_bullseye.a", --empty for no particles
		fWhiteOutTime = 0.2, --disable with -1
		Sound = {
			sndFile = "Sounds/building/cryo1.wav", --empty for no sound
			fVolume = 255,
			fMinDistance = 3,
			fMaxDistance = 1000,
		},
	},
	
	inside = 0,
	tele_player = 0,
	just_arrived = 0,
};
-------------------------------------------------------------------------------
function Teleporter:OnPropertyChange()
	self:OnReset();
end
-------------------------------------------------------------------------------

function Teleporter:OnInit()
	self:OnReset();
end
-------------------------------------------------------------------------------

function Teleporter:OnReset()

	if ( self.Properties.objModel ~= "" ) then
		self:LoadObject( self.Properties.objModel, 0, 1 );
		self:CreateStaticEntity( 100, -1 );
		self:DrawObject( 0, 1 );
	end

	local Min = { x=-self.Properties.vectorBBox.x/2, y=-self.Properties.vectorBBox.y/2, z=-self.Properties.vectorBBox.z/2 };
	local Max = { x=self.Properties.vectorBBox.x/2, y=self.Properties.vectorBBox.y/2, z=self.Properties.vectorBBox.z/2 };
	self:SetBBox( Min, Max );

	if (self.Properties.sndSound ~= "") then
		self.Sound=Sound:Load3DSound(self.Properties.Sound.sndFile);
		Sound:SetSoundVolume(self.Sound, self.Properties.Sound.fVolume);
		Sound:SetMinMaxDistance(self.Sound, self.Properties.Sound.fMinDistance, self.Properties.Sound.fMaxDistance);
		Sound:SetSoundLoop(self.Sound,0);
	else
		self.Sound = nil;
	end
	
	self:TrackColliders(1);
	self:EnableUpdate(0);
	
	if ( self.Properties.bEnabled == 1 ) then
		self:Event_Enable();
	else
		self:Event_Disable();
	end

end
-------------------------------------------------------------------------------

function Teleporter:OnShutDown()
end
-------------------------------------------------------------------------------

function Teleporter:OnUpdate()
end
-------------------------------------------------------------------------------

function Teleporter:OnSave(stm)
end
-------------------------------------------------------------------------------

function Teleporter:OnLoad(stm)
end
-------------------------------------------------------------------------------

function Teleporter:OnEnterArea( player )
 	
 	self.inside = 1;
	self.tele_player = player;

 	if ( self.Properties.bInstantActivation == 1 and self.just_arrived == 0 ) then
 		self:OnEnergize();
 	end
 	
end
-------------------------------------------------------------------------------
function Teleporter:OnLeaveArea( player )
	self.inside = 0;
	self.just_arrived = 0;

end
-------------------------------------------------------------------------------
function Teleporter:OnEnergize( )
 	if (  self.Properties.bEnabled == 0 ) then
 		return;
 	end
 	
	if ( self:IsValidSource( self.tele_player ) == 0 ) then
		return;
	end

	local ents = System:GetEntities(); --check all entities on the map
	for idx,dest in ents do
		if (dest.classname == "Teleporter" ) then --correct entity type
			if ( dest:GetName() == self.Properties.sDestination ) then --we have a destination

				dest:Event_Arriving();
				
				local destPos = dest:GetPos();
				destPos.z=destPos.z-(dest.Properties.vectorBBox.z/2);
				
				self.tele_player:SetPos(destPos); --move player
				if ( dest.Properties.bKeepRotation == 0 ) then
					self.tele_player:SetAngles( dest:GetAngles() ); --rotate player to match the destination
				end

				self:OnShowFX(); --show effects here
				dest:OnShowFX(); --show effects at destination

				BroadcastEvent(dest, "Arriving");
				BroadcastEvent(self, "Departing");
			end
		end
	end

end
-------------------------------------------------------------------------------
function Teleporter:OnShowFX()
	--create particle
	if ( self.Properties.ParticleEffect ~= "" ) then
		local dir = self:GetDirectionVector();
		Particle:SpawnEffect( self:GetPos(), dir, self.Properties.ParticleEffect, self.Properties.fParticleScale );
	end
	--whiteout effect in HUD if it's the local player that teleports now
	if ( self.Properties.fWhiteOutTime ~= -1 and self.tele_player == _localplayer ) then
		local vec = _localplayer:GetPos();
		System:ProjectToScreen(vec);
		System:SetScreenFx("FlashBang", 1);
		System:SetScreenFxParamFloat("FlashBang", "FlashBangFlashPosX", vec.x);
		System:SetScreenFxParamFloat("FlashBang", "FlashBangFlashPosY", vec.y);
		System:SetScreenFxParamFloat("FlashBang", "FlashBangTimeScale", self.Properties.fWhiteOutTime );
	end
	--play sound
	if ( self.Sound ) then
		Sound:SetSoundPosition( self.Sound, self:GetPos() );
		Sound:PlaySound( self.Sound );
	end
	

end
-------------------------------------------------------------------------------

function Teleporter:Event_Enable()
	self:TrackColliders(1);
	BroadcastEvent(self, "Enable");
	self.Properties.bEnabled = 1;
end
-------------------------------------------------------------------------------

function Teleporter:Event_Disable()
	self:TrackColliders(0);
	BroadcastEvent(self, "Disable");
	self.Properties.bEnabled = 0;
	self.inside = 0;
end
-------------------------------------------------------------------------------
function Teleporter:Event_Energize()
	if ( self.Properties.bInstantActivation == 0 and self.inside == 1 and self.just_arrived == 0 ) then
		self:OnEnergize( );
		BroadcastEvent(self, "Energize");
	end
end
-------------------------------------------------------------------------------
function Teleporter:Event_Departing()
	self.just_arrived = 0;
end
-------------------------------------------------------------------------------
function Teleporter:Event_Arriving()
	self.just_arrived = 1;
end
-------------------------------------------------------------------------------

function Teleporter:IsValidSource( entity )
	-- Allow anything
 	if ( self.Properties.bAnyEntity == 1 ) then
		return 1;
	elseif ( entity.type ~= "Player" ) then
		return 0;
	end
	-- Only players
	if (self.Properties.bOnlyPlayer == 1 and entity.type ~= "Player") then
		return 0;
	end
	-- Only affects AI
	if (self.Properties.bOnlyAI == 1 and entity.ai == nil) then
		return 0;
	end
	-- Only local player.
	if (self.Properties.bOnlyMyPlayer == 1 and entity ~= _localplayer) then
		return 0;
	end
	-- Only player in vehicle
	if (self.Properties.bInVehicleOnly == 1 and not entity.theVehicle) then
		return 0;
	end
	-- Never player in vehicle
	if (self.Properties.bOnFootOnly == 1 and entity.theVehicle) then
		return 0;
	end
	-- ignore dead player
	if (entity.cnt.health <= 0) then 
		return 0;
	end
	
	return 1;
end

-------------------------------------------------------------------------------




