--VehicleRealism.
--by aarbro [aarbro@hotmail.com]


VehicleRealism = {

	Editor = {
		Model="Objects/EXP/editor/editor_car.cgf",
	},

	Properties = {
		bEnabled = 1,
    VehicleName = "",
    TerrainModifier = { --percentage of full power remaining
    	bActive = 1,
    	concrete = 1.0,
    	rock = 1.0,
    	pebble = 0.8,
    	sand_dry = 0.6,
    	sand_wet = 0.7,
    	grass = 0.5,
    	grass_dry = 0.5,
    	grass_tall = 0.4,
    	grass_wet = 0.3,
    	leaves = 0.3,
    	mud = 0.1,
    	swamp = 0.05,
    },
    Fuel = {
    	bActive = 1,
    	fTankMax = 75, --litres
    	fTankCurrent = 75, --litres
    	fConsumption = 1, --litres/minute..
    	fRPM = 5000, --       ..at this RPM
    	fRefuelAmount = 1, --how much (in liters) to fill per event call
    },
    DamageModifier = { 
    	bActive = 1,
    	fAtDamLight = 50,
    	fAtDamMedium = 25,
    	fAtDamSevere = 10,
    	fEngEffLight = 0.9,
    	fEngEffMedium = 0.5,
    	fEngEffSevere = 0.2,
    	fRepairAmount = 1, --how much (in %) to repair per event call
    },
    Powerup = { 
	    Nitro = { 
    		Key = "n",
    	},
    },
    HornSound = { 
   		bActive = 1,
   		sndSound = "Sounds/EXP/horn.wav",
			fSoundVolume = 255,
			Key = "rctrl",
    },
    
	},

	original_pow = nil,
	old_mod_eff = -1,
	
	
};
-------------------------------------------------------------------------------
function VehicleRealism:OnPropertyChange()
	self:OnReset();
end
-------------------------------------------------------------------------------

function VehicleRealism:OnInit()
	self.nitro = {};
	self.nitro.available = 0;
	self.nitro.active = 0;
	self.nitro.effect = 0; --next nitro, if picked while one in use already 
	self.nitro.effect_now = 0; --current nitro in use
	self.nitro.time = 0;
	self:OnReset();
end
-------------------------------------------------------------------------------
function VehicleRealism:OnShutDown()
	--reset to the original power of the vehicle when exiting in Sandbox
	local vehicle = System:GetEntityByName( self.Properties.VehicleName );
	if ( vehicle ) then
		if ( vehicle.IsCar == 1 ) then 
			vehicle.CarDef.engine_power = self.original_pow;
		elseif ( vehicle.IsBoat == 1 ) then 
			vehicle.boat_params.Speedv = self.original_pow;
		end
	end
end
-------------------------------------------------------------------------------

function VehicleRealism:OnReset()

	self:EnableUpdate(1);
	if ( self.Properties.bEnabled == 1 ) then
		self:Event_Enable();
	else
		self:Event_Disable();
	end

	if ( self.Properties.HornSound.sndSound ~= "" ) then
		self.sound=Sound:LoadSound(self.Properties.HornSound.sndSound);
		Sound:SetSoundVolume(self.sound, self.Properties.HornSound.fSoundVolume);
		Sound:SetSoundLoop(self.sound,0);
	else
		self.sound = nil;
	end

	self.slowdown = 0;
	self.nitro.available = 0;
	self.nitro.active = 0;
if (Hud~=nil) then
	Hud:EnableCustomHud(1); --turn on global hud updates
end
	self.nitro_texture = System:LoadTexture( "Objects/EXP/nitro_icon.dds" );
	
end

-------------------------------------------------------------------------------
function VehicleRealism:OnUpdate(dt)

	--save the original power of the vehicle when run for the first time
	if ( not self.original_pow ) then
		local tmp_veh = System:GetEntityByName( self.Properties.VehicleName );
		if ( tmp_veh ) then
			if ( tmp_veh.IsCar == 1 ) then 
				self.original_pow = tmp_veh.CarDef.engine_power;
			elseif ( tmp_veh.IsBoat == 1 ) then 
				self.original_pow = tmp_veh.boat_params.Speedv;
			end
		end
	end
	
	if ( not _localplayer.theVehicle ) then --player is not in a vehicle
		return;
	end

	if ( self.Properties.VehicleName ~= _localplayer.theVehicle:GetName() ) then --wrong vehicle
		return;
	end

	local vehicle = _localplayer.theVehicle;
 	local mod_eff = 1;

 	local Key = Input:GetXKeyPressedName();
	
	-----------
	--TERRAIN--
	-----------
	if ( self.Properties.TerrainModifier.bActive == 1 and vehicle.IsCar == 1 ) then
  	--define number of wheels on vehicle
  	local wheeln = 4;
  	if (vehicle.CarDef.wheel_num) then
  		wheeln = vehicle.CarDef.wheel_num;
  	end
  	
  	--check material under each wheel
  	local terr_eff = 0;
  	for i=0,wheeln-1, 1 do
  		local wheelstats = vehicle.cnt:GetWheelStatus( i );
  		local material = Game:GetMaterialBySurfaceID(wheelstats.surfaceIndex);
  		local wheelmat = 1.0;
  		for i, v in self.Properties.TerrainModifier do --find terrain modifier
  			if ( i == material.type ) then
  				--Hud:AddMessage( v.." - "..i, 5 );
  				wheelmat = v;
  			end
  		end
  		terr_eff = terr_eff + wheelmat / wheeln;
		end
		mod_eff = terr_eff;
	end

	--Hud:AddMessage( mod_eff, 5 );
	--Hud:AddMessage( vehicle.fEngineHealth, 5 );

	----------
	--DAMAGE--
	----------
	if ( self.Properties.DamageModifier.bActive == 1 ) then
  	if ( vehicle.fEngineHealth < self.Properties.DamageModifier.fAtDamSevere ) then
  		mod_eff = mod_eff * self.Properties.DamageModifier.fEngEffSevere;
  	elseif ( vehicle.fEngineHealth < self.Properties.DamageModifier.fAtDamMedium ) then
  		mod_eff = mod_eff * self.Properties.DamageModifier.fEngEffMedium;
  	elseif ( vehicle.fEngineHealth < self.Properties.DamageModifier.fAtDamLight ) then
  		mod_eff = mod_eff * self.Properties.DamageModifier.fEngEffLight;
  	end
	end

	--------
	--FUEL--
	--------
	if ( self.Properties.Fuel.bActive == 1 ) then
		local rev = 0;
		if ( vehicle.IsCar == 1 ) then 
			local vstatus = _localplayer.theVehicle.cnt:GetVehicleStatus();
			rev = vstatus.engineRPM / 10;
		elseif ( vehicle.IsBoat == 1 ) then 
			rev = vehicle.cnt:GetVehicleVelocity();
		end
		--Hud:AddMessage( format("%08u",rev), 5 );
		local cons = ( self.Properties.Fuel.fConsumption * dt / 60 ) * (rev / self.Properties.Fuel.fRPM );
		self.Properties.Fuel.fTankCurrent = self.Properties.Fuel.fTankCurrent - cons;
		if ( self.Properties.Fuel.fTankCurrent < 0 ) then
			self.Properties.Fuel.fTankCurrent = 0;
		end
		--Hud:AddMessage( cons, 5 );

		if ( self.Properties.Fuel.fTankCurrent <= 0 ) then
			mod_eff = 0;
			if ( _localplayer.theVehicle ) then
				if ( _localplayer.theVehicle == vehicle ) then
					Hud.label = "Out of fuel!";
				end
			end
		end
	end

	-----------
	--POWERUP--
	-----------
 	if ( self.nitro.available == 1 ) then
 		if ( tostring(Key) == self.Properties.Powerup.Nitro.Key ) then
			self.nitro.active = 1;
			self.nitro.available = 0;
			self.nitro.effect_now = self.nitro.effect;
			self:SetTimer( self.nitro.time * 1000 );
 		end
 	end
	if ( self.nitro.active == 1 ) then
		mod_eff = mod_eff * ( self.nitro.effect_now + 1 );
	end
		
	-------------
	--HORNSOUND--
	-------------
  if ( self.Properties.HornSound.bActive == 1 ) then
  	if ( tostring(Key) == self.Properties.HornSound.Key ) then
 			Sound:PlaySound( self.sound );
  	end
  end

	-----------------------
	--APPLY ALL MODIFIERS--
	-----------------------
	if ( self.old_mod_eff ~= mod_eff ) then --new power, change physics
		self.old_mod_eff = mod_eff;
		if ( vehicle.IsCar == 1 ) then
			vehicle.CarDef.engine_power = self.original_pow * mod_eff;
			vehicle:SetPhysicParams(PHYSICPARAM_VEHICLE, vehicle.CarDef);
		elseif ( vehicle.IsBoat == 1 ) then 
			vehicle.boat_params.Speedv = self.original_pow * mod_eff;
			vehicle.cnt:SetWaterVehicleParameters(vehicle.boat_params);
		end
	end
	
end
-------------------------------------------------------------------------------
function VehicleRealism:OnRepair() --repair 1 unit 
	local vehicle = System:GetEntityByName(self.Properties.VehicleName);
	vehicle.fEngineHealth = vehicle.fEngineHealth + self.Properties.DamageModifier.fRepairAmount;
	if ( vehicle.fEngineHealth >= 100 ) then
		vehicle.fEngineHealth = 100;
	end
	vehicle.cnt:SetVehicleEngineHealth(vehicle.fEngineHealth);
end
-------------------------------------------------------------------------------
function VehicleRealism:OnRefuel() --fill 1 unit
	self.Properties.Fuel.fTankCurrent = self.Properties.Fuel.fTankCurrent + self.Properties.Fuel.fRefuelAmount;
	if ( self.Properties.Fuel.fTankCurrent >= self.Properties.Fuel.fTankMax ) then
		self.Properties.Fuel.fTankCurrent = self.Properties.Fuel.fTankMax;
	end
end
-------------------------------------------------------------------------------
function VehicleRealism:Event_Enable()
	self.Properties.bEnabled = 1;
	self:EnableUpdate(1);
	BroadcastEvent(self, "Enable");
end
-------------------------------------------------------------------------------
function VehicleRealism:Event_Disable()
	self.Properties.bEnabled = 0;
	self:EnableUpdate(0);
	BroadcastEvent(self, "Disable");
end
-------------------------------------------------------------------------------
function VehicleRealism:OnTimer()
	self.nitro.active = 0; --ran out of nitro
end
-------------------------------------------------------------------------------
function VehicleRealism:OnUpdateCustomHud()

	if ( not _localplayer.theVehicle ) then --player is not in a vehicle
		return;
	end

	if ( self.Properties.VehicleName ~= _localplayer.theVehicle:GetName() ) then --wrong vehicle
		return;
	end

	if ( self.nitro.available == 1 ) then --icon for nitro available
		System:DrawImageColor(
			self.nitro_texture,
			725,--self.Properties.Dial.LocationX,
			400,--self.Properties.Dial.LocationY,
			100,--self.Properties.Dial.Width,
			100,--self.Properties.Dial.Height,
			4,--self.Properties.Dial.Blending,
			1,--self.Properties.Dial.ColorR,
			1,--self.Properties.Dial.ColorG,
			1,--self.Properties.Dial.ColorB,
			1--self.Properties.Dial.Transparency
		);
	end
	if ( self.nitro.active == 1 ) then --show nitro is active now
		System:DrawImageColor(
			self.nitro_texture,
			250,--self.Properties.Dial.LocationX,
			150,--self.Properties.Dial.LocationY,
			300,--self.Properties.Dial.Width,
			300,--self.Properties.Dial.Height,
			4,--self.Properties.Dial.Blending,
			1,--self.Properties.Dial.ColorR,
			1,--self.Properties.Dial.ColorG,
			1,--self.Properties.Dial.ColorB,
			0.25--self.Properties.Dial.Transparency
		);
	end

end
