------------------------------------------------------------------------
--	Helper functions for GameRules implementation
--
--	created by Alberto
--  modified by MartinM
-------------------------------------------------------------------------
Script:LoadScript("SCRIPTS/Server/ServerCmds.lua")
Script:LoadScript("SCRIPTS/GL/GLcommands.lua");
Script:LoadScript("SCRIPTS/GL/GLplayertrack.lua");
-- teams:
--
-- players
-- spectators

-- If someone in script needs to know if we are running multiplayer or single player game.
GameRules.bMultiplayer = 1;
GameRules.bSingleplayer = nil;
GameRules.bIsTeamBased = nil;

CGS_INPROGRESS = 0;
CGS_COUNTDOWN = 1;
CGS_PREWAR = 2;
CGS_INTERMISSION = 3;

GameRules.states_map=
{
	[CGS_INPROGRESS] ="INPROGRESS",
	[CGS_COUNTDOWN] ="COUNTDOWN",
	[CGS_PREWAR] ="PREWAR",
	[CGS_INTERMISSION] = "INTERMISSION",
}
GameRules.TeamText = {red = "@TeamNameRed", blue = "@TeamNameBlue", players = "@TeamNamePlayers", spectators = "@TeamNameSpectators",};

GameRules.RespawnPoints={};										-- {[1]={x=,y=,z=,xA=,yA=,zA=,name=,lastUsedTime=},} use RecreateTableOfRespawnPoints(), lastUsedTime might be nil
GameRules.idCurrentMissionObject=nil;					-- used to make only one MissionObject entity active at a time
--3up killspree sound start
GameRules.Headshot=Sound:LoadSound("Sounds/Mod/KillVoice/headshot.wav");
GameRules.Domination=Sound:LoadSound("Sounds/Mod/KillVoice/dominating.wav");
GameRules.KillvoiceSound= {
	Sound:LoadSound("Sounds/Mod/KillVoice/doublekill.wav"),
	Sound:LoadSound("Sounds/Mod/KillVoice/multikill.wav"),
	Sound:LoadSound("Sounds/Mod/KillVoice/rampage.wav"),
	Sound:LoadSound("Sounds/Mod/KillVoice/killingspree.wav"),
	Sound:LoadSound("Sounds/Mod/KillVoice/ultrakill.wav"),
	Sound:LoadSound("Sounds/Mod/KillVoice/unstoppable.wav"),
	Sound:LoadSound("Sounds/Mod/KillVoice/godlike.wav")

};
--3up killspree sound end

GameRules.idScoreboard=nil;										-- scoreboard entity id

if not GameRules.InitialPlayerStatistics then
	GameRules.InitialPlayerStatistics = {};			-- used for MP statistics but can be used for SP as well
end


GameRules.InitialPlayerStatistics["nKill"]=0;					-- this entry can be used by MPStatistics:AddStatisticsDataEntity();
GameRules.InitialPlayerStatistics["nSelfKill"]=0;			-- this entry can be used by MPStatistics:AddStatisticsDataEntity();
GameRules.InitialPlayerStatistics["nBulletShot"]=0;		-- this entry can be used by MPStatistics:AddStatisticsDataEntity();
GameRules.InitialPlayerStatistics["nBulletHit"]=0;			-- this entry can be used by MPStatistics:AddStatisticsDataEntity();


--3up advanced stats start

GameRules.InitialPlayerStatistics["nHeadshot"]=0;			-- this entry can be used by MPStatistics:AddStatisticsDataEntity();
GameRules.InitialPlayerStatistics["nKnife"]=0;			-- this entry can be used by MPStatistics:AddStatisticsDataEntity();
GameRules.InitialPlayerStatistics["nHumiliate"]=0;			-- this entry can be used by MPStatistics:AddStatisticsDataEntity();
GameRules.InitialPlayerStatistics["nExplode"]=0;			-- this entry can be used by MPStatistics:AddStatisticsDataEntity();
GameRules.InitialPlayerStatistics["nTrap"]=0;			-- this entry can be used by MPStatistics:AddStatisticsDataEntity();
GameRules.InitialPlayerStatistics["nLame"]=0;			-- this entry can be used by MPStatistics:AddStatisticsDataEntity();
GameRules.InitialPlayerStatistics["nSirenKill"]=0;			-- this entry can be used by MPStatistics:AddStatisticsDataEntity();
GameRules.InitialPlayerStatistics["nKillKing"]=0;			-- this entry can be used by MPStatistics:AddStatisticsDataEntity();

GameRules.InitialPlayerStatistics["nHeal"]=0;			-- this entry can be used by MPStatistics:AddStatisticsDataEntity();
GameRules.InitialPlayerStatistics["nHealOther"]=0;			-- this entry can be used by MPStatistics:AddStatisticsDataEntity();
GameRules.InitialPlayerStatistics["nHealKing"]=0;			-- this entry can be used by MPStatistics:AddStatisticsDataEntity();

--3up advanced stats end



-- e.g. GameRules.ClientCommandTable["CPC"]=function(...
if not GameRules.ClientCommandTable then
	GameRules.ClientCommandTable={};				-- used from OnClientCmd() to process commands on the server from the client
end

--[GL playertracking]
GLplayerTrack:Init();
--[end GL]

--------------------------------------------------------------------
-- Return Server Rules Table from 1..n
-- Each Rule is a table from 1..2 with 1=rulename 2=value
--------------------------------------------------------------------
function GameRules:GetServerRules()
	function PushRule(RuleTable, Rule)
		tinsert(RuleTable, {Rule, getglobal(Rule),});
	end

	local Rules = {};
	PushRule(Rules, "gr_ScoreLimit");
	PushRule(Rules, "gr_TimeLimit");
	PushRule(Rules, "gr_DamageScale");
	PushRule(Rules, "gr_HeadshotMultiplier");
	PushRule(Rules, "gr_RespawnTime");
	PushRule(Rules, "gr_PrewarOn");
	PushRule(Rules, "gr_DropFadeTime");
	PushRule(Rules, "gr_FriendlyFire");
	PushRule(Rules, "gr_MinTeamLimit");
	PushRule(Rules, "gr_MaxTeamLimit");
	PushRule(Rules, "gr_InvulnerabilityTimer");	
	PushRule(Rules, "gr_KingTimer");	
	PushRule(Rules, "gr_KingTimerUpd");	
	PushRule(Rules, "gr_currFlag");
	PushRule(Rules, "gr_stopwatch");

	PushRule(Rules, "gr_Disable_SniperRL");
	PushRule(Rules, "gr_MaxSniperLimit");									
	PushRule(Rules, "gr_Announce1");
	PushRule(Rules, "gr_AnnounceOn1");											
	PushRule(Rules, "gr_Announce2");										
	PushRule(Rules, "gr_AnnounceOn2");		

	PushRule(Rules, "gr_AnnoJoin1");
	PushRule(Rules, "gr_AnnoJoin2");									
	PushRule(Rules, "gr_AnnoJoin3");									
	PushRule(Rules, "gr_Lockdown");										
	PushRule(Rules, "gr_Lockdowntime");										
	PushRule(Rules, "gr_MaxDeathLimit");
	PushRule(Rules, "gr_HighPingKick");
	PushRule(Rules, "gr_HighPing");


	PushRule(Rules, "gr_NextMap");
	PushRule(Rules, "gr_DedicatedServer");
	
	PushRule(Rules, "sv_punkbuster");

	PushRule(Rules, "gr_classdef");

--	PushRule(Rules, "p_jump_force");
--	PushRule(Rules, "p_jump_run_d");
--	PushRule(Rules, "p_jump_run_h");
--	PushRule(Rules, "p_jump_run_time");
--	PushRule(Rules, "p_jump_walk_d");
--	PushRule(Rules, "p_jump_walk_h");
--	PushRule(Rules, "p_jump_walk_time");
	PushRule(Rules, "r_ShowLines");
	-- speed hack --
	PushRule(Rules, "gr_sh_enable");
	PushRule(Rules, "gr_sh_walk");
	PushRule(Rules, "gr_sh_run");
	-- speed hack end --
	return Rules;
end


-------------------------------------------------------------------------------
--
function GetSlotPlayer(slot)
    return System:GetEntity(slot:GetPlayerId());
end

-------------------------------------------------------------------------------
--
function GameRules:GetPlayerScoreInfo(ServerSlot, Stream)
	Stream:WriteByte(ServerSlot:GetId());
	Stream:WriteShort(ServerSlot:GetPing());
end

-------------------------------------------------------------------------------
--
function DeletePlayer(id)
--	System:Log("DeletePlayer("..tostring(id)..")");
	if(id and id~=0)then
		Server:RemoveFromTeam(id);
		Server:RemoveEntity(id);
	end
end

-------------------------------------------------------------------------------
--
function GetPlayers()
	local players={}
	local slots=Server:GetServerSlotMap()
	for i, slot in slots do
	    local ent = System:GetEntity(slot:GetPlayerId());
		if ent and ent.type=="Player" then
		   players[ent.id]=ent;
		end
	end
	return players;
end



-------------------------------------------------------------------------------
--
function GameRules:Restart(x, quiet)
	local rrtime = 1;

	if(tonumber(x))then						-- can be converted to a number
		rrtime = tonumber(x);
	end
	
	self.restartbegin = _time;
	self.restartend = _time + rrtime;
	




	if (not quiet) then
		Server:BroadcastText("@GameRestartIn $3"..rrtime.."$o @GameStartingInSeconds", 1);
	end
end

-------------------------------------------------------------------------------
--
function GameRules:SetPlayerTimeLimit(Slot)
	local bReady = Slot:IsContextReady();
	
	if (bReady and tonumber(bReady) ~= 0) then
		local dam=format("%.2f", tonumber(self.timelimit));

		Slot:SendCommand("UTL "..dam);
	end
end

-------------------------------------------------------------------------------
--
--function GameRules:SetPlayerKingTimeLimit(Slot, kingtimer)
--	local dam=format("%.2f", tonumber(kingtimer));
--	Slot:SendCommand("UKL "..dam);
--end


-------------------------------------------------------------------------------
--
function GameRules:UpdateTimeLimit(timelimit)
	self.timelimit = timelimit;
	
	local SlotMap = Server:GetServerSlotMap();
	
	for i, Slot in SlotMap do
		self:SetPlayerTimeLimit(Slot);
--		if (tonumber(getglobal("gr_tMode")) == 3) then
--			local player=GetSlotPlayer(Slot);
--			self:SetPlayerKingTimeLimit(Slot, player.cnt.KingTimer);
--		end
	end
end

-------------------------------------------------------------------------------
-- force scoreboard on all players
function GameRules:ForceScoreBoard(yes)
	local SlotMap = Server:GetServerSlotMap();
	
	for i, Slot in SlotMap do
		Game:ForceScoreBoard(Slot:GetPlayerId(), yes);

--3up clear death start
		if (tonumber(getglobal("gr_MaxDeathLimit")) > 0) then
			local newent=GetSlotPlayer(Slot);
			if newent then
				newent.cnt.deaths = 0;
			end
		end
		if (tonumber(getglobal("gr_tMode")) == 3) then
			local newent=GetSlotPlayer(Slot);
			if newent then
				newent.cnt.vip = 2;
				Slot:SendCommand("FLG V");
				--Slot:SendCommand("UKL");
				newent.cnt.KingTimer = tonumber(getglobal("gr_KingTimer"));
			end
		end


--3up clear death end
	end
end

-------------------------------------------------------------------------------
-- multicast the gamestate (one of the CGS_* above)
function GameRules:NewGameState(state)
	local slots = Server:GetServerSlotMap();
	
	for i, slot in slots do
		slot:SetGameState(state, _time-self.mapstart);
  end
  
--  System:Log("self:GotoState("..self.states_map[state]..");");
 	self:GotoState(self.states_map[state]);
end

-------------------------------------------------------------------------------
-- stats for respawning
--function GameRules:GetInitialPlayerProperties(server_slot)
--	return self.InitialPlayerProperties;
--end


-------------------------------------------------------------------------------
-- spawn a new player and put it in back in a team
-- custom properties can be nil(if the mod is not class based)
function GameRules:SpawnPlayer(server_slot,classid,team,custom_properties)

	--create a new player entity(in this MOD it always connect as spectator)
--	if(team)then
--		System:Log("GameRules:SpawnPlayer team="..team)
--	end
	
--3up highpingkick
	if (toNumberOrZero(getglobal("gr_HighPingKick")) == 1) then
		if ((server_slot:GetPing() * 2) > toNumberOrZero(getglobal("gr_HighPing"))) then
			Server:BroadcastText(server_slot:GetName().." was kicked because of a high ping.");
			GameRules:KickSlot(server_slot);
			return;
		end
	end
--3up highpingkick



	
	local locInitialPlayerProperties=self:GetInitialPlayerProperties(server_slot);

--3up class limit	start
	if locInitialPlayerProperties then
		if locInitialPlayerProperties.sPlayerClass then
			local class = locInitialPlayerProperties.sPlayerClass;
			


--			local maxsnipers = toNumberOrZero(getglobal("gr_MaxSniperLimit"));
--		  	server_slot:SendText("classid "..class);
--			if team~="spectators" and strlower(class)=="sniper" and self:GetSniperCount(team)>=maxsnipers then
--			if strlower(class)=="sniper" and self:GetSniperCount(team)>=maxsnipers then
--			  	server_slot:SendText("Reached maximum class limit for sniper on "..team);
--				-- locInitialPlayerProperties.sPlayerClass = "Grunt";
--				server_slot:SendCommand("CPC Grunt 1 1 1 1");
--				locInitialPlayerProperties=self:GetInitialPlayerProperties(server_slot);
--			end

		end
	end
-- *******

	local _model = server_slot:GetModel();
	local team_color;
	local rp;


	-- make sure to reset viewlayers
	server_slot:SendCommand("GI WS");		

	if(team=="spectators") then				
		rp=Server:GetRandomRespawnPoint(team);
		if(rp==nil)then
			rp = Server:GetRandomRespawnPoint();
			if (rp==nil) then
				System:Log("GameRules:SpawnPlayer spawnpoints are missing")
				rp = {x=0, y=0, z=0, xA=0, yA=0, zA=0};
			else
				System:Log("GameRules:SpawnPlayer spawnpoints for '"..team.."' are missing")
			end
		end
	else	-- not "spectators"
		rp=self:GetTeamRespawnPoint(team,System:GetEntity(server_slot:GetPlayerId()));	--NOTE GetTeamRespawnPoint has to be implemented in the gamerules for each team MOD
		if (rp==nil) then
			System:Log("GameRules:SpawnPlayer haven't found a TeamRespawnPoint")
			rp=Server:GetRandomRespawnPoint();
			if (rp==nil) then
				rp={x=0, y=0, z=0, xA=0, yA=0, zA=0};
			end
		end

		if locInitialPlayerProperties and locInitialPlayerProperties.model then
			_model=locInitialPlayerProperties.model;
		end

		if(team=="red")then
			team_color={1,0,0};
		elseif(team=="blue")then
			team_color={0,0,1};
		end
	end
	
	if(not team_color)then
	
		local sColor=tonumber(server_slot:GetColor())+1;			-- [0|1|2|3|4|5|6|7|8|9]
		
		team_color = MultiplayerUtils.ModelColor[sColor];
		
		if (not team_color) then
			team_color = {0,0,0};
		end
--		System:Log("sColor = "..sColor);
	end
	
--	System:Log("***********************************************");
--	dump(team_color);
		
    -- TODO: replace randomness by actual collision detection
	local pos = {x = rp.x, y = rp.y, z = rp.z};
	local ang = {x = rp.xA, y = rp.yA, z = rp.zA};
	
	local sEntName=server_slot:GetName();
	
	local newent=Server:SpawnEntity({
		classid=classid,
		name=sEntName,
		pos=pos,
		angles=ang,
		model=_model,
		color=team_color,
		properties=custom_properties
	})
	
	local oldid=server_slot:GetPlayerId();

	--bind the new created player to the serverslot
	server_slot:SetPlayerId(newent.id);
	
	--delete the old player entity
	if oldid~=0 then
		DeletePlayer(oldid);
	end
	
	local debug=GetSlotPlayer(server_slot);
	if debug~=newent then
		error.error=1;
	end
	
	self:OnAfterSpawnEntity(server_slot);	
	Game:ForceScoreBoard(newent.id, 0);
	
	return newent;
end


-------------------------------------------------------------------------------
-- is updating GameRules.RespawnPoints
function GameRules:RecreateTableOfRespawnPoints()
	GameRules.RespawnPoints={};
	local i=1;
	
--	System:Log("RecreateTableOfRespawnPoints");--debug

	-- spawn at the first unoccupied tagpoint
	local pt = Server:GetFirstRespawnPoint();

	while(pt)do
		local entry=new(pt);
		GameRules.RespawnPoints[i] = entry;
		i=i+1;
		
--		System:Log("RecreateTableOfRespawnPoints "..i);--debug

		pt=Server:GetNextRespawnPoint();
	end	
end


-------------------------------------------------------------------------------
function GameRules:GetFreeTeamRespawnPoint(team,entityToIgnore)
	local respawnPoint=nil;
	local isPointExisting=0;
	local dz=0; -- Used to raise the spawn posititions if all checkpoints are full

	local ents=System:GetEntities();		-- very inefficient, needed for IsIntersectingPlayerOrVehicle

--	System:Log("GetFreeTeamRespawnPoint -------------------------");--debug

	-- Repeat until we find a point
	while (respawnPoint==nil) do
		-- spawn at the first unoccupied tagpoint
		
		for key,pt in GameRules.RespawnPoints do
			if(pt.name==team)then
				isPointExisting=1;
--				pt.z = pt.z + dz;
--				System:Log("GetFreeTeamRespawnPoint ...");--debug
--				dump(pt);	--debug
				if(GameRules:IsIntersectingPlayerOrVehicle(pt.x, pt.y, pt.z+dz, ents, entityToIgnore)==0)then
--					System:Log("IsIntersectingPlayerOrVehicle =0");	--debug
					if(respawnPoint)then
						if(respawnPoint.lastUsedTime and (not pt.lastUsedTime or pt.lastUsedTime<respawnPoint.lastUsedTime))then 
							respawnPoint = pt;		-- found a better one
--							System:Log("IsIntersectingPlayerOrVehicle a");	--debug
						end
					else
						respawnPoint = pt;		-- I found the first acceptable
--						System:Log("IsIntersectingPlayerOrVehicle b");	--debug
					end
				end
			end
		end	

		if(respawnPoint)then
			respawnPoint.lastUsedTime=_time;

			respawnPoint = new(respawnPoint);		-- create a copy because we will change the point
			respawnPoint.z=respawnPoint.z+dz;

			return respawnPoint;
		end

		if(isPointExisting==0)then
			return nil;				-- there is no spawnpoint for this team
		end
	
		-- all points full!  Raise up a bit
		dz=dz+2;
		
		if(dz>20)then
			System:Error("GameRules:GetFreeTeamRespawnPoint "..tostring(team).." failed");
			return nil;				-- to prevent endless loop
		end;	
	end

	return respawnPoint;
end

-------------------------------------------------------------------------------
-- returns the number of intersecting players at a particular point
function GameRules:IsIntersectingPlayerOrVehicle(x,y,z, ents, entityToIgnore)

	for i, entity in ents do
		if (entity~=entityToIgnore and entity.type=="Player") then

			local pos = entity:GetPos();
			local _dx=pos.x-x;
			local _dy=pos.y-y;
			local _dz=pos.z-z;
			-- if the entity is closer than 1 meter then consider it intersecting
			if (((_dx * _dx) + (_dy * _dy) + (_dz * _dz)) < 1) then
				return 1;
			end;
			
		elseif (entity.type=="Vehicle") then
			local bbox=entity:GetBBox();
			local min=bbox.min; local max=bbox.max;
			if (x > min.x and x < max.x and
					y > min.y and y < max.y and
					z > min.z and z < max.z) then
						return 1;

			end
		end
	end
	
	return 0;
end

-------------------------------------------------------------------------------
--
function GameRules:OnAfterSpawnEntity( server_slot )

	local newent=GetSlotPlayer(server_slot);
		
	if newent.type=="Player" then
	
		local locInitialPlayerProperties=self:GetInitialPlayerProperties(server_slot);
		
		--set the default stats
		if locInitialPlayerProperties then
			newent.cnt.health			= locInitialPlayerProperties.health;
			newent.cnt.max_health	= locInitialPlayerProperties.health;
			newent.cnt.armor			= locInitialPlayerProperties.armor;	

				if (newent.cnt.vip == nil) then
					newent.cnt.vip = 0;
				end		
--3up extra variables start
			newent.cnt.leghealth			= locInitialPlayerProperties.leghealth;
			newent.cnt.max_leghealth	= locInitialPlayerProperties.leghealth;
			newent.cnt.basespeed			= locInitialPlayerProperties.basespeed;	
			newent.cnt.carrymux			= locInitialPlayerProperties.carrymux;	
			newent.cnt.fallscale			= locInitialPlayerProperties.fallscale;	
			newent.cnt.killcount			= locInitialPlayerProperties.killcount;	
			newent.cnt.gasmask			= locInitialPlayerProperties.gasmask;	
--3up extra variables end

		

			if locInitialPlayerProperties.StaminaTable then
				newent.cnt:InitStaminaTable( locInitialPlayerProperties.StaminaTable );
			end	
	
			if locInitialPlayerProperties.DynProp then
				newent.cnt:SetDynamicsProperties( locInitialPlayerProperties.DynProp );
			end
			
			if locInitialPlayerProperties.move_params then
				newent.cnt:SetMoveParams( locInitialPlayerProperties.move_params );
			end	
			
			newent.Properties.equipEquipment = locInitialPlayerProperties.equipEquipment;
			-- have to make a copy of the table at this point, because otherwise when the
			-- initial player properties change (class change, but before spawn) these
			-- changes will alias to Properties.equipEquipment and we won't know what stuff
			-- the player has anymore
			newent.currentEquipment = new(EquipPacks[newent.Properties.equipEquipment]);
			newent.sCurrentPlayerClass=locInitialPlayerProperties.sPlayerClass;		-- to identify class name
			
			-- your class is now <MultiplayerClassDefiniton.PlayerClasses>
			BasicPlayer.InitAllWeapons(newent, 1);	-- because equipEquipment has changed

			if locInitialPlayerProperties.sPlayerClass then
--3up adjust speed start
				GameRules:AdjustSpeed(server_slot, locInitialPlayerProperties.sPlayerClass);
--				server_slot:SendCommand("YCN "..locInitialPlayerProperties.sPlayerClass.." "..tostring(newent.cnt.basespeed));
--3up adjust speed end
				Server:BroadcastCommand("YCN "..server_slot:GetPlayerId().." "..locInitialPlayerProperties.sPlayerClass.." "..tostring(newent.cnt.basespeed));
			else
				Server:BroadcastCommand("YCN "..server_slot:GetPlayerId().." DefaultMultiPlayer");
			end


--			BasicPlayer.InitAllWeapons(newent, 1);	-- because equipEquipment has changed
		end
			
		if (gr_InvulnerabilityTimer~=nil and toNumberOrZero(gr_InvulnerabilityTimer)>0) then
--			System:Log("Player "..newent:GetName().." spawned, activated invulnerbility");
			newent.invulnerabilityTimer=toNumberOrZero(gr_InvulnerabilityTimer);
			-- turn on invulnerability shader
--			System:Log("FX: Setting invulnerability shader: "..tostring(newent.name));
			Server:BroadcastCommand("FX", g_Vectors.v000, g_Vectors.v000, newent.id, 2);
		end
-- 3up king
--		if (tonumber(getglobal("gr_tMode")) == 3) and (gr_KingTimer~=nil and toNumberOrZero(gr_KingTimer)>0) then
--			System:Log("Player "..newent:GetName().." spawned, activated invulnerbility");
--			newent.cnt.KingTimer=toNumberOrZero(gr_KingTimer);
--			newent.KingTimer=toNumberOrZero(gr_KingTimer);
--			-- turn on invulnerability shader
--			System:Log("FX: Setting invulnerability shader: "..tostring(newent.name));
--			Server:BroadcastCommand("FX", g_Vectors.v000, g_Vectors.v000, newent.id, 2);
--		end
--3up king end	
		newent.cnt.alive=1;
--3up extra variable initialize start
--		if (tonumber(getglobal("gr_tMode")) == 3) then
--			newent.cnt.vip=2;
--		else
--			newent.cnt.vip=0;
--		end
		newent.cnt.killcount=0;
		newent.cnt.gasmask=0;
--3up extra variable initialize end
		newent:EnablePhysics(1);

		-- we would like its viewdistratio to be max (== 255)
		newent:SetViewDistRatio(255);
		
-- 3up hero highlight notdone
		if self.bShowUnitHightlight then
			local highlightpos=new(newent:GetPos());
			local highlight;
			local kinghighlight;
			kinghighlight = Server:SpawnEntity("KingHighlight",highlightpos);
			highlight = Server:SpawnEntity("UnitHighlight",highlightpos);
		
			newent.idUnitHighlight = highlight.id;		-- only done on the server
			newent.idKingHighlight = kinghighlight.id;		-- only done on the server
		
			-- send "follow this player" command
			Server:BroadcastCommand("FX "..tonumber(newent.id), g_Vectors.v000, g_Vectors.v000,highlight.id,0);
			Server:BroadcastCommand("FX "..tonumber(newent.id), g_Vectors.v000, g_Vectors.v000,kinghighlight.id,0);
		end

		-- reset viewlayers
		server_slot:SendCommand("GI WS");				
		-- deactivate radar enemies
		server_slot:SendCommand("HUD RR");		
	end
end

function GameRules:GetPlayerTeamCount( team )
	local ret=0;
	local slots = Server:GetServerSlotMap();

	for i, slot in slots do
		local player_id=slot:GetPlayerId();

		if Game:GetEntityTeam(player_id)==team then
			ret=ret+1;
		end
	end

	return ret;
end

--3up class limit	start
function GameRules:GetSniperCount( team )
	local ret=0;
	local slots = Server:GetServerSlotMap();

	for i, slot in slots do
		local player_id=slot:GetPlayerId();
		local player = System:GetEntity(player_id);
		if player.sCurrentPlayerClass then
			if Game:GetEntityTeam(player_id)==team and strlower(player.sCurrentPlayerClass)=="sniper" then
				ret=ret+1;
			end
		end
	end

	return ret;
end
--3up class limit	end

-------------------------------------------------------------------------------
-- called to change the team for a client, either because the mode has
-- changed, or the player hAs requested so manually (OnClientJoinTeamRequest).
-- the change may be denied if required
function GameRules:ChangeTeam(server_slot, new_team, force)
	local player_id = server_slot:GetPlayerId();
	local requested_classid = PLAYER_CLASS_ID;
	
	if player_id ~= 0 then
		local player = System:GetEntity(player_id);
		local old_team = Game:GetEntityTeam(player_id);
					
		if force or new_team ~= old_team then
			System:Log("player '"..tostring(server_slot:GetName()).."$1' changed team from "..tostring(old_team).." to "..tostring(new_team));
    
   		if new_team=="spectators" then
				requested_classid=SPECTATOR_CLASS_ID;
				
				if (self.respawnList) then
					self.respawnList[server_slot] = nil;
				end

				local newEntity=self:SpawnPlayer(server_slot,requested_classid,new_team);
				Server:AddToTeam("spectators",newEntity.id);

				return;
			end

			if (self.respawnList and  (self:GetGameState() ~= CGS_PREWAR)) then
				System:Log("changing team! "..new_team.." team!");
				self.respawnList[server_slot] = {classid = requested_classid, team = new_team,};
			else
				local newEntity=self:SpawnPlayer(server_slot,requested_classid,new_team);
				Server:AddToTeam(new_team, newEntity.id);
				
				if (self.SetTeamObjectivesOnePlayer) then
					self:SetTeamObjectivesOnePlayer(server_slot);
				end
					
				if (self.UpdateCaptureProgressOnePlayer) then
					self:UpdateCaptureProgressOnePlayer(server_slot);
				end				
			end
		end
	end
end

function GameRules:GetTeamMemberCountRL(team)	
	local iCount = 0;	
	
	if (self.respawnList) then
		local Slots = Server:GetServerSlotMap();
		
		for i,ServerSlot in Slots do
		
			local RespawnInfo = self.respawnList[ServerSlot];
			local szPlayerTeam;
			
			if (not RespawnInfo) then
				szPlayerTeam = Game:GetEntityTeam(ServerSlot:GetPlayerId());
			else
				szPlayerTeam = RespawnInfo.team;
				
				if (not szTeam) then
					szPlayerTeam = Game:GetEntityTeam(ServerSlot:GetPlayerId());
				end
			end
			
			if (team == szPlayerTeam) then
				iCount = iCount + 1;
			end
		end
	else
		iCount = iCount + Server:GetTeamMemberCount(team);
	end

	return iCount;
end


-------------------------------------------------------------------------------
function GameRules:HandleJoinTeamRequest(server_slot,new_team,class)
	
--[GL start]
	
	--player got some kinda respawn punishment
	if GLcommands:cannotSpawn(server_slot)==1 then
		return;
	end
		
	--first, check if player is forced to 1 team
	local forcedTeam=GLplayerTrack:GetBySs(server_slot, "forcedteam");
	
	if forcedTeam ~="" then 
		if new_team~="spectators" then
			new_team=forcedTeam;
			Server:BroadcastText(getglobal("gl_ssmod_id").."$o player "..server_slot:GetName().."$o is not allowed to be in other team than "..new_team);
		end
	end
	
	--check for global teamforcing
	if forcedTeam == "" then
		if toNumberOrZero(getglobal("gl_forcebluejoin")) == 1 and GLcommands:ProtectedName(server_slot:GetName())~=1 then
			if new_team == "red" then
				new_team = "blue";
				Server:BroadcastText(getglobal("gl_ssmod_id").." player "..server_slot:GetName().."$o cannot join (red team is locked)");
			end
		end
		if toNumberOrZero(getglobal("gl_forceredjoin")) == 1 and GLcommands:ProtectedName(server_slot:GetName())~=1 then
			if new_team == "blue" then
				new_team = "red";
				Server:BroadcastText(getglobal("gl_ssmod_id").." player "..server_slot:GetName().."$o cannot join (blue team is locked)");
			end		
		end
	end
	--[end GL]


--3up : lockdown *******
	local lockdown = toNumberOrZero(getglobal("gr_Lockdown"));
	local lockdowntime = toNumberOrZero(getglobal("gr_Lockdowntime"));

--*******

	local maxplayers = toNumberOrZero(getglobal("gr_MaxTeamLimit"));
	
	if (maxplayers and maxplayers > 0 and self.bIsTeamBased) then
		if new_team~="spectators" and self:GetPlayerTeamCount(new_team)>=maxplayers then
	  	Server:BroadcastText("@ReachedMaxTeam "..new_team);
			return;
		end
--3up : lockdown *******
		if lockdown == 1 then
			if (_time-self.mapstart > lockdowntime) then
				server_slot:SendText("Server is LOCKED "..floor(_time-self.mapstart).."s is past the lock time of "..lockdowntime.."s");
				server_slot:SendText("Please wait in spectator until the next game starts");
				return;
			end
		end
-- *******

	end

	self:ChangeTeam(server_slot,new_team, nil);
	
	if(new_team=="spectators" or new_team=="players")then
	  Server:BroadcastText(server_slot:GetName().."$o @HasJoinedThe "..GameRules.TeamText[new_team]);
	else
	  Server:BroadcastText(server_slot:GetName().."$o @HasJoinedTeam "..GameRules.TeamText[new_team]);
	end
	
  server_slot:Ready(nil);
  server_slot:ResetPlayTime();
end


-------------------------------------------------------------------------------
function GameRules:ResetReadyState(state)
	local slots = Server:GetServerSlotMap();
	for i, slot in slots do
		slot:Ready(state);
	end
end


-------------------------------------------------------------------------------
--return a non nil value if the slot has a "player entity" assigned to him
function IsPlayer(slot)
    return System:GetEntity(slot:GetPlayerId()).type=="Player";
end


-------------------------------------------------------------------------------
-- callback for when the player uses the "team" console command
function GameRules:OnClientJoinTeamRequest(server_slot,team_name)
	self:Invoke("OnClientJoinTeamRequest",server_slot,team_name);
end



-------------------------------------------------------------------------------
function GameRules:GetGameState()
	local curstate=self:GetState();
	for i,val in self.states_map do
		if(val==curstate)then
			return i
		end
	end
end


-------------------------------------------------------------------------------
-- clean up when a client leaves
function GameRules:OnClientDisconnect( server_slot )
	-- this must be called before the player is deleted,
	-- because we might want to do some processing on player data	
	--[GL store last used name]
	GLplayerTrack:SetBySs(server_slot,"name",server_slot:GetName(),0);
	--[end GL]

	self:Invoke("OnClientDisconnect",server_slot);

	Server:BroadcastText(server_slot:GetName().."$o @HasLeftGame");
	--remove the player from the team
	--remove the entity
	local player_id = server_slot:GetPlayerId();
	server_slot:SetPlayerId(0); -- set an invalid player id
	DeletePlayer(player_id);
	
	self:RemovePlayerFromScoreboard(server_slot:GetId());
end

-------------------------------------------------------------------------------
-- first time connect, mod can decide how to add the new client to the
-- game (spectator or not, etc). needs to spawn new entity.
function GameRules:OnClientConnect( server_slot, requested_classid )

	local newPlayer=self:SpawnPlayer(server_slot,SPECTATOR_CLASS_ID,"spectators");
	
	--inform player about current game state
	server_slot:SetGameState(self:GetGameState(), _time-self.mapstart);
	server_slot:Ready(nil);
	server_slot:ResetPlayTime();

	Server:BroadcastText(server_slot:GetName().."$o @HasConnected");
	if (tostring(getglobal("gr_AnnoJoin1"))) then
    		server_slot:SendText(tostring(getglobal("gr_AnnoJoin1")));
	end

	if (tostring(getglobal("gr_AnnoJoin2"))) then
    		server_slot:SendText(tostring(getglobal("gr_AnnoJoin2")));
	end

	if (tostring(getglobal("gr_AnnoJoin3"))) then
    		server_slot:SendText(tostring(getglobal("gr_AnnoJoin3")));
	end


	Server:AddToTeam("spectators",newPlayer.id);
	
	local pt;
	pt=Server:GetRandomRespawnPoint("spectators")
	if(pt==nil)then
		pt = Server:GetFirstRespawnPoint();
		System:Log("ERROR: Spawn points for 'spectators' are missing, please check your map");
		if (pt==nil) then
			System:Log("ERROR: There are no spawn points");
			pt = {x=0, y=0, z=0, xA=0, yA=0, zA=0};
		end
	end
	
	newPlayer:SetPos(pt);	
	newPlayer:SetAngles({x=pt.xA, y=pt.yA, z=pt.zA});

	if self.bShowUnitHightlight then
		-- send "follow this player" command to the UnitHightlight objects
	  local slots = Server:GetServerSlotMap();
		for i, slot in slots do
	    local ent = System:GetEntity(slot:GetPlayerId());
	    
		      if ent and ent.idUnitHighlight then
				server_slot:SendCommand("FX "..tonumber(ent.id), g_Vectors.v000, g_Vectors.v000,ent.idUnitHighlight,0);
			end
			if ent and ent.idKingHighlight then
				server_slot:SendCommand("FX "..tonumber(ent.id), g_Vectors.v000, g_Vectors.v000,ent.idKingHighlight,0);
			end
	 	end
	end

	if floor(toNumberOrZero(getglobal("gr_RespawnTime")))~=0 and self.respawnCycle then
		server_slot:SendCommand("RTR "..tostring(self.respawnCycleTimer));
	end
	
	if (self.SetPlayerTimeLimit) then
		self:SetPlayerTimeLimit(server_slot);
	end
	
	-- update the teamcoloring/invulnerability shader for the player entities on the client that just connected
	local SlotMap = Server:GetServerSlotMap();
--	System:Log("Sending Team Color To "..server_slot:GetName().."...");
	
	for i, Slot in SlotMap do
		local Player = GetSlotPlayer(Slot);
		
		if (Player and Player.ApplyTeamColor) then

			if (not Player.cnt.KingTimer or (Player.cnt.KingTimer and Player.cnt.KingTimer <= 0)) then
				-- this player is not invulnerable
				System:Log("Player "..Slot:GetName().." is normal!")	;				
--				server_slot:SendCommand("FX", g_Vectors.v000, g_Vectors.v000, Slot:GetPlayerId(), 1);
			else
				-- this player is invulnerable
				System:Log("Player "..Slot:GetName().." is king!");				
--				server_slot:SendCommand("FX", g_Vectors.v000, g_Vectors.v000, Slot:GetPlayerId(), 2);
			end


			if (not Player.invulnerabilityTimer or (Player.invulnerabilityTimer and Player.invulnerabilityTimer <= 0)) then
				-- this player is not invulnerable
				System:Log("Player "..Slot:GetName().." is teamcolored!")	;				
				server_slot:SendCommand("FX", g_Vectors.v000, g_Vectors.v000, Slot:GetPlayerId(), 1);
			else
				-- this player is invulnerable
				System:Log("Player "..Slot:GetName().." is invulnerable!");				
				server_slot:SendCommand("FX", g_Vectors.v000, g_Vectors.v000, Slot:GetPlayerId(), 2);
			end
		end
	end
	
	self:AddPlayerToScoreboard(server_slot:GetId());
	--[GL playertracking]
	--player login 
	GLplayerTrack:OnConnect(server_slot);  
	--[end GL]
end


-------------------------------------------------------------------------------
function GameRules:OnUpdate()
	--statemachine update
	
	if (toNumberOrZero(getglobal("gr_TimeLimit")) > 999) then
		setglobal("gr_TimeLimit", 999);
	end
	if (toNumberOrZero(getglobal("gr_TimeLimit")) < 0) then
		setglobal("gr_TimeLimit", 0);
	end

	self:CheckPlayerCount();		-- might go to PREWAR or back when min player count is not ok

	if (self.restartbegin and (_time >= floor(self.restartend))) then
		self.restartbegin = nil;
		self.restartend = nil;
		self:DoRestart();
	end

--3up add announce
	local lockdowntime = toNumberOrZero(getglobal("gr_Lockdowntime"));
	local anno1 = toNumberOrZero(getglobal("gr_AnnounceOn1")); 
	local anno2 = toNumberOrZero(getglobal("gr_AnnounceOn2"));
	


	if ((_time-self.mapstart-lockdowntime >= 0) and (_time-self.mapstart-lockdowntime < 10) and (anno1 == 1)) then
			if (toNumberOrZero(getglobal("gr_MaxDeathLimit")) > 0) then
				Server:BroadcastText("$6Welcome to $5Annihilation$6 by $3Escape Key");				
				Server:BroadcastText("$6You only have "..toNumberOrZero(getglobal("gr_MaxDeathLimit")).. " lives");				
				Server:BroadcastText("$6The teams are $3locked$6. You can not switch.");				
				System:Log("lockdown");
			else
				Server:BroadcastText(getglobal("gr_Announce1"));
			end
			System:Log(getglobal("gr_Announce1"));
			setglobal("gr_AnnounceOn2", 1);
			setglobal("gr_AnnounceOn1", 0);
	end
	if ((_time-self.mapstart-lockdowntime >= 10) and (_time-self.mapstart-lockdowntime < 20) and (anno2 == 1)) then
			if (toNumberOrZero(getglobal("gr_MaxDeathLimit")) > 0) then
				Server:BroadcastText("$6If you lose your lives you will spectate until the end.");				
				Server:BroadcastText("$6Bonus lives are granted for flag captures and eliminations");				
				Server:BroadcastText("$6Email comments to $5threeup@gmail.com");
				System:Log("lockdown");
			else
				Server:BroadcastText(getglobal("gr_Announce2"));
			end
		System:Log(getglobal("gr_Announce2"));
		setglobal("gr_AnnounceOn1", 1);
		setglobal("gr_AnnounceOn2", 0);
	end

	anno1 = toNumberOrZero(getglobal("gr_AnnounceOn1"));
	anno2 = toNumberOrZero(getglobal("gr_AnnounceOn2"));

	if ((_time-self.mapstart-lockdowntime-60 >= 0) and (_time-self.mapstart-lockdowntime-60 < 10) and (anno1 == 1)) then
			if (toNumberOrZero(getglobal("gr_MaxDeathLimit")) > 0) then
				Server:BroadcastText("$6Welcome to $5Annihilation$6 by $3Escape Key");				
				Server:BroadcastText("$6You only have "..toNumberOrZero(getglobal("gr_MaxDeathLimit")).. " lives");				
				Server:BroadcastText("$6The teams are $3locked$6. You can not switch.");				
				System:Log("lockdown");
			else
				--Server:BroadcastText(getglobal("gr_Announce1"));
			end
			System:Log(getglobal("gr_Announce1"));
			setglobal("gr_AnnounceOn2", 1);
			setglobal("gr_AnnounceOn1", 0);
	end
	if ((_time-self.mapstart-lockdowntime-60 >= 10) and (_time-self.mapstart-lockdowntime-60 < 20) and (anno2 == 1)) then
			if (toNumberOrZero(getglobal("gr_MaxDeathLimit")) > 0) then
				Server:BroadcastText("$6If you lose your lives you will spectate until the end.");				
				Server:BroadcastText("$6Bonus lives are granted for flag captures and eliminations");				
				Server:BroadcastText("$6Email comments to $5threeup@gmail.com");
				System:Log("lockdown");
			else
				--Server:BroadcastText(getglobal("gr_Announce2"));
			end
		System:Log(getglobal("gr_Announce2"));
		setglobal("gr_AnnounceOn1", 1);
		setglobal("gr_AnnounceOn2", 0);
	end

	anno1 = toNumberOrZero(getglobal("gr_AnnounceOn1"));
	anno2 = toNumberOrZero(getglobal("gr_AnnounceOn2"));

	if ((_time-self.mapstart-lockdowntime-300 >= 0) and (_time-self.mapstart-lockdowntime-300 < 10) and (anno1 == 1)) then
			if (toNumberOrZero(getglobal("gr_MaxDeathLimit")) > 0) then
				Server:BroadcastText("$6Welcome to $5Annihilation$6 by $3Escape Key");				
				Server:BroadcastText("$6You only have "..toNumberOrZero(getglobal("gr_MaxDeathLimit")).. " lives");				
				Server:BroadcastText("$6The teams are $3locked$6. You can not switch.");				
				System:Log("lockdown");
			else
				Server:BroadcastText(getglobal("gr_Announce1"));
	end
			System:Log(getglobal("gr_Announce1"));
			setglobal("gr_AnnounceOn2", 1);
			setglobal("gr_AnnounceOn1", 0);
	end
	if ((_time-self.mapstart-lockdowntime-300 >= 10) and (_time-self.mapstart-lockdowntime-300 < 20) and (anno2 == 1)) then
			if (toNumberOrZero(getglobal("gr_MaxDeathLimit")) > 0) then
				Server:BroadcastText("$6If you lose your lives you will spectate until the end.");				
				Server:BroadcastText("$6Bonus lives are granted for flag captures and eliminations");				
				Server:BroadcastText("$6Email comments to $5threeup@gmail.com");
				System:Log("lockdown");
			else
				Server:BroadcastText(getglobal("gr_Announce2"));
			end
		System:Log(getglobal("gr_Announce2"));
		setglobal("gr_AnnounceOn1", 1);
		setglobal("gr_AnnounceOn2", 0);
	end
--3up add announce end

	self:Update();
	MapCycle:Update();
	
	if GameRules.ScoreboardUpdate	then
		GameRules:ScoreboardUpdate();
	else
		System:Log("ERROR: GameRules:ScoreboardUpdate() not implemented");
	end
end


-------------------------------------------------------------------------------
-- player requests to be killed (typed in the "kill" console command)
function GameRules:OnKill(server_slot)
	self:Invoke("OnKill",server_slot);
end


-------------------------------------------------------------------------------
-- called for every damage on every player, so should be relatively efficient
-- most of a mods specific quirks are likely to go in here
function GameRules:OnDamage( hit )

--  System:Log("Lib GameRules:OnDamage");
  
  local theTarget = hit.target;
  local theShooter = hit.shooter;
   
  if (theTarget and theTarget.type == "Player" and theTarget.invulnerabilityTimer~=nil and  theTarget.invulnerabilityTimer>0) then
--	System:Log("Player "..theTarget:GetName().." is invulnerable, no damage is applied");
  	return -- Player is invulnerable, don't accept damage
  end
      
	self:Invoke("OnDamage",hit);
end


-------------------------------------------------------------------------------
-- called from the map reloading process
-- gamerules.lua is reloaded and thus reset before this is called
function GameRules:OnMapChange()

	System:Log("GameRules:OnMapChange");

	if tostring(getglobal("gr_PrewarOn")) ~= "0" then
		self:NewGameState(CGS_PREWAR);
	else
		self:NewGameState(CGS_INPROGRESS);
	end
	
	if (MapCycle) then
		MapCycle:OnMapChanged();
	end
	
	self:RecreateTableOfRespawnPoints();		-- update GameRules.RespawnPoints

	self:CreateScorebordEntity();
end


-------------------------------------------------------------------------------
function GameRules:CreateScorebordEntity()
	-- create only if needed
	if GameRules.idScoreboard then
		return
	end

	-- create Scoreboard
	local Scoreboard = Server:SpawnEntity({ classid=SYNCHED2DTABLE_CLASS_ID, name="Scoreboard" });

	if Scoreboard then
		GameRules.idScoreboard = Scoreboard.id;
	else
		GameRules.idScoreboard = nil;
	end

--	System:Log("create Scoreboard entity "..tostring(Scoreboard.id));
end

-------------------------------------------------------------------------------
function GameRules:COUNTDOWN_OnBeginState()
	self.countnext = 0;
end


-------------------------------------------------------------------------------
function GameRules:COUNTDOWN_OnUpdate()
	if _time>self.countnext+self.mapstart then
   	    if self.countnext==floor(gr_CountDown) then
    	    self.mapstart = _time;
        	self:NewGameState(CGS_INPROGRESS);
        	
	        local slots = Server:GetServerSlotMap();
        	for i, slot in slots do
--	        	    local ent = System:GetEntity(slot:GetPlayerId());
--	        	    if ent~=nil and ent.type=="Player" then
--	        	        self:ChangeTeam(slot, Game:GetEntityTeam(ent.id), 1);
--	        	    end
        	    slot:ResetPlayTime();
        	    
        	    -- set the default class for player
            end
            
        	GameRules:OnMapChange();	-- init map entites
   	    else
   	    	Server:BroadcastText("@CountdownNo "..(floor(gr_CountDown)-self.countnext), 0.95);
   	        self.countnext = self.countnext+1;
   	    end
    end
end




-------------------------------------------------------------------------------
-- called after player has died and presses fire to respawn
-- normal procedure is for this function to recreate an entity
function GameRules:OnClientRequestRespawn( server_slot, requested_classid )

	--[GL start player cannot spawn]
--	if GLcommands:cannotSpawn(server_slot)==1 then
--		return;
--	end
	--[GL end]
	GLplayerTrack:OnRespawn(server_slot);

	--local player;
	local player= System:GetEntity(server_slot:GetPlayerId());

	
	-- If this game mode supports respawning in groups then put that player in
	-- the requested respawn list rather than respawning them immediately.
	-- They will be respawned in DoOntimerRespawnCycleTimer defined in GameRulesTeamLib.lua
	if toNumberOrZero(getglobal("gr_quickspawn"))~=1 or GameRules:IsDefender(player) or 
		toNumberOrZero(getglobal("gr_tMode"))~=1  then
		if floor(toNumberOrZero(getglobal("gr_RespawnTime")))~=0 and self.respawnCycle then
	--		System:Log("client request respawn! same team!");
			if (self.respawnList[server_slot]) then
				self.respawnList[server_slot] = { classid = requested_classid, team = self.respawnList[server_slot].team, };
			else			
				server_slot:SendText("@RespawningIn "..self.respawnCycleTimer.." @GameStartingInSeconds", min(self.respawnCycleTimer, 6));
				self.respawnList[server_slot] = { classid = requested_classid, };

				if toNumberOrZero(getglobal("gr_tMode"))==2 then
					if toNumberOrZero(getglobal("gr_currFlag"))==3 then
						if (GameRules:IsAttacker(player)) then
							server_slot:SendText("Your Mission: Defend Gold at Flag 2 and Steal Gold from Flag 3");
						else
							server_slot:SendText("Your Mission: Defend Gold at Flag 3 and Steal Gold from Flag 2");
						end
					else
						if (GameRules:IsAttacker(player)) then
							server_slot:SendText("Your Mission: Defend Gold at Flag 1 and Steal Gold from Flag 2");
						else
							server_slot:SendText("Your Mission: Defend Gold at Flag 2 and Steal Gold from Flag 1");
						end
					end
				end

				if toNumberOrZero(getglobal("gr_tMode"))==1 then
					if toNumberOrZero(getglobal("gr_currFlag"))==1 then
						if (GameRules:IsAttacker(player)) then
							server_slot:SendText("Your Mission: Attack Flag 1");
						else
							server_slot:SendText("Your Mission: Defend Flag 1");
						end
					end
					if toNumberOrZero(getglobal("gr_currFlag"))==2 then
						if (GameRules:IsAttacker(player)) then
							server_slot:SendText("Your Mission: Attack Flag 2");
						else
							server_slot:SendText("Your Mission: Defend Flag 2");
						end
					end
					if toNumberOrZero(getglobal("gr_currFlag"))==3 then
						if (GameRules:IsAttacker(player)) then
							server_slot:SendText("Your Mission: Attack Flag 3");
						else
							server_slot:SendText("Your Mission: Defend Flag 3");
						end
					end
				end
				if toNumberOrZero(getglobal("gr_tMode"))==3 then
					if toNumberOrZero(getglobal("gr_currFlag"))==1 then
						if (GameRules:IsAttacker(player)) then
							server_slot:SendText("Your Mission: Eliminate all enemy at Flag 1 defense");
						else
							server_slot:SendText("Your Mission: Eliminate all enemy at Flag 1 offense");
						end
					end
					if toNumberOrZero(getglobal("gr_currFlag"))==2 then
						if (GameRules:IsAttacker(player)) then
							server_slot:SendText("Your Mission: Eliminate all enemy at Flag 2 defense");
						else
							server_slot:SendText("Your Mission: Eliminate all enemy at Flag 2 offense");
						end
					end
					if toNumberOrZero(getglobal("gr_currFlag"))==3 then
						if (GameRules:IsAttacker(player)) then
							server_slot:SendText("Your Mission: Eliminate all enemy at Flag 3 defense");
						else
							server_slot:SendText("Your Mission: Eliminate all enemy at Flag 3 offense");
						end
					end
				end


			end	
		
			return; 
		end;
	end
	
	if(server_slot.last_death and (_time-server_slot.last_death<self.respawndelay))then
		return
	end
	
--	if(server_slot.last_death)then
--		System:Log("server_slot.last_death="..server_slot.last_death.." _time=".._time);
--	else
--		System:Log("server_slot.last_death=(nil)");
--	end
	
	self:RespawnPlayer(server_slot);
end




-------------------------------------------------------------------------------
function GameRules:RespawnPlayer( server_slot )
	-- store old stats
	local prevPlayerEntity=GetSlotPlayer(server_slot);
	
	local oldscore;
	local olddeaths;
	local oldvip;
	
	if prevPlayerEntity~=nil and prevPlayerEntity.type=="Player" then
		oldscore=prevPlayerEntity.cnt.score;
		olddeaths=prevPlayerEntity.cnt.deaths;
		oldvip=prevPlayerEntity.cnt.vip;
	end

	-- create new player
	local oldteam=Game:GetEntityTeam(server_slot:GetPlayerId());
	local requested_classid = PLAYER_CLASS_ID;
	
	local player=self:SpawnPlayer(server_slot,requested_classid,oldteam);

	-- restore old stats
	if(oldscore or olddeaths or oldvip) and player and player.cnt then
		player.cnt.score=oldscore;
		player.cnt.deaths = olddeaths;
		player.cnt.vip = oldvip;
	end

	Server:AddToTeam(oldteam,player.id);	
end


-------------------------------------------------------------------------------
function GameRules:OnSpectatorSwitchModeRequest(spect)
	local curhost=spect.cnt:GetHost();
	local newhost=0;
	local players=GetPlayers();
--	System:LogToConsole("OnSpectatorSwitchModeRequest curr host="..curhost);
	if(count(players)==0)then 
--		System:LogToConsole("OnSpectatorSwitchModeRequest no players");
		return 
	end;
	
	for id,ent in players do
--		System:LogToConsole("pp");
		if(curhost==0)then
			spect.cnt:SetHost(id);
			return
		else
			if(curhost==id)then
				curhost=0;
			end
		end
	end
	
	spect.cnt:SetHost(0);
end




-------------------------------------------------------
-------------------------------------------------------
-- return 1=if interaction is accepted, otherwise nil
function GameRules:IsInteractionPossible(actor,entity)
	-- prevent a player who is in a vehicle from interacting with the entity
	if actor then
		if (actor.theVehicle) then
			return nil;
		end

		-- prevent corpses from interacting with something
		if(actor.cnt and actor.cnt.health and actor.cnt.health < 1)then
			return nil;
		end
	end

	local iCurstate=self:GetGameState();

	if(iCurstate~=CGS_INPROGRESS)then
		return nil;
	end
	
	return 1;
end


-------------------------------------------------------
-------------------------------------------------------
-- reset the state of all entities
function GameRules:ResetMapEntities()

	local entities=System:GetEntities();
	
	if(entities)then
		for i,ent in entities do
			if(ent)then
				if (ent.deleteOnGameReset) then
					Server:RemoveEntity(ent.id);
					ent = nil;
				elseif (ent.OnMultiplayerReset) then
					ent:OnMultiplayerReset();
				end

				-- store the initial position and angles of the entity, so they can be restored by the Phoenix
				if ent ~= nil and ent.GetPos then
					if not ent.PhoenixData then
						ent.PhoenixData = {};
						ent.PhoenixData.pos = new(ent:GetPos());
						ent.PhoenixData.angles = new(ent:GetAngles());	
					else
						-- we have phoenix data, so reset these guys
						local data = ent.PhoenixData;
						ent:SetPos(data.pos);
						ent:SetAngles(data.angles);
						if (ent.OnReset) then
							ent:OnReset();
						end
					end
				end
			end
		end
	end

	-- respawn all players
	
	local slots = Server:GetServerSlotMap();
	for i, slot in slots do
		local ent=System:GetEntity(slot:GetPlayerId());

		if ent and ent.type=="Player" then
			self:RespawnPlayer(slot);							-- was already a player so respawn
		end
	end
	
	self:CreateScorebordEntity();
end


-------------------------------------------------------
-- is invoked on server from SendCommand() call on client
function GameRules:OnClientCmd( server_slot ,cmd)
--	System:Log("RCV CLIENT CMD = ["..cmd.."]");

	local toktable=tokenize(cmd);
	
	if toktable~={} then
		local sCommand=toktable[1];
		
		local Function=self.ClientCommandTable[sCommand];
		
		if Function then
			Function(cmd,server_slot,toktable);
		end
	end
end



-------------------------------------------------------------------------------
-- count player that are in ready state (e.g. typed in /ready)
function GameRules:ReadyPlayersCount()
	local count=0;
	local slots = Server:GetServerSlotMap();
	for i, slot in slots do
		if(slot:IsReady())then
			local ent=System:GetEntity(slot:GetPlayerId());
			if(ent and ent.type=="Player")then
				count=count+1;
			end
		end
	end
	return count;
end

-------------------------------------------------------------------------------
-- callback from map cleanup
function GameRules:OnShutdown()
end






-------------------------------------------------------------------------------
--------- voting system -------------------------------------------------------
-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
-- callback for the "callvote" console command. votes can be called on
-- anything that we have a "game command" for (see below).
-- only 1 vote can be in progress at any one time, and votes timeout after 1 minute
function GameRules:OnCallVote(server_slot, command, arg1)
	if(command=="ready")then
	    if IsPlayer(server_slot)==nil then
			server_slot:SendText("@VoteFailNoSpec");
			return
		end
		
		self:ReadyPlayer(server_slot);
        return
	end

	self.voting_state:OnCallVote(server_slot, command, arg1);
end


-------------------------------------------------------------------------------
-- callback for the "vote" console command, which casts an actual vote
-- if this vote causes a majority on "yes", execute the "game command"
-- that was voted on
function GameRules:OnVote(server_slot, vote)
	self.voting_state:OnVote(server_slot, vote);
end


-------------------------------------------------------------------------------
-- specific voting version for the "ready" console command
-- (needs unanimous votes, can happen during another vote)
function GameRules:ReadyPlayer(server_slot)
	if self:GetState()~="PREWAR" then
		server_slot:SendText("@ReadyOnlyInPrewar");
		return
	end

	Server:BroadcastText(server_slot:GetName().." @IsReady");

	server_slot:Ready(1);
end



-- \return team_name or nil 
function GameRules.ReturnTeamThatDoesDamage(shooter,hit)
	if hit then
		if hit.weapon and hit.weapon.LaunchedByTeam then
			return hit.weapon.LaunchedByTeam;
		else
			if shooter==nil then
				return;
			end

			return Game:GetEntityTeam(shooter.id);			
		end
	end
end


-- currrently only for explosive damage
-- \return 1 or nil 
function GameRules.ReturnIfPlayerDamagesHimself(target,shooter,hit)
	if hit and hit.weapon and hit.weapon.shooterSSID and target then
		return Server:GetServerSlotBySSId(hit.weapon.shooterSSID):GetPlayerId() == target.id;
	end
	
	if shooter and target==shooter then
		return 1
	end
end


-------------------------------------------------------------------------------
-- if this function is specified in the gamerules, the pickups fade away after a while
function GameRules:GetPickupFadeTime()
	local UserVal=toNumberOrZero(getglobal("gr_DropFadeTime"));
	
	if not UserVal then
		UserVal=10;
	end
	
	if UserVal<1 then 
		UserVal=1;
	end
	
	if UserVal>100 then 
		UserVal=100;
	end

	return UserVal;
end


-------------------------------------------------------------------------------
-- private function to get the same damage calculation in all mods
-- /return nil=player is still living or no player was hit
--         1=player was killed by enemy
--         2=player self kill
--         3=team kill
function GameRules:UsualDamageCalculation(hit)
	local target = hit.target;
	local shooter = hit.shooter;
	local damage = hit.damage;
--	System:Log("\003 UsualDamageCalculation damage: "..hit.damage);

	-- usually used for friendly fire
	if self.IgnoreDamageBetween and self:IgnoreDamageBetween(target,shooter,hit)==1 then 
--		System:Log("UsualDamageCalculation IgnoreDamageBetween damage: "..hit.damage);		--debug
		return 
	end
	
	if not shooter then																								-- explosive damage (entity id might be reassigned)
		local ss_shooter=Server:GetServerSlotBySSId(hit.shooterSSID);
    if ss_shooter then		-- might be nil e.g. vehicle destroyes itself
    	shooter = System:GetEntity(ss_shooter:GetPlayerId());
    end
  end

	--if is falling damage the shooter could be nil
	if(shooter==nil)then 
--		System:Log("GAMERULES>SHOOTER == TARGET")
		shooter=hit.target; 
	end

	if(hit.explosion ~= nil)then
		local expl=target:IsAffectedByExplosion();
		if(expl<=0)then return end
		damage = expl * damage;
	end

--	System:Log("\003 UsualDamageCalculation shooter: "..shooter.id);

	if target==nil or target.type ~= "Player" or target.cnt.health<=0 then 
--		System:Log("GAMERULES>TARGET IS NOT A PLAYER");
		return nil;
	end   -- don't care about damage in prewar etc.

	local zone = target.cnt:GetBoneHitZone( hit.ipart);
    if zone==0 then zone = 2 end;
    
--	System:Log("\003 UsualDamageCalculation zone: "..zone);

  local headshot;				-- nil/1

	if(hit.target_material and hit.target_material.type=="head") then		-- HEAD SHOT
	  	headshot=1;
     	damage=damage*tonumber(gr_HeadshotMultiplier);
	end
	
--3up legdamage start
--	if(hit.target_material and hit.target_material.type=="leg") then		-- LEG SHOT
--	  	target.cnt.leghealth= target.cnt.leghealth - damage;
--
--		if (target.cnt.leghealth < 0) then
--			target.cnt.leghealth = 0;
--		end
--		local legpercent = 0.95;
--		legpercent = target.cnt.leghealth / target.cnt.max_leghealth;
--		if (legpercent < 0.5) then
--			legpercent = 0.5;
--		end
--
--

--		local bsh = target.basespeed / 2;
--	
--	
--		target.move_params.speed_run=  (bsh * legpercent) + bsh;
--		target.move_params.speed_walk=  (bsh * 0.77 * legpercent) + (bsh * 0.77);
--		target.move_params.speed_swim=  (bsh * 0.72 * legpercent) + (bsh * 0.72);
--
--		target.move_params.speed_run_strafe= target.move_params.speed_run;
--		target.move_params.speed_walk_strafe=  target.move_params.speed_walk;
--		target.move_params.speed_swim_strafe=  target.move_params.speed_swim;
--
--
--		target.move_params.speed_run_back=  target.move_params.speed_run;
--		target.move_params.speed_walk_back=  target.move_params.speed_walk;
--		target.move_params.speed_swim_back=  target.move_params.speed_swim;
--
--
----3up
--		--Server:BroadcastText("runspeed: "..target.move_params.speed_run);
--		target.cnt:SetMoveParams( target.move_params );
--	  	--target.cnt.leghealth= target.cnt.leghealth - damage;
--		--Server:BroadcastText("L E G "..target.cnt.leghealth.." of "..target.cnt.max_leghealth);
--	end
--3up legdamage end


	damage = damage * tonumber(gr_DamageScale);
    
--	System:Log("\003 UsualDamageCalculation before health and armor: "..target.cnt.health);
	if (damage > 0) then
		--- apply damage first to armor, if it is present
		if (hit.damage_type ~= "healthonly") then
			if (target.cnt.armor > 0) then
				target.cnt.armor= target.cnt.armor - (damage*0.5);
				-- clamp to zero
				if (target.cnt.armor < 0) then
					damage = -target.cnt.armor;
					target.cnt.armor = 0;
				else
					damage = 0;
				end
			end
		end
	end
	
--3up legdamage
	local oldhealth = target.cnt.health;
	if (hit.damage_type == "healthonly" and target.cnt.max_health == 140) then
		damage = 0;
	end
--	if hit.damage_type then
--		System:Log("\003 UsualDamageCalculation "..hit.damage_type.." - "..target.cnt.gasmask);
--	end
--	System:Log("\003 xx- "..tostring(target.gasmask));
	
--	if target.classid then
--		System:Log("\003 - "..tostring(target.classid));
--	end

--	if target.classname then
--		System:Log("\003 -b "..tostring(target.classname));
--	end
--	if target.cnt.max_health then
--		System:Log("\003 h "..target.cnt.max_health);
--	end

	target.cnt.health = target.cnt.health - damage;
   	
--	System:Log("\003 UsualDamageCalculation after health and armor: "..target.cnt.health);


--3up legdamage heal start
--	if (oldhealth < target.cnt.health) then
--
--		if(target.cnt.leghealth>target.cnt.max_leghealth)then
--			target.cnt.leghealth=target.cnt.max_leghealth;
--		end
--		local legpercent = 1;
--		legpercent = target.cnt.leghealth / target.cnt.max_leghealth;
--		if (legpercent < 0.5) then
--			legpercent = 0.5;
--		end
--
--		local bsh = target.basespeed / 2;
--	
--		target.move_params.speed_run=  (bsh * legpercent) + bsh;
--		target.move_params.speed_walk=  (bsh * 0.77 * legpercent) + (bsh * 0.77);
--		target.move_params.speed_swim=  (bsh * 0.72 * legpercent) + (bsh * 0.72);
--
--		target.move_params.speed_run_strafe= target.move_params.speed_run;
--		target.move_params.speed_walk_strafe=  target.move_params.speed_walk;
--		target.move_params.speed_swim_strafe=  target.move_params.speed_swim;
--
--
--		target.move_params.speed_run_back=  target.move_params.speed_run;
--		target.move_params.speed_walk_back=  target.move_params.speed_walk;
--		target.move_params.speed_swim_back=  target.move_params.speed_swim;
--
--		Server:BroadcastText("L1E G "..target.cnt.leghealth.." of "..target.cnt.max_leghealth);
--		Server:BroadcastText("L1E G speed "..target.move_params.speed_run);
--		target:NotifyMessage("L1E G "..target.cnt.leghealth.." of "..target.cnt.max_leghealth);
--		target:NotifyMessage("L1E G speed "..target.move_params.speed_run);
---3up
--		target.cnt:SetMoveParams( target.move_params );
--	end

--3up legdamage heal end

 	-- negative damage (medic tool) is is bounded to max_health
	if(target.cnt.health>target.cnt.max_health) then
		target.cnt.health=target.cnt.max_health;
	end
--3up legdamage heal start
--	if(target.cnt.leghealth>target.cnt.max_leghealth) then
--		target.cnt.leghealth=target.cnt.max_leghealth;
--	end
--3up legdamage heal end

	if target.cnt.health<=0 then
--		System:Log("\003 UsualDamageCalculation Killed");

		local shooterTeam = self.ReturnTeamThatDoesDamage(shooter,hit);


--3up kill spree sound start
		if (headshot==1 and shooter==_localplayer) then
			Sound:PlaySound(GameRules.Headshot);
		end
		
		if(hit.shooter==_localplayer) then

			if(GameRules.kills==2) then
				Sound:PlaySound(GameRules.KillvoiceSound[1]);
			end
			if(GameRules.kills==3) then
				Sound:PlaySound(GameRules.KillvoiceSound[2]);
			end
			if(GameRules.kills==5) then
				Sound:PlaySound(GameRules.KillvoiceSound[3]);
			end
			if(GameRules.kills==7) then
				Sound:PlaySound(GameRules.KillvoiceSound[4]);
			end
			if(GameRules.kills==10) then
				Sound:PlaySound(GameRules.KillvoiceSound[5]);
			end
			if(GameRules.kills==13) then
				Sound:PlaySound(GameRules.KillvoiceSound[6]);
			end
			if(GameRules.kills==15) then
				Sound:PlaySound(GameRules.KillvoiceSound[7]);
			end
			
		end
--3up kill spree sound end


		target.cnt.health = 0;
		if shooter.id==target.id or shooter.type ~= "Player" then 
			return 2; -- shoot self or by neutral entity
		elseif Game:GetEntityTeam(target.id)==shooterTeam then
			return 3; -- player was killed by a team member
		else
			return 1; -- player was killed by enemy
		end
	end
	
	return nil;	-- player is still living
end

-------------------------------------------------------------------------------
-- this function sets the score of a player
--3up killcount 
function GameRules:SetPlayerScore(player, score, killcount, deaths, vip)
	player.cnt.score = score;
--3up killcount 
	player.cnt.killcount = killcount;
	
	if (deaths) then
		player.cnt.deaths = deaths;
	end

	if (vip) then
		player.cnt.vip = vip;
	else
		player.cnt.vip = 0;
	end


end

-------------------------------------------------------------------------------
-- this function sets the scores of every player
--3up killcount 
function GameRules:ResetPlayerScores(score, killcount, deaths, vip)

	local ServerSlotMap = Server:GetServerSlotMap();
	
	for i, Slot in ServerSlotMap do
		local Player = System:GetEntity(Slot:GetPlayerId());
		
		if (Player and (Player.type == "Player")) then
--3up killcount 
			self:SetPlayerScore(Player, score, killcount, deaths, vip);
		end
		if (tonumber(getglobal("gr_tMode")) == 2) and Player then
			Player.cnt.vip = 0;
		end
	end

	if (MPStatistics) then
		MPStatistics:Reset();
	end
	--[GL playertracking]
	GLplayerTrack:Init();
	--[end GL]
end


function GameRules:HaveKingWon(target, wasvip)
	if (tonumber(getglobal("gr_tMode")) == 3) then
			if GameRules:IsAttacker(target) and (wasvip > 0) then
				local attackvip=0;
				local attackpeople=0;
				local slots=Server:GetServerSlotMap();
				for i, slot in slots do
					local ent = System:GetEntity(slot:GetPlayerId());
					if ent then
						if (ent.type ~= "spectator") then
							attackpeople=attackpeople+1;
						end
						if ent and (GameRules:IsAttacker(ent)) and (ent.type ~= "spectator") 
							and (ent.cnt.vip > 0) then
							attackvip=attackvip+1;
						end
					end
				end
				Server:BroadcastText("There are "..attackvip.." kings on Team ROCK");

				if (toNumberOrZero(attackvip) == 0) and (attackpeople > 0) then	
					Server:BroadcastText("Team ROCK has been eliminated");
					local team_score=Game:GetTeamScore(self.defenderTeam);
					team_score=team_score+1;
					Server:SetTeamScore(self.defenderTeam,team_score);

					self:SwitchTeams();
					self.mapstart = _time;
					if (self.switchTeamsTimer and self.switchTeamsTimer > 0) then
						self.mapstart = self.mapstart + self.switchTeamsTimer;
					end		
				end
			else
				local defensevip=0;
				local defensepeople=0;
				local slots=Server:GetServerSlotMap();
				for i, slot in slots do
					local ent = System:GetEntity(slot:GetPlayerId());
					if ent then
						if (ent.type ~= "spectator") then
							defensepeople=defensepeople+1;
						end
						if ent and (not GameRules:IsAttacker(ent)) and (ent.type ~= "spectator") 
						and (ent.cnt.vip > 0) then
							defensevip=defensevip+1;
						end
					end
				end
				Server:BroadcastText("There are "..defensevip.." kings on Team SCISSORS.");

				if (toNumberOrZero(defensevip) == 0) and (defensepeople > 0) then	
					Server:BroadcastText("Team SCISSORS has been eliminated");
					local team_score=Game:GetTeamScore(self.attackerTeam);
					team_score=team_score+1;
					Server:SetTeamScore(self.attackerTeam,team_score);

					self:SwitchTeams();
					self.mapstart = _time;
					if (self.switchTeamsTimer and self.switchTeamsTimer > 0) then
						self.mapstart = self.mapstart + self.switchTeamsTimer;
					end		
				end

			end
	end
end

-------------------------------------------------------------------------------
-- private function to get the same score calculation in all mods
-- most of a mods specific quirks are likely to go in here
-- /param hit={ target=<entity>, shooter=<entity> }
-- /param damage_ret typically from UsualDamageCalculation()
function GameRules:UsualScoreCalculation( hit, damage_ret )
	
	if damage_ret==nil then																											-- player is still alive
		return
	end
	
	local target = hit.target;
	local shooter = hit.shooter;
	local situation = 0;                                                        -- normal kill
	local delta = 1;

	target:GotoState("Dead");		

	local ss_target=Server:GetServerSlotByEntityId(target.id);					-- serverslot
	local ss_shooter;																										-- serverslot

	if shooter then																											-- non explosive damage
		ss_shooter=Server:GetServerSlotByEntityId(shooter.id);
	else																																-- explosive damage (entity id might be reassigned)
		ss_shooter=Server:GetServerSlotBySSId(hit.shooterSSID);
    if ss_shooter then		-- might be nil e.g. vehicle destroyes itself
    	shooter = System:GetEntity(ss_shooter:GetPlayerId());
    end
  end
	
-- 	System:Log(">>>> damage_ret:"..damage_ret);

	
	local weapon = "World";
	
	if(hit.weapon)then
		if(hit.weapon.IsBoat)then
			weapon="Boat";
		elseif(hit.weapon.IsVehicle)then
			weapon="Vehicle";
		elseif(hit.weapon.name)then
			weapon = hit.weapon.name;
		else
			weapon = hit.weapon.classname;
		end
	end

			local iCurrState=tonumber(GameRules.CurrentProgressStep);
--			if iCurrState then
--				Server:BroadcastText("Gun "..weapon.." State "..iCurrState);
--			end
			
	
	if damage_ret==2 then			-- shoot self or by neutral entity
		situation = 1;
		MPStatistics:AddStatisticsDataEntity(target,"nSelfKill",1);
		delta = -1;
	else
		if(hit.target_material and hit.target_material.type=="head") then					-- HEAD SHOT
			if ss_shooter then
			MPStatistics:AddStatisticsDataSSId(ss_shooter:GetId(),"nHeadshot",1);	-- successfully killed by headshot
			end
		end

		if damage_ret==1 then																											-- player was killed by enemy
			MPStatistics:AddStatisticsDataSSId(ss_shooter:GetId(),"nKill",1);
			if weapon == "Falcon" then
				MPStatistics:AddStatisticsDataSSId(ss_shooter:GetId(),"nHumiliate",1);
			end
			if weapon == "Machete" then
				MPStatistics:AddStatisticsDataSSId(ss_shooter:GetId(),"nHumiliate",1);
				MPStatistics:AddStatisticsDataSSId(ss_shooter:GetId(),"nKnife",1);
			end
			if (weapon == "OICWGrenade") or (weapon == "AG36Grenade") or (weapon == "Rocket") or (weapon == "HandGrenade") then
				MPStatistics:AddStatisticsDataSSId(ss_shooter:GetId(),"nExplode",1);
			end
			if (weapon == "ProjProximityMine") or (weapon == "StickyExplosive")then
				MPStatistics:AddStatisticsDataSSId(ss_shooter:GetId(),"nTrap",1);
			end
			if (iCurrState) then
				if (iCurrState > 1) then
					MPStatistics:AddStatisticsDataSSId(ss_shooter:GetId(),"nSirenKill",1);
					Server:BroadcastText("CLUTCH kill by "..ss_shooter:GetName().." with the "..weapon);
				end
			end


		elseif damage_ret==3 then																									-- player was killed by a team member
		  	  situation = 2;
			MPStatistics:AddStatisticsDataSSId(ss_shooter:GetId(),"nTeamKill",1);
			delta = -1;
--3up kick tk start
			local tklimit = -toNumberOrZero(getglobal("gr_TKLimit"));

			if (toNumberOrZero(getglobal("gr_TKLimit")) ~= 0) and (shooter.cnt.score - 1 <= tklimit) then
				Server:BroadcastText("$o "..ss_shooter:GetName().."$0 is being kicked for TKing.");
				GameRules:KickSlot(ss_shooter);
			end
--3up kick tk end
		end
	end
	
	-- score changes?
	if ((shooter and shooter.type == "Player") or (damage_ret == 2)) then
		if (not shooter) then
			target.cnt.score=target.cnt.score+delta;
--3up killcount 
			target.cnt.killcount=target.cnt.killcount+delta;
		else
			shooter.cnt.score=shooter.cnt.score+delta;
--3up killcount 
			shooter.cnt.killcount=shooter.cnt.killcount+delta;

		end
	end
	
--3up : maxdeaths *******
	local maxdeaths = tonumber(getglobal("gr_MaxDeathLimit"));
--3up *******

	local wasvip = 0;
	if (target.cnt.vip) then
		wasvip = target.cnt.vip;
	end
	
	if (tonumber(getglobal("gr_tMode")) == 3) then
		if (target.cnt.vip > 1) then
			Server:BroadcastCommand("DRAW N", g_Vectors.v000, g_Vectors.v000, target.id, 5);

			local entityid = target.idKingHighlight;
			Server:BroadcastCommand("FXO 0", g_Vectors.v000, g_Vectors.v000, entityid, 5);


			target.cnt.vip = 0;
			if GameRules:IsAttacker(target) then
				ss_target:SendCommand("FLG NA");
			else
				ss_target:SendCommand("FLG ND");
			end
		end
		if shooter then
			if (wasvip > 1) then
				shooter.cnt.vip = shooter.cnt.vip + 1;
			end
			if (shooter.cnt.vip >= 2) then			
				local level = shooter.cnt.vip;
				local timer = tonumber(getglobal("gr_KingTimer"));
				if (level >= 3 and level <= 5) then
					timer = timer*0.9;
				end
				if (level >= 6 and level <= 8) then
					timer = timer*0.75;
				end
				if (level >= 9 and level <= 11) then
					timer = timer*0.6;
				end
				if (level >= 12 and level <= 15) then
					timer = timer*0.5;
				end
				if (level >= 16) then
					timer = timer*0.4;
				end

				shooter.cnt.KingTimer = timer;
				ss_shooter:SendCommand("UKL "..timer);
				Server:BroadcastCommand("DRAW Y", g_Vectors.v000, g_Vectors.v000, shooter.id, 5);

				local entityid = shooter.idKingHighlight;
				Server:BroadcastCommand("FXO 1", g_Vectors.v000, g_Vectors.v000, entityid, 5);

			end

			if (shooter.cnt.vip == 2) then
				ss_shooter:SendCommand("FLG V");
				Server:BroadcastCommand("DRAW Y", g_Vectors.v000, g_Vectors.v000, shooter.id, 5);
				Server:BroadcastText(shooter:GetName().." has regained their crown");
				local entityid = shooter.idKingHighlight;
				Server:BroadcastCommand("FXO 1", g_Vectors.v000, g_Vectors.v000, entityid, 5);

			end
		end
	end

	if (target.type == "Player") then
		target.cnt.deaths=target.cnt.deaths+1;
		target.cnt.vip=0;
	end


--3up end of king game
	self:HaveKingWon(target,wasvip);

--3up : maxdeaths *******
		if (target.cnt.deaths >= maxdeaths and maxdeaths > 0) then

			Server:BroadcastText(target:GetName().."$1 has been $3Annihilated$1 by "..shooter:GetName());
			shooter.cnt.deaths = shooter.cnt.deaths - 1;

			if GameRules:IsAttacker(target) then
				local newteam = "spectators";
				Server:AddToTeam(newteam,target.id);
				self:ChangeTeam(ss_target, newteam, 1);
				target.type = "spectator";

				local attackvip=0;
				--3up foobar
				local slots=Server:GetServerSlotMap();
				for i, slot in slots do
					local ent = System:GetEntity(slot:GetPlayerId());
					if ent and (GameRules:IsAttacker(ent)) and (ent.type ~= "spectator") then
						attackvip=attackvip+1;
					end
				end
				Server:BroadcastText("There are "..attackvip.." attackers remaining.");

				if (toNumberOrZero(attackvip) == 0) and (_time - self.mapstart > 60) then	
					Server:BroadcastText("Attackers have been eliminated");
					local team_score=Game:GetTeamScore(self.defenderTeam);
					team_score=team_score+1;
					Server:SetTeamScore(self.defenderTeam,team_score);

					self:SwitchTeams();
					self.mapstart = _time;
					if (self.switchTeamsTimer and self.switchTeamsTimer > 0) then
						self.mapstart = self.mapstart + self.switchTeamsTimer;
					end		
				end
			else
				local newteam = "spectators";
				Server:AddToTeam(newteam,target.id);
				self:ChangeTeam(ss_target, newteam, 1);
				target.type = "spectator";

				local defensevip=0;
				--3up foobar
				local slots=Server:GetServerSlotMap();
				for i, slot in slots do
					local ent = System:GetEntity(slot:GetPlayerId());
					if ent and (not GameRules:IsAttacker(ent)) and (ent.type ~= "spectator") then
						defensevip=defensevip+1;
					end
				end
				Server:BroadcastText("There are "..defensevip.." defenders remaining.");

				if (toNumberOrZero(defensevip) == 0) and (_time - self.mapstart > 60) then	
					Server:BroadcastText("Defenders have been eliminated");
					local team_score=Game:GetTeamScore(self.attackerTeam);
					team_score=team_score+1;
					Server:SetTeamScore(self.attackerTeam,team_score);

					self:SwitchTeams();
					self.mapstart = _time;
					if (self.switchTeamsTimer and self.switchTeamsTimer > 0) then
						self.mapstart = self.mapstart + self.switchTeamsTimer;
					end		
				end

			end



		else
			if (maxdeaths >= 1) then
				Server:BroadcastText(target:GetName().." has died "..target.cnt.deaths.." out of "..maxdeaths);
			end
		end
		if  (situation == 2) and (maxdeaths >= 1) then
			shooter.cnt.deaths=shooter.cnt.deaths+2;
			target.cnt.deaths=target.cnt.deaths-1;
			Server:BroadcastText(shooter:GetName().." has TKED and lost 2 lives ");

			--self:OnDamage({ target = shooter, shooter = shooter, damage = 1000, ipart = 0 });
		end

--3up *******


-- 	System:Log(">>>> delta:"..delta);

--3up kill material start
	local	targetMatType = "body";
	if (hit.target_material) then
		targetMatType = hit.target_material.type;
	end

	-- send hud text message to all clients
	local slots = Server:GetServerSlotMap();
	local ver = toNumberOrZero(getglobal("tacto_version"))
	for i, slot in slots do
		if ss_shooter then
--	    	System:Log(">>>> 2 PKP "..target.id.." "..ss_shooter:GetPlayerId().." "..situation);
			
			slot:SendCommand("PKP "..target.id.." "..ss_shooter:GetPlayerId().." "..situation.." "..weapon.." "..targetMatType.." "..target.cnt.killcount.." "..shooter.cnt.killcount.." "..ver);				-- usual print for others
		else
--	    	System:Log(">>>> 3 PKP "..target.id.." 1 "..situation);
			slot:SendCommand("PKP "..target.id.." 0 "..situation.." "..weapon.." "..targetMatType.." "..target.cnt.killcount.." "..target.cnt.killcount.." "..ver);				-- self kill - instead of passing a dummy value could be done better
		end			
 	end	
--3up kill material end


	if ss_target then
		ss_target.last_death=_time;
	else
		System:Log("server slot is nil");
	end

	Game:ForceScoreBoard(target.id, 1);

	return delta;
end



function GameRules:GotoNextMap()
	local nextMap = getglobal("gr_NextMap");

	Game:LoadLevelMPServer(nextMap);
end


-- get the entries for MPStatistics
function GameRules:GetInitialPlayerStatistics()
	return new(self.InitialPlayerStatistics);
end


-------------------------------------------------------------------------------
--
function GameRules:ChangeMap(mapname, gametype)

	local szGameType = gametype;

	if (not mapname or strlen(mapname) < 1) then
		return nil;
	end

	if (not gametype or strlen(gametype) < 1) then
		szGameType = getglobal("g_GameType");
	end

	System:Log("\001Map: "..mapname.." ("..szGameType..")");
	
	if (not Game:CheckMap(mapname, szGameType)) then
	
		if (Game:CheckMap(mapname)) then
			local szDefaultGameType = Game:GetMapDefaultMission(mapname);
			
			if (not szDefaultGameType) then
				System:Warning( "Map '"..mapname.."' does not support '"..szGameType.."'. No default Game Type found! Skipping this map!");
				MapCycle:OnMapChanged();
				MapCycle:OnMapFinished();
			
				return nil;
			else
				System:Warning( "Map '"..mapname.."' does not support '"..szGameType.."'. Using '"..szDefaultGameType.."' instead!");				
				szGameType = szDefaultGameType;
			end
		else
			System:Warning( "Map '"..mapname.."' not found! Skipping this map!");
			MapCycle:OnMapChanged();
			MapCycle:OnMapFinished();

			return nil;
		end
	end

	Server:BroadcastText("@ChangeMapTo "..mapname.." ("..szGameType..")", 10);

	setglobal("g_GameType", szGameType);
	setglobal("gr_NextMap", mapname);
	GameRules:NewGameState(CGS_INTERMISSION);
	GLplayerTrack:Init();
	
	return 1;
end

-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
-- below follow votable commands players can use for this mod.
-- to add a command, just add a similar function below: the name of the
-- function is what players type as first argument to "callvote", "arg"
-- is an optional second argument

GameCommands = {};

GameCommands.nextflag=
{
	-- \return nil=call is not possible, otherwise the vote has started
	OnCallVote=function(self,serverslot,arg)
		return 1;
	end,

	OnExecute=function(self)
	  Server:BroadcastText("@RestartingGame");
	  GLcommands:nextflag();
	  
	end,
	
	-- arguments
}

GameCommands.mutacto=
{
	-- \return nil=call is not possible, otherwise the vote has started
	OnCallVote=function(self,serverslot,arg)
		return 1;
	end,

	OnExecute=function(self)
	  Server:BroadcastText("@RestartingGame");
	  GLcommands:classtoggle(1);
	  GameRules:GotoNextMap();
	end,
	
	-- arguments
}

GameCommands.hutacto=
{
	-- \return nil=call is not possible, otherwise the vote has started
	OnCallVote=function(self,serverslot,arg)
		return 1;
	end,

	OnExecute=function(self)
	  Server:BroadcastText("@RestartingGame");
	  GLcommands:classtoggle(2);
	  GameRules:GotoNextMap();
	end,
	
	-- arguments
}

GameCommands.assault=
{
	-- \return nil=call is not possible, otherwise the vote has started
	OnCallVote=function(self,serverslot,arg)
		return 1;
	end,

	OnExecute=function(self)
	  Server:BroadcastText("@RestartingGame");
	  GLcommands:gomode(1);

	end,
	
	-- arguments
}

GameCommands.plunder=
{
	-- \return nil=call is not possible, otherwise the vote has started
	OnCallVote=function(self,serverslot,arg)
		return 1;
	end,

	OnExecute=function(self)
	  Server:BroadcastText("@RestartingGame");
	  GLcommands:gomode(2);

	end,
	
	-- arguments
}


GameCommands.king=
{
	-- \return nil=call is not possible, otherwise the vote has started
	OnCallVote=function(self,serverslot,arg)
		return 1;
	end,

	OnExecute=function(self)
	  Server:BroadcastText("@RestartingGame");
	  GLcommands:gomode(3);
	  
	end,
	
	-- arguments
}

GameCommands.anni=
{
	-- \return nil=call is not possible, otherwise the vote has started
	OnCallVote=function(self,serverslot,arg)
		self.maxdeath=arg;
		return 1;
	end,

	OnExecute=function(self)
		if(self.maxdeath)then
			GLcommands:anni(self.maxdeath)
		end
	end,
	maxdeath=0,
}


GameCommands.map=
{
	-- \return nil=call is not possible, otherwise the vote has started
	OnCallVote=function(self,serverslot,arg)
		self.nextmap=arg;
		return 1;
	end,

	OnExecute=function(self)
		GameRules:ChangeMap(self.nextmap);
	end,

	-- arguments
	nextmap=nil,
}




GameCommands.kick=
{
	-- \return nil=call is not possible, otherwise the vote has started
	OnCallVote=function(self,serverslot,arg)

		local KickSlot=Server:GetServerSlotBySSId(arg);						-- id was specified
		
		if(not KickSlot)then
			KickSlot=MultiplayerUtils:GetServerslotFromName(arg);		-- name was specified
		end
		
		if(not KickSlot)then
			System:Log("GameCommands.kick: Slot of '"..arg.."' not found");
			serverslot:SendText("Empty Slot",3);
			return;
		end
		
		if(_localplayer and KickSlot==Server:GetServerSlotByEntityId(_localplayer.id))then
			System:Log("GameCommands.kick: don't kick the server");
			serverslot:SendText("@VoteUnknCommand",3);
			return;
		end
		
		self.PlayerSSID=KickSlot:GetId();
		return 1;
	end,

	OnExecute=function(self)
--	  Server:BroadcastText("@KickingPlayer "..arg);
 		GameRules:KickID(self.PlayerSSID);
	end,

	-- arguments
	PlayerSSID=nil,
}



GameCommands.restart=
{
	-- \return nil=call is not possible, otherwise the vote has started
	OnCallVote=function(self,serverslot,arg)
		return 1;
	end,

	OnExecute=function(self)
	  Server:BroadcastText("@RestartingGame");
	  GameRules:Restart(5);
	end,
	
	-- arguments
}



-------------------------------------------------------------------------------
-- Kicks a player
function GameRules:Kick(player)

	if (not player or strlen(player) < 1) then
		return;
	end

	local KickSlot = MultiplayerUtils:GetServerslotFromName(player);

--	System:Log("GameRules:Kick "..player.." "..tostring(KickSlot));

	GameRules:KickSlot(KickSlot);	
end

-------------------------------------------------------------------------------
--
function GameRules:KickID(id)
	if (not id or strlen(id) < 1) then
		return;
	end

	local KickSlot = Server:GetServerSlotBySSId(id);
	
	GameRules:KickSlot(KickSlot);
end

-------------------------------------------------------------------------------
--
function GameRules:KickSlot(ServerSlot)
	if (_localplayer and ServerSlot) then
		if (_localplayer.id == ServerSlot:GetPlayerId()) then
			System:Log("don't kick the server");
			return;
		end
	end

	-- found nothing?
	if(not ServerSlot)then
		System:Log("failed to kick the player");
		return;
	end

--		printf("kicked player id %d...", KickIDServerSlot:GetPlayerId());
	System:Log("kicked player "..tostring(ServerSlot:GetName()));
	Server:BroadcastText("@KickingPlayer "..tostring(ServerSlot:GetName()));
	ServerSlot:Disconnect("@Kicked");
end

-------------------------------------------------------------------------------
--
function GameRules:Ban(player)
	if (not player or strlen(player) < 1) then
		return;
	end

	local BanSlot = MultiplayerUtils:GetServerslotFromName(player);

	GameRules:BanSlot(BanSlot);
end

-------------------------------------------------------------------------------
--
function GameRules:BanID(id)
	if (not id or strlen(id) < 1) then
		return;
	end

	local BanSlot = Server:GetServerSlotBySSId(id);
	
	GameRules:BanSlot(BanSlot);
end

-------------------------------------------------------------------------------
--
function GameRules:BanSlot(ServerSlot)
	if (_localplayer and ServerSlot) then
		if (_localplayer.id == ServerSlot:GetPlayerId()) then
			System:Log("don't ban the server");
			return;
		end
	end

	if (ServerSlot) then
		ServerSlot:BanByID();
		GameRules:KickSlot(ServerSlot);
	end
end

-------------------------------------------------------------------------------
--
function GameRules:Unban(id)
	if (id) then
		Server:Unban(tonumber(id));
	end
end
-------------------------------------------------------------------------------
-- Does initilization for respawn cycles
function GameRules:StartGameRulesLibTimer()
	if floor(toNumberOrZero(getglobal("gr_RespawnTime")))~=0 and self.respawnCycle then
		self.respawnList={};	
		self.respawnCycleTimer = 1; -- Add this counter to the table.  1 second so we spawn all dead players from the last map immediately.
	end
	
	self:SetTimer(1000); -- Set the on timer event for respawns
end

-------------------------------------------------------------------------------
function GameRules:DoGameRulesLibTimer()
	local bPerformedRespawn = 0;
	
	local bDoCycle;
	
	local fRespawnTime=floor(toNumberOrZero(getglobal("gr_RespawnTime")));
	
	if fRespawnTime~=0 and self.respawnCycle then
		bDoCycle=1;
	end

	if self.respawnCycleTimer and not bDoCycle then				-- gr_RespawnTime was set to 0
		Server:BroadcastCommand("RTR");
		self.respawnCycleTimer=nil;
		self.respawnList=nil;
	end

	if self.respawnCycleTimer and self.respawnCycleTimer > fRespawnTime then					-- gr_RespawnTime was lowered
		self.respawnCycleTimer=fRespawnTime;
		Server:BroadcastCommand("RTR "..self.respawnCycleTimer);
	end

	-- Does respawn cycles
	if bDoCycle then
		if self.respawnCycleTimer then
			self.respawnCycleTimer=self.respawnCycleTimer-1;
		end
			
		if (not self.respawnCycleTimer or self.respawnCycleTimer<=0) then
			bPerformedRespawn = 1;
			if self.respawnList then
				for server_slot,respawn_info in self.respawnList do
					
					local prevPlayerEntity=GetSlotPlayer(server_slot);
					
					local requested_classid = respawn_info.classid;
					local newteam = respawn_info.team;
					
					-- if no newteam specified, the new team is the old team
					if (not newteam) then
						newteam=Game:GetEntityTeam(server_slot:GetPlayerId());
					end
					
					local oldscore;
					local olddeaths;
					local oldvip;
					
					if prevPlayerEntity~=nil and prevPlayerEntity.type=="Player" then
						oldscore=prevPlayerEntity.cnt.score;
						olddeaths=prevPlayerEntity.cnt.deaths;
						oldvip=prevPlayerEntity.cnt.vip;
					end
					
					local player;
					
					System:Log("spawning player in "..newteam.." team!");
					local player=self:SpawnPlayer(server_slot,requested_classid,newteam);

					-- copy old stats
					if(oldscore or olddeaths or oldvip)then
						player.cnt.score=oldscore;
						player.cnt.deaths=olddeaths;
						player.cnt.vip=oldvip;
					end

					Server:AddToTeam(newteam,player.id);

					if (self.SetTeamObjectivesOnePlayer) then
						self:SetTeamObjectivesOnePlayer(server_slot);
					end
					
					if (self.UpdateCaptureProgressOnePlayer) then
						self:UpdateCaptureProgressOnePlayer(server_slot);
					end
				end
			end
			
			self.respawnList={};
			self.respawnCycleTimer = fRespawnTime;
			
--			printf("SENT RTR! " .. GameRules.respawnCycleTimer);
			Server:BroadcastCommand("RTR "..tostring(GameRules.respawnCycleTimer));
			
		else
			--for server_slot,requested_classid in self.respawnList do
			--	server_slot:SendText("@RespawningIn "..self.respawnCycleTimer, 1)
			--end
		end
	end

	local slots = Server:GetServerSlotMap();
	
	for i, slot in slots do
		local player = GetSlotPlayer(slot);
		if (player and player.invulnerabilityTimer~=nil and player.ApplyTeamColor~=nil) then
			player.invulnerabilityTimer = player.invulnerabilityTimer - 1;
			if (player.invulnerabilityTimer<=0) then
				--BasicPlayer.SecondShader_TeamColoring(self);
				-- turn off invulnerability shader
--				System:Log("Player "..player:GetName().." lost invulnerbility (timer)");
				Server:BroadcastCommand("FX", g_Vectors.v000, g_Vectors.v000, player.id, 1);
				player.invulnerabilityTimer=nil;
			else
				Server:BroadcastCommand("FX", g_Vectors.v000, g_Vectors.v000, player.id, 2);
				--slot:SendText("@InvEndsIn "..tostring(player.invulnerabilityTimer), 1);
			end
		end

--3up king

		if (tonumber(getglobal("gr_tMode")) == 3) and (player and player.cnt.KingTimer ~=nil and player.cnt.vip > 1) then
			player.cnt.KingTimer = player.cnt.KingTimer - 1;
			if (player.cnt.KingTimer==0) then
				--BasicPlayer.SecondShader_TeamColoring(self);
				-- turn off invulnerability shader
				System:Log("Player "..player:GetName().." lost his crown (timer)");
--				Server:BroadcastCommand("FX", g_Vectors.v000, g_Vectors.v000, player.id, 1);
				slot:SendCommand("UKL "..player.cnt.KingTimer);
				player.cnt.KingTimer= -5;
				Server:BroadcastCommand("DRAW N", g_Vectors.v000, g_Vectors.v000, player.id, 5);

				local entityid = player.idKingHighlight;
				Server:BroadcastCommand("FXO 0", g_Vectors.v000, g_Vectors.v000, entityid, 5);


				if GameRules:IsAttacker(player) then
					slot:SendCommand("FLG NA");
				else
					slot:SendCommand("FLG ND");
				end
				player.cnt.vip=0;

				self:HaveKingWon(player,1);
			else
				local a=player.cnt.KingTimer;
				local b= tonumber(getglobal("gr_KingTimerUpd"));
				local c=a - b*floor(a/b);
				if (c == 0) then
					slot:SendCommand("UKL "..player.cnt.KingTimer);
				end

--				Server:BroadcastCommand("FX", g_Vectors.v000, g_Vectors.v000, player.id, 2);
				--slot:SendText("@InvEndsIn "..tostring(player.cnt.KingTimer), 1);
			end
		end

--3up king end

	end
	
	-- process all entities - only done after a respawn wave :)
	if ((bPerformedRespawn == 1) or (fRespawnTime == 0)) then
		local ents=System:GetEntities();
		for i, entity in ents do
			if (entity.type == "Phoenix") then
				entity:RaiseFromAshes();
			end
		end
	end
	
	-- Keep the timer going	
	self:SetTimer(1000);	
end

--------------------------------------------------------
-- check the players counts (min players and max players
------------------------------------------------------
function GameRules:CheckPlayerCount()
	local iMinTeamLimit = toNumberOrZero(getglobal("gr_MinTeamLimit"));
	
	
	if (not self.bIsTeamBased) then
		if (tostring(getglobal("gr_PrewarOn")) ~= "0") then
			iMinTeamLimit = 2;
		else
			iMinTeamLimit = 0;
		end
	end
		
	local iCurrentState = self:GetGameState();
		
	if ((iMinTeamLimit < 1) and (iCurrentGameState == CGS_INPROGRESS)) then
		return;
	end

	local iTeam01Count = GameRules:GetTeamMemberCountRL("red");
	local iTeam02Count = GameRules:GetTeamMemberCountRL("blue");

	if(not self.bIsTeamBased)then
		iTeam01Count = GameRules:GetTeamMemberCountRL("players");
		iTeam02Count = iTeam01Count;
	end
	
	if (iTeam01Count < iMinTeamLimit or iTeam02Count < iMinTeamLimit) then
				
		if (iCurrentState == CGS_INPROGRESS) then
			self:NewGameState(CGS_PREWAR);
			if (self.bIsTeamBased) then
				Server:BroadcastText("@PlayerCountNotOk "..iMinTeamLimit.." @PlayerCountNotOk2", 2);
			else
				Server:BroadcastText("@PlayerCountNotOkNoTeam "..iMinTeamLimit.." @PlayerCountNotOk2NoTeam", 2);
			end
		end
	elseif ((iCurrentState == CGS_PREWAR) and ((not self.restartbegin) or self.restartbegin == 0)) then
		self:Restart(3, 1);
		Server:BroadcastText("@GameStartingIn "..(3).." @GameStartingInSeconds", 2);
	end
end



--------------------------------------------------------
-- update Synched2DTable scoreboard
------------------------------------------------------
function GameRules:RemovePlayerFromScoreboard( idClient )
--	System:Log("GameRules:RemovePlayerFromScoreboard");						--debug

	if (GameRules.idScoreboard == nil) then
		return;
	end
	local Entity = System:GetEntity(GameRules.idScoreboard).cnt;
	
	local iY;
	local iLines=Entity:GetLineCount();
	
	for iY=0,iLines-1 do
		local idThisClient = Entity:GetEntryXY(ScoreboardTableColumns.ClientID,iY)-1;		-- first element is the clientid-1

		-- clear line
		if idThisClient==idClient then
--			System:Log("GameRules:RemovePlayerFromScoreboard idThisClient==idClient");						--debug
			Entity:SetEntriesY(iY,0);
			return;
		end
	end
	
--	System:Log("Error: GameRules:RemovePlayerFromScoreboard not found");
end


--------------------------------------------------------
-- update Synched2DTable scoreboard
------------------------------------------------------
function GameRules:AddPlayerToScoreboard( idClient )
	local Entity = System:GetEntity(GameRules.idScoreboard).cnt;
	
	if not Entity then
		System:LogError("GameRules:AddPlayerToScoreboard internal error");			-- better than script error but shouln't happen
		return;
	end
	
	local iY;
	local iLines=Entity:GetLineCount();
	
	for iY=0,iLines-1 do
		local idThisClient = Entity:GetEntryXY(ScoreboardTableColumns.ClientID,iY)-1;		-- first element is the clientid-1

		if idThisClient==-1 then
			-- set player (should be already cleared)
			Entity:SetEntryXY(ScoreboardTableColumns.ClientID,iY,idClient+1);
			System:Log("GameRules:AddPlayerToScoreboard line "..iY);		--debug
			return;
		end
	end

	Entity:SetEntryXY(ScoreboardTableColumns.ClientID,iLines,idClient+1);
	System:Log("GameRules:AddPlayerToScoreboard line "..iLines);		--debug
end


--------------------------------------------------------
-- update Synched2DTable scoreboard
------------------------------------------------------
function GameRules:SetScoreboardEntryXY( iX, idClient, Value )
--	System:Log("GameRules:SetScoreboardEntryXY "..iX.." "..idClient.." "..tostring(Value));		--debug
	local Entity = System:GetEntity(GameRules.idScoreboard).cnt;
	
	if iX==0 then
		return;					-- first element is the clientid-1
	end
	
	local iY;
	local iLines=Entity:GetLineCount();
	
	for iY=0,iLines-1 do
		local idThisClient = Entity:GetEntryXY(ScoreboardTableColumns.ClientID,iY)-1;		-- first element is the clientid-1

		if idThisClient==idClient then
--			System:Log("GameRules:SetScoreboardEntryXY line "..iY);		--debug
			Entity:SetEntryXY(iX,iY,Value);
			return;
		end
	end
end


